@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -27,12 +27,10 @@ CommandMap cmdMap[] =
   { "attention",     CMD_ATTENTION },
   { "author",        CMD_AUTHOR },
   { "b",             CMD_BOLD },
-  { "bug",           CMD_BUG },
   { "c",             CMD_CODE },
   { "code",          CMD_STARTCODE },
   { "copydoc",       CMD_COPYDOC },
   { "date",          CMD_DATE },
-  { "deprecated",    CMD_DEPRECATED },
   { "dontinclude",   CMD_DONTINCLUDE },
   { "dotfile",       CMD_DOTFILE },
   { "e",             CMD_EMPHASIS },
@@ -43,6 +41,7 @@ CommandMap cmdMap[] =
   { "endlink",       CMD_ENDLINK },
   { "endsecreflist", CMD_ENDSECREFLIST },
   { "endverbatim",   CMD_ENDVERBATIM },
+  { "endxmlonly",    CMD_ENDXMLONLY },
   { "exception",     CMD_EXCEPTION },
   { "form",          CMD_FORMULA },
   { "htmlinclude",   CMD_HTMLINCLUDE },
@@ -74,13 +73,16 @@ CommandMap cmdMap[] =
   { "sa",            CMD_SA },
   { "secreflist",    CMD_SECREFLIST },
   { "section",       CMD_SECTION },
+  { "subsection",    CMD_SUBSECTION },
+  { "subsubsection", CMD_SUBSUBSECTION },
+  { "paragraph",     CMD_PARAGRAPH },
   { "see",           CMD_SA },
   { "since",         CMD_SINCE },
   { "skip",          CMD_SKIP },
   { "skipline",      CMD_SKIPLINE },
-  { "test",          CMD_TEST },
+  { "xmlonly",       CMD_XMLONLY },
+  { "xrefitem",      CMD_XREFITEM },
   { "throw",         CMD_EXCEPTION },
-  { "todo",          CMD_TODO },
   { "until",         CMD_UNTIL },
   { "verbatim",      CMD_VERBATIM },
   { "verbinclude",   CMD_VERBINCLUDE },
@@ -99,6 +101,11 @@ CommandMap cmdMap[] =
   { "%",             CMD_PERCENT },
   { "~",             CMD_LANGSWITCH },
   { "_internalref",  CMD_INTERNALREF },
+  { "dot",           CMD_DOT },
+  { "enddot",        CMD_ENDDOT },
+  { "manonly",       CMD_MANONLY },
+  { "endmanonly",    CMD_ENDMANONLY },
+  { "includelineno", CMD_INCWITHLINES },
   { 0,               0 }
 };
 
@@ -175,6 +182,12 @@ CommandMap htmlTagMap[] =
   { "h1",         HTML_H1 },
   { "h2",         HTML_H2 },
   { "h3",         HTML_H3 },
+  { "h4",         HTML_H4 },
+  { "h5",         HTML_H5 },
+  { "h6",         HTML_H6 },
+  { "span",       HTML_SPAN },
+  { "div",        HTML_DIV },
+  { 0,            0 }
 };
 
 //----------------------------------------------------------------------------
