@@ -2,7 +2,7 @@
  *
  * $Id: entry.h,v 1.36 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -21,13 +21,15 @@
 #include "qtbc.h"
 #include <qlist.h>
 
-enum Protection { Public, Protected, Private } ;
+struct SectionInfo;
+
+enum Protection { Public, Protected, Private, Package } ;
 enum Specifier { Normal, Virtual, Pure } ;
-enum MethodTypes { Method, Signal, Slot, DCOP, Property };
+enum MethodTypes { Method, Signal, Slot, DCOP, Property, Event };
 
 struct ListItemInfo
 {
-  const char *type;
+  QCString type;
   int itemId;
 };
 
@@ -36,7 +38,8 @@ struct ListItemInfo
 struct BaseInfo 
 {
   /*! Creates an object representing an inheritance relation */
-  BaseInfo(const char *n,Protection p,Specifier v) : name(n),prot(p),virt(v) {}
+  BaseInfo(const char *n,Protection p,Specifier v) : 
+    name(n),prot(p),virt(v) {}
   QCString   name; //!< the name of the base class
   Protection prot; //!< inheritance type
   Specifier  virt; //!< virtualness
@@ -103,7 +106,7 @@ class ArgumentList : public QList<Argument>
     ArgumentList() : QList<Argument>(), 
                      constSpecifier(FALSE),
                      volatileSpecifier(FALSE),
-                     pureSpecifier(FALSE) {}
+                     pureSpecifier(FALSE) { setAutoDelete(TRUE); }
     /*! Destroys the argument list */
    ~ArgumentList() {}
     bool hasDocumentation() const;
@@ -132,15 +135,13 @@ struct Grouping
   enum GroupPri_t 
   {
     GROUPING_LOWEST,
-    GROUPING_AUTO_WEAK =
-      GROUPING_LOWEST,   //!< membership in group was defined via @weakgroup @{ @}
-    GROUPING_AUTO_ADD,    //!< membership in group was defined via @add[to]group @{ @}
-    GROUPING_AUTO_DEF,    //!< membership in group was defined via @defgroup @{ @}
-    GROUPING_AUTO_HIGHEST =
-      GROUPING_AUTO_DEF,
-    GROUPING_INGROUP,      //!< membership in group was defined by @ingroup
-    GROUPING_HIGHEST =
-      GROUPING_INGROUP
+    GROUPING_AUTO_WEAK = 
+      GROUPING_LOWEST,     //!< membership in group was defined via \@weakgroup 
+    GROUPING_AUTO_ADD,     //!< membership in group was defined via \@add[to]group 
+    GROUPING_AUTO_DEF,     //!< membership in group was defined via \@defgroup
+    GROUPING_AUTO_HIGHEST = GROUPING_AUTO_DEF,
+    GROUPING_INGROUP,      //!< membership in group was defined by \@ingroup
+    GROUPING_HIGHEST = GROUPING_INGROUP
   };
 
   static const char *getGroupPriName( GroupPri_t priority )
@@ -184,51 +185,62 @@ class Entry
       EXCEPTION_SEC    = 0x00000008,
       NAMESPACE_SEC    = 0x00000010,
       INTERFACE_SEC    = 0x00000020,
+      PROTOCOL_SEC     = 0x00000040,
+      CATEGORY_SEC     = 0x00000080,
       COMPOUND_MASK    = CLASS_SEC | STRUCT_SEC | UNION_SEC | 
-                         INTERFACE_SEC | EXCEPTION_SEC,
+                         INTERFACE_SEC | EXCEPTION_SEC | 
+                         PROTOCOL_SEC | CATEGORY_SEC,
       SCOPE_MASK       = COMPOUND_MASK | NAMESPACE_SEC,
       
-      CLASSDOC_SEC     = 0x00000100, 
-      STRUCTDOC_SEC    = 0x00000200,
-      UNIONDOC_SEC     = 0x00000400,
-      EXCEPTIONDOC_SEC = 0x00000800,
-      NAMESPACEDOC_SEC = 0x00001000,
-      INTERFACEDOC_SEC = 0x00002000,
+      CLASSDOC_SEC     = 0x00000800, 
+      STRUCTDOC_SEC    = 0x00001000,
+      UNIONDOC_SEC     = 0x00002000,
+      EXCEPTIONDOC_SEC = 0x00004000,
+      NAMESPACEDOC_SEC = 0x00008000,
+      INTERFACEDOC_SEC = 0x00010000,
+      PROTOCOLDOC_SEC  = 0x00020000,
+      CATEGORYDOC_SEC  = 0x00040000,
       COMPOUNDDOC_MASK = CLASSDOC_SEC | STRUCTDOC_SEC | UNIONDOC_SEC | 
-                         INTERFACEDOC_SEC | EXCEPTIONDOC_SEC,
+                         INTERFACEDOC_SEC | EXCEPTIONDOC_SEC | PROTOCOLDOC_SEC |
+                         CATEGORYDOC_SEC,
 
-      SOURCE_SEC       = 0x00010000,
-      HEADER_SEC       = 0x00020000,
+      SOURCE_SEC       = 0x00400000,
+      HEADER_SEC       = 0x00800000,
       FILE_MASK        = SOURCE_SEC | HEADER_SEC,
 
-      ENUMDOC_SEC      = 0x00100000,
-      ENUM_SEC         = 0x00200000,
-      EMPTY_SEC        = 0x00300000, 
-      PAGEDOC_SEC      = 0x00400000, 
-      VARIABLE_SEC     = 0x00500000,
-      FUNCTION_SEC     = 0x00600000,
-      TYPEDEF_SEC      = 0x00700000,
-      MEMBERDOC_SEC    = 0x00800000, 
-      OVERLOADDOC_SEC  = 0x00900000,
-      EXAMPLE_SEC      = 0x00a00000, 
-      VARIABLEDOC_SEC  = 0x00b00000,
-      FILEDOC_SEC      = 0x00c00000,
-      DEFINEDOC_SEC    = 0x00d00000,
-      INCLUDE_SEC      = 0x00e00000,
-      DEFINE_SEC       = 0x00f00000,
-      GROUPDOC_SEC     = 0x01000000,
-      USINGDIR_SEC     = 0x01100000,
-      MAINPAGEDOC_SEC  = 0x01200000,
-      MEMBERGRP_SEC    = 0x01300000,
-      USINGDECL_SEC    = 0x01400000,
-      PACKAGE_SEC      = 0x01500000,
-      PACKAGEDOC_SEC   = 0x01600000
+      ENUMDOC_SEC      = 0x01000000,
+      ENUM_SEC         = 0x02000000,
+      EMPTY_SEC        = 0x03000000, 
+      PAGEDOC_SEC      = 0x04000000, 
+      VARIABLE_SEC     = 0x05000000,
+      FUNCTION_SEC     = 0x06000000,
+      TYPEDEF_SEC      = 0x07000000,
+      MEMBERDOC_SEC    = 0x08000000, 
+      OVERLOADDOC_SEC  = 0x09000000,
+      EXAMPLE_SEC      = 0x0a000000, 
+      VARIABLEDOC_SEC  = 0x0b000000,
+      FILEDOC_SEC      = 0x0c000000,
+      DEFINEDOC_SEC    = 0x0d000000,
+      INCLUDE_SEC      = 0x0e000000,
+      DEFINE_SEC       = 0x0f000000,
+      GROUPDOC_SEC     = 0x10000000,
+      USINGDIR_SEC     = 0x11000000,
+      MAINPAGEDOC_SEC  = 0x12000000,
+      MEMBERGRP_SEC    = 0x13000000,
+      USINGDECL_SEC    = 0x14000000,
+      PACKAGE_SEC      = 0x15000000,
+      PACKAGEDOC_SEC   = 0x16000000,
+      OBJCIMPL_SEC     = 0x17000000
     };
     enum MemberSpecifier
     {
-      Inline    = 0x1,
-      Explicit  = 0x2,
-      Mutable   = 0x4
+      Inline    = 0x01,
+      Explicit  = 0x02,
+      Mutable   = 0x04,
+      Settable  = 0x08,
+      Gettable  = 0x10,
+      Readable  = 0x20,
+      Writable  = 0x40
     };
 
     Entry();
@@ -253,6 +265,7 @@ class Entry
     int  memSpec;             //!< member specifiers
     int  initLines;           //!< define/variable initializer lines to show 
     bool subGrouping;         //!< automatically group class members?
+    bool callGraph;           //!< do we need to draw the call graph?
     Specifier    virt;        //!< virtualness of the entry 
     Entry       *parent;      //!< parent node in the tree
     QCString	 type;        //!< member type 
@@ -263,15 +276,19 @@ class Entry
     QList<ArgumentList> *tArgLists; //!< template argument declarations
     QCString	 program;     //!< the program text
     QCString     initializer; //!< initial value (for variables)
-    QCString     includeFile; //!< include file (2 arg of \class, must be unique)
-    QCString     includeName; //!< include name (3 arg of \class)
+    QCString     includeFile; //!< include file (2 arg of \\class, must be unique)
+    QCString     includeName; //!< include name (3 arg of \\class)
     QCString     doc;         //!< documentation block (partly parsed)
     int          docLine;     //!< line number at which the documentation was found
     QCString     docFile;     //!< file in which the documentation was found
     QCString     brief;       //!< brief description (doc block)
     int          briefLine;   //!< line number at which the brief desc. was found
     QCString     briefFile;   //!< file in which the brief desc. was found
+    QCString     inbodyDocs;  //!< documentation inside the body of a function
+    int          inbodyLine;  //!< line number at which the body doc was found
+    QCString     inbodyFile;  //!< file in which the body doc was found
     QCString     relates;     //!< related class (doc block)
+    bool         relatesDup;  //!< keep duplicate doc in original file also
     QCString     inside;      //!< name of the class in which documents are found
     QCString     exception;   //!< throw specification
     int          bodyLine;    //!< line number of the definition in the source
@@ -280,17 +297,18 @@ class Entry
     QList<Entry>    *sublist; //!< entries that are children of this one
     QList<BaseInfo> *extends; //!< list of base classes    
     QList<Grouping> *groups;  //!< list of groups this entry belongs to
-    QList<QCString> *anchors; //!< list of anchors defined in this entry
+    QList<SectionInfo> *anchors; //!< list of anchors defined in this entry
     QCString	fileName;     //!< file this entry was extracted from
     int		startLine;    //!< start line of entry in the source
     QList<ListItemInfo> *sli; //!< special lists (test/todo/bug/deprecated/..) this entry is in
     TagInfo    *tagInfo;      //!< tag file info
     static int  num;          //!< counts the total number of entries
+    bool        objc;         //!< Objective-C construct
     enum 
     {
-      GROUPDOC_NORMAL,        //<! defgroup
-      GROUPDOC_ADD,           //<! addgroup
-      GROUPDOC_WEAK           //<! weakgroup
+      GROUPDOC_NORMAL,        //!< defgroup
+      GROUPDOC_ADD,           //!< addgroup
+      GROUPDOC_WEAK           //!< weakgroup
     } groupDocType;           //!< kind of group
     /// return the command name used to define GROUPDOC_SEC
     const char *groupDocCmd() const
