@@ -1,11 +1,10 @@
 /******************************************************************************
  * ftvhelp.cpp,v 1.0 2000/09/06 16:09:00
  *
- * Kenney Wong <kwong@ea.com>
+ * Contributed by Kenney Wong <kwong@ea.com>
+ * Modified by Dimitri van Heesch (c) 2003
  *
  * Folder Tree View for offline help on browsers that do not support HTML Help.
- * Uses the FTV structure from: 
- * http://www.geocities.com/Paris/LeftBank/2178/ftexample.html
  */
 
 #include <stdio.h>
@@ -20,10 +19,7 @@
 #include "doxygen.h"
 #include "language.h"
 
-
-const char treeview_data[]=
-#include "treeview.h"
-;
+#define MAX_INDENT 1024
 
 unsigned char ftv2blank_png[] = {
   0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
@@ -42,7 +38,6 @@ unsigned char ftv2blank_png[] = {
   0x32, 0xcb, 0x72, 0x8f, 0x7c, 0x12, 0x00, 0x00, 0x00, 0x00, 0x49, 0x45,
   0x4e, 0x44, 0xae, 0x42, 0x60, 0x82
 };
-unsigned int ftv2blank_png_len = 174;
 
 unsigned char ftv2doc_png[] = {
   0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
@@ -68,7 +63,6 @@ unsigned char ftv2doc_png[] = {
   0x83, 0x47, 0xbd, 0x00, 0x00, 0x00, 0x00, 0x49, 0x45, 0x4e, 0x44, 0xae,
   0x42, 0x60, 0x82
 };
-unsigned int ftv2doc_png_len = 255;
 
 unsigned char ftv2folderclosed_png[] = {
   0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
@@ -94,7 +88,6 @@ unsigned char ftv2folderclosed_png[] = {
   0x13, 0x15, 0x98, 0x60, 0xbd, 0x81, 0x0b, 0x00, 0x00, 0x00, 0x00, 0x49,
   0x45, 0x4e, 0x44, 0xae, 0x42, 0x60, 0x82
 };
-unsigned int ftv2folderclosed_png_len = 259;
 
 unsigned char ftv2folderopen_png[] = {
   0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
@@ -120,7 +113,6 @@ unsigned char ftv2folderopen_png[] = {
   0x0d, 0xa4, 0x29, 0x14, 0xcb, 0xda, 0x47, 0xac, 0x44, 0x00, 0x00, 0x00,
   0x00, 0x49, 0x45, 0x4e, 0x44, 0xae, 0x42, 0x60, 0x82
 };
-unsigned int ftv2folderopen_png_len = 261;
 
 unsigned char ftv2lastnode_png[] = {
   0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
@@ -144,7 +136,6 @@ unsigned char ftv2lastnode_png[] = {
   0x88, 0x10, 0xca, 0x33, 0x3d, 0x00, 0x00, 0x00, 0x00, 0x49, 0x45, 0x4e,
   0x44, 0xae, 0x42, 0x60, 0x82
 };
-unsigned int ftv2lastnode_png_len = 233;
 
 unsigned char ftv2link_png[] = {
   0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
@@ -178,7 +169,6 @@ unsigned char ftv2link_png[] = {
   0xcf, 0x07, 0x71, 0x95, 0x2b, 0xa1, 0x10, 0x78, 0xd0, 0xff, 0x00, 0x00,
   0x00, 0x00, 0x49, 0x45, 0x4e, 0x44, 0xae, 0x42, 0x60, 0x82
 };
-unsigned int ftv2link_png_len = 358;
 
 unsigned char ftv2mlastnode_png[] = {
   0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
@@ -196,7 +186,6 @@ unsigned char ftv2mlastnode_png[] = {
   0xc0, 0xdc, 0x69, 0xc8, 0x00, 0x00, 0x00, 0x00, 0x49, 0x45, 0x4e, 0x44,
   0xae, 0x42, 0x60, 0x82
 };
-unsigned int ftv2mlastnode_png_len = 160;
 
 unsigned char ftv2mnode_png[] = {
   0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
@@ -217,7 +206,6 @@ unsigned char ftv2mnode_png[] = {
   0x96, 0x03, 0x00, 0x00, 0x00, 0x00, 0x49, 0x45, 0x4e, 0x44, 0xae, 0x42,
   0x60, 0x82
 };
-unsigned int ftv2mnode_png_len = 194;
 
 unsigned char ftv2node_png[] = {
   0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
@@ -241,7 +229,6 @@ unsigned char ftv2node_png[] = {
   0x52, 0x00, 0xe2, 0xfa, 0x45, 0x3a, 0xe1, 0x00, 0x00, 0x00, 0x00, 0x49,
   0x45, 0x4e, 0x44, 0xae, 0x42, 0x60, 0x82
 };
-unsigned int ftv2node_png_len = 235;
 
 unsigned char ftv2plastnode_png[] = {
   0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
@@ -259,7 +246,6 @@ unsigned char ftv2plastnode_png[] = {
   0x00, 0x6e, 0xc1, 0x02, 0xc2, 0xe5, 0xed, 0x75, 0xa7, 0x00, 0x00, 0x00,
   0x00, 0x49, 0x45, 0x4e, 0x44, 0xae, 0x42, 0x60, 0x82
 };
-unsigned int ftv2plastnode_png_len = 165;
 
 unsigned char ftv2pnode_png[] = {
   0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
@@ -280,7 +266,6 @@ unsigned char ftv2pnode_png[] = {
   0xba, 0x6b, 0x07, 0x2f, 0xaa, 0xcb, 0x1f, 0x6f, 0x00, 0x00, 0x00, 0x00,
   0x49, 0x45, 0x4e, 0x44, 0xae, 0x42, 0x60, 0x82
 };
-unsigned int ftv2pnode_png_len = 200;
 
 unsigned char ftv2vertline_png[] = {
   0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
@@ -304,50 +289,297 @@ unsigned char ftv2vertline_png[] = {
   0x3a, 0x00, 0x00, 0x00, 0x00, 0x49, 0x45, 0x4e, 0x44, 0xae, 0x42, 0x60,
   0x82
 };
-unsigned int ftv2vertline_png_len = 229;
 
 
-struct ImageInfo
+FTVImageInfo image_info[] =
 {
-  const char *name;
-  unsigned char *data;
-  unsigned int len;
-} image_info[] =
-{
-  { "ftv2blank.png",ftv2blank_png,ftv2blank_png_len },
-  { "ftv2doc.png",ftv2doc_png,ftv2doc_png_len },
-  { "ftv2folderclosed.png",ftv2folderclosed_png,ftv2folderclosed_png_len },
-  { "ftv2folderopen.png",ftv2folderopen_png,ftv2folderopen_png_len },
-  { "ftv2lastnode.png",ftv2lastnode_png,ftv2lastnode_png_len },
-  { "ftv2link.png",ftv2link_png,ftv2link_png_len },
-  { "ftv2mlastnode.png",ftv2mlastnode_png,ftv2mlastnode_png_len },
-  { "ftv2mnode.png",ftv2mnode_png,ftv2mnode_png_len },
-  { "ftv2node.png",ftv2node_png,ftv2node_png_len },
-  { "ftv2plastnode.png",ftv2plastnode_png,ftv2plastnode_png_len },
-  { "ftv2pnode.png",ftv2pnode_png,ftv2pnode_png_len },
-  { "ftv2vertline.png",ftv2vertline_png,ftv2vertline_png_len },
-  { 0,0,0 }
+  { "&nbsp;", "ftv2blank.png",ftv2blank_png,174,16,22 },
+  { "*",  "ftv2doc.png",ftv2doc_png,255,24,22 },
+  { "+",  "ftv2folderclosed.png",ftv2folderclosed_png,259,24,22 },
+  { "-",  "ftv2folderopen.png",ftv2folderopen_png,261,24,22 },
+  { "\\", "ftv2lastnode.png",ftv2lastnode_png,233,16,22 },
+  { "-",  "ftv2link.png",ftv2link_png,358,24,22 },
+  { "\\", "ftv2mlastnode.png",ftv2mlastnode_png,160,16,22 },
+  { "o",  "ftv2mnode.png",ftv2mnode_png,194,16,22 },
+  { "o",  "ftv2node.png",ftv2node_png,235,16,22 },
+  { "\\", "ftv2plastnode.png",ftv2plastnode_png,165,16,22 },
+  { "o",  "ftv2pnode.png",ftv2pnode_png,200,16,22 },
+  { "|",  "ftv2vertline.png",ftv2vertline_png,229,16,22 },
+  { 0,0,0,0,0,0 }
 };
 
-static void generateFolderTreeViewData()
+struct FTVNode
 {
-  // Generate tree view script
-  QCString fileName=Config_getString("HTML_OUTPUT")+"/treeview.js";
-  QFile f(fileName);
-  if (!f.open(IO_WriteOnly))
-  {
-    err("Cannot open file %s for writing!\n",fileName.data());
-    return;
-  }
-  else
-  {
-    QTextStream t(&f);
-    t << treeview_data;
-    f.close();
-  }
+  FTVNode(bool dir,const char *r,const char *f,const char *a,const char *n)
+    : isLast(TRUE), isDir(dir),ref(r),file(f),anchor(a),name(n), 
+      parent(0) { children.setAutoDelete(TRUE); }
+  bool isLast;
+  bool isDir;
+  QCString ref;
+  QCString file;
+  QCString anchor;
+  QCString name;
+  QList<FTVNode> children;
+  FTVNode *parent;
+};
 
+
+//----------------------------------------------------------------------------
+
+FTVHelp *FTVHelp::m_theInstance = 0;
+
+/*! Construm_cts an ftv help object. 
+ *  The object has to be \link initialize() initialized\endlink before it can 
+ *  be used.
+ */
+FTVHelp::FTVHelp() 
+{
+  /* initial depth */
+  //m_dc = 0;
+  //m_cf = 0;
+  m_indentNodes = new QList<FTVNode>[MAX_INDENT];
+  m_indentNodes[0].setAutoDelete(TRUE);
+  m_indent=0;
+}
+
+FTVHelp::~FTVHelp()
+{
+  delete[] m_indentNodes;
+}
+
+/*! return a reference to the one and only instance of this class. 
+ */
+FTVHelp *FTVHelp::getInstance()
+{
+  if (m_theInstance==0) m_theInstance = new FTVHelp;
+  return m_theInstance;
+}
+
+/*! This will create a folder tree view table of contents file (tree.js).
+ *  \sa finalize()
+ */
+void FTVHelp::initialize()
+{
+}
+
+/*! Finalizes the FTV help. This will finish and close the
+ *  contents file (index.js).
+ *  \sa initialize()
+ */
+void FTVHelp::finalize()
+{
+  generateTreeView();
+}
+
+/*! Increase the level of the contents hierarchy. 
+ *  This will start a new sublist in contents file.
+ *  \sa decContentsDepth()
+ */
+int FTVHelp::incContentsDepth()
+{
+  //int i; for (i=0;i<m_dc+1;i++) m_cts << "  ";
+  m_indent++;
+  ASSERT(m_indent<MAX_INDENT);
+  return m_indent;
+}
+
+/*! Decrease the level of the contents hierarchy.
+ *  This will end the current sublist.
+ *  \sa incContentsDepth()
+ */
+int FTVHelp::decContentsDepth()
+{
+  //int i; for (i=0;i<m_dc;i++) m_cts << "  ";
+
+  ASSERT(m_indent>0);
+  if (m_indent>0)
+  {
+    m_indent--;
+    QList<FTVNode> *nl = &m_indentNodes[m_indent];
+    FTVNode *parent = nl->getLast();
+    QList<FTVNode> *children = &m_indentNodes[m_indent+1];
+    while (!children->isEmpty())
+    {
+      parent->children.append(children->take(0));
+    }
+  }
+  return m_indent;
+}
+
+/*! Add a list item to the contents file.
+ *  \param isDir TRUE if the item is a directory, FALSE if it is a text
+ *  \param ref  the URL of to the item.
+ *  \param file the file containing the definition of the item
+ *  \param anchor the anchor within the file.
+ *  \param name the name of the item.
+ */
+void FTVHelp::addContentsItem(bool isDir,
+                              const char *ref,
+                              const char *file,
+                              const char *anchor, 
+                              const char *name
+                             )
+{
+  QList<FTVNode> *nl = &m_indentNodes[m_indent];
+  FTVNode *newNode = new FTVNode(isDir,ref,file,anchor,name);
+  if (!nl->isEmpty())
+  {
+    nl->getLast()->isLast=FALSE;
+  }
+  nl->append(newNode);
+  if (m_indent>0)
+  {
+    QList<FTVNode> *pnl = &m_indentNodes[m_indent-1];
+    newNode->parent = pnl->getLast();
+  }
+  
+}
+
+static int folderId=1;
+
+void FTVHelp::generateIndent(QTextStream &t, FTVNode *n,int level)
+{
+  if (n->parent)
+  {
+    generateIndent(t,n->parent,level+1);
+  }
+  // from the root up to node n do...
+  if (level==0) // item before a dir or document
+  {
+    if (n->isLast)
+    {
+      if (n->isDir)
+      {
+        t << "<img " << FTV_IMGATTRIBS(plastnode) << "onclick=\"toggleFolder('folder" << folderId << "', this)\"/>";
+      }
+      else
+      {
+        t << "<img " << FTV_IMGATTRIBS(lastnode) << "/>";
+      }
+    }
+    else
+    {
+      if (n->isDir)
+      {
+        t << "<img " << FTV_IMGATTRIBS(pnode) << "onclick=\"toggleFolder('folder" << folderId << "', this)\"/>";
+      }
+      else
+      {
+        t << "<img " << FTV_IMGATTRIBS(node) << "/>";
+      }
+    }
+  }
+  else // item at another level
+  {
+    if (n->isLast)
+    {
+      t << "<img " << FTV_IMGATTRIBS(blank) << "/>";
+    }
+    else
+    {
+      t << "<img " << FTV_IMGATTRIBS(vertline) << "/>";
+    }
+  }
+}
+
+void FTVHelp::generateLink(QTextStream &t,FTVNode *n)
+{
+  QCString *dest;
+  //printf("FTVHelp::generateLink(ref=%s,file=%s,anchor=%s\n",
+  //    n->ref.data(),n->file.data(),n->anchor.data());
+  if (n->file.isEmpty()) // no link
+  {
+    t << "<b>" << n->name << "</b>";
+  }
+  else // link into other frame
+  {
+    if (!n->ref.isEmpty()) // link to entity imported via tag file
+    {
+      t << "<a class=\"elRef\" ";
+      t << "doxygen=\"" << n->ref << ":";
+      if ((dest=Doxygen::tagDestinationDict[n->ref])) t << *dest << "/";
+      t << "\" ";
+    }
+    else // local link
+    {
+      t << "<a class=\"el\" ";
+    }
+    t << "href=\"";
+    if (!n->ref.isEmpty())
+    {
+      if ((dest=Doxygen::tagDestinationDict[n->ref])) t << *dest << "/";
+    }
+    t << n->file << Doxygen::htmlFileExtension;
+    if (!n->anchor.isEmpty()) t << "#" << n->anchor;
+    t << "\" target=\"basefrm\">";
+    t << n->name;
+    t << "</a>";
+    if (!n->ref.isEmpty())
+    {
+      t << "&nbsp;[external]";
+    }
+  }
+}
+
+void FTVHelp::generateTree(QTextStream &t, const QList<FTVNode> &nl,int level)
+{
+  QCString spaces;
+  spaces.fill(' ',level*2+8);
+  QListIterator<FTVNode> nli(nl);
+  FTVNode *n;
+  for (nli.toFirst();(n=nli.current());++nli)
+  {
+    t << spaces << "<p>";
+    generateIndent(t,n,0);
+    if (n->isDir)
+    {
+      t << "<img " << FTV_IMGATTRIBS(folderclosed) << "onclick=\"toggleFolder('folder" << folderId << "', this)\"/>";
+      generateLink(t,n);
+      t << "</p>\n";
+      t << spaces << "<div id=\"folder" << folderId << "\">\n";
+      folderId++;
+      generateTree(t,n->children,level+1);
+      t << spaces << "</div>\n";
+    }
+    else
+    {
+      t << "<img " << FTV_IMGATTRIBS(doc) << "/>";
+      generateLink(t,n);
+      t << "</p>\n";
+    }
+  }
+}
+
+void FTVHelp::generateTreeViewImages()
+{
+  static bool done=FALSE;
+  if (done) return;
+  done=TRUE;
+
+  // Generate tree view images
+  FTVImageInfo *p = image_info;
+  while (p->name)
+  {
+    QCString fileName=Config_getString("HTML_OUTPUT")+"/"+p->name;
+    QFile f(fileName);
+    if (f.open(IO_WriteOnly)) 
+    {
+      f.writeBlock((char *)p->data,p->len);
+    }
+    else
+    {
+      fprintf(stderr,"Warning: Cannot open file %s for writing\n",fileName.data());
+    }
+    f.close();
+    p++;
+  }
+} 
+
+void FTVHelp::generateTreeView()
+{
   // Generate alternative index.html as a frame
-  fileName=Config_getString("HTML_OUTPUT")+"/index"+Doxygen::htmlFileExtension;
+  QCString fileName=Config_getString("HTML_OUTPUT")+"/index"+Doxygen::htmlFileExtension;
+  QFile f;
   f.setName(fileName);
   if (!f.open(IO_WriteOnly))
   {
@@ -360,6 +592,7 @@ static void generateFolderTreeViewData()
 #if QT_VERSION >= 200
     t.setEncoding(QTextStream::Latin1);
 #endif
+    t << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n";
     t << "<html><head>";
     t << "<meta http-equiv=\"Content-Type\" content=\"text/html;charset="
       << theTranslator->idLanguageCharset() << "\">\n";
@@ -380,7 +613,7 @@ static void generateFolderTreeViewData()
     t << "</html>" << endl;
     f.close();
   }
-  
+
   // Generate tree view frame
   fileName=Config_getString("HTML_OUTPUT")+"/tree"+Doxygen::htmlFileExtension;
   f.setName(fileName);
@@ -392,8 +625,16 @@ static void generateFolderTreeViewData()
   else
   {
     QTextStream t(&f);
-    t << "<html><head>" << endl;
-    t << "<link rel=\"stylesheet\" href=\"";
+#if QT_VERSION >= 200
+    t.setEncoding(QTextStream::Latin1);
+#endif
+    t << "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n";
+    t << "  <head>\n";
+    t << "    <meta http-equiv=\"Content-Type\" content=\"text/xhtml;charset=\""
+      << theTranslator->idLanguageCharset() << "\" />\n";
+    t << "    <meta http-equiv=\"Content-Style-Type\" content=\"text/css\" />\n";
+    t << "    <meta http-equiv=\"Content-Language\" content=\"en\" />\n";
+    t << "    <link rel=\"stylesheet\" href=\"";
     QCString cssname=Config_getString("HTML_STYLESHEET");
     if (cssname.isEmpty())
     {
@@ -409,178 +650,131 @@ static void generateFolderTreeViewData()
       t << cssfi.fileName();
     }
     t << "\">" << endl;
-    t << "<script src=\"treeview.js\"></script>" << endl;
-    t << "<script src=\"tree.js\"></script>" << endl;
-    t << "<script>" << endl;
-    t << "initializeDocument()" << endl;
-    t << "</script>" << endl;
-    t << "</head>" << endl;
-    t << "<body bgcolor=\"#ffffff\">" << endl;
-    t << "</body>" << endl;
-    t << "</html>" << endl;
-    f.close();
-  }
-
-  // Generate tree view images
-  ImageInfo *p = image_info;
-  while (p->name)
-  {
-    QCString fileName=Config_getString("HTML_OUTPUT")+"/"+p->name;
-    QFile f(fileName);
-    if (f.open(IO_WriteOnly)) 
-      f.writeBlock((char *)p->data,p->len);
-    else
+    t << "    <title>TreeView</title>\n";
+    t << "    <style type=\"text/css\">\n";
+    t << "    <!--\n";
+    t << "    .directory { font-size: 10pt; font-weight: bold; }\n";
+    t << "    .directory h3 { margin: 0px; margin-top: 1em; font-size: 11pt; }\n";
+    t << "    .directory p { margin: 0px; white-space: nowrap; }\n";
+    t << "    .directory div { display: none; margin: 0px; }\n";
+    t << "    .directory img { vertical-align: middle; }\n";
+    t << "    -->\n";
+    t << "    </style>\n";
+    t << "    <script type=\"text/javascript\">\n";
+    t << "    <!-- // Hide script from old browsers\n";
+    t << "    \n";
+    t << "    function findChildNode(node, name) \n";
+    t << "    {\n";
+    t << "      var temp;\n";
+    t << "      if (node == null) \n";
+    t << "      {\n";
+    t << "        return null;\n";
+    t << "      } \n";
+    t << "      node = node.firstChild;\n";
+    t << "      while (node != null) \n";
+    t << "      {\n";
+    t << "        if (node.nodeName == name) \n";
+    t << "        {\n";
+    t << "          return node;\n";
+    t << "        }\n";
+    t << "        temp = findChildNode(node, name);\n";
+    t << "        if (temp != null) \n";
+    t << "        {\n";
+    t << "          return temp;\n";
+    t << "        }\n";
+    t << "        node = node.nextSibling;\n";
+    t << "      }\n";
+    t << "      return null;\n";
+    t << "    }\n";
+    t << "\n";
+    t << "    function toggleFolder(id, imageNode) \n";
+    t << "    {\n";
+    t << "      var folder = document.getElementById(id);\n";
+    t << "      var l = 0;\n";
+    t << "      var vl = \"" FTV_ICON_FILE(vertline) "\";\n";
+    t << "      if (imageNode != null && imageNode.nodeName != \"IMG\") \n";
+    t << "      {\n";
+    t << "        imageNode = findChildNode(imageNode, \"IMG\");\n";
+    t << "        if (imageNode!=null) l = imageNode.src.length;\n";
+    t << "      }\n";
+    t << "      if (folder == null) \n";
+    t << "      {\n";
+    t << "      } \n";
+    t << "      else if (folder.style.display == \"block\") \n";
+    t << "      {\n";
+    t << "        while (imageNode != null && \n";
+    t << "               imageNode.src.substring(l-vl.length,l) == vl)\n";
+    t << "        {\n";
+    t << "          imageNode = imageNode.nextSibling;\n";
+    t << "          l = imageNode.src.length;\n";
+    t << "        }\n";
+    t << "        if (imageNode != null) \n";
+    t << "        {\n";
+    t << "          l = imageNode.src.length;\n";
+    t << "          imageNode.nextSibling.src = \"" FTV_ICON_FILE(folderclosed) "\";\n";
+    t << "          if (imageNode.src.substring(l-13,l) == \"" FTV_ICON_FILE(mnode) "\")\n";
+    t << "          {\n";
+    t << "            imageNode.src = \"" FTV_ICON_FILE(pnode) "\";\n";
+    t << "          }\n";
+    t << "          else if (imageNode.src.substring(l-17,l) == \"" FTV_ICON_FILE(mlastnode) "\")\n";
+    t << "          {\n";
+    t << "            imageNode.src = \"" FTV_ICON_FILE(plastnode) "\";\n";
+    t << "          }\n";
+    t << "        }\n";
+    t << "        folder.style.display = \"none\";\n";
+    t << "      } \n";
+    t << "      else \n";
+    t << "      {\n";
+    t << "        while (imageNode != null && \n";
+    t << "               imageNode.src.substring(l-vl.length,l) == vl)\n";
+    t << "        {\n";
+    t << "          imageNode = imageNode.nextSibling;\n";
+    t << "          l = imageNode.src.length;\n";
+    t << "        }\n";
+    t << "        if (imageNode != null) \n";
+    t << "        {\n";
+    t << "          l = imageNode.src.length;\n";
+    t << "          imageNode.nextSibling.src = \"" FTV_ICON_FILE(folderopen) "\";\n";
+    t << "          if (imageNode.src.substring(l-13,l) == \"" FTV_ICON_FILE(pnode) "\")\n";
+    t << "          {\n";
+    t << "            imageNode.src = \"" FTV_ICON_FILE(mnode) "\";\n";
+    t << "          }\n";
+    t << "          else if (imageNode.src.substring(l-17,l) == \"" FTV_ICON_FILE(plastnode) "\")\n";
+    t << "          {\n";
+    t << "            imageNode.src = \"" FTV_ICON_FILE(mlastnode) "\";\n";
+    t << "          }\n";
+    t << "        }\n";
+    t << "        folder.style.display = \"block\";\n";
+    t << "      }\n";
+    t << "    }\n";
+    t << "\n";
+    t << "    // End script hiding -->        \n";
+    t << "    </script>\n";
+    t << "  </head>\n";
+    t << "\n";
+    t << "  <body>\n";
+    t << "    <div class=\"directory\">\n";
+    t << "      <h3>";
+    QCString &projName = Config_getString("PROJECT_NAME");
+    if (projName.isEmpty())
     {
-      fprintf(stderr,"Warning: Cannot open file %s for writing\n",fileName.data());
-    }
-    f.close();
-    p++;
-  }
-}
-
-//----------------------------------------------------------------------------
-
-FTVHelp *FTVHelp::m_theInstance = 0;
-
-/*! Construm_cts an ftv help object. 
- *  The object has to be \link initialize() initialized\endlink before it can 
- *  be used.
- */
-FTVHelp::FTVHelp()
-{
-  /* initial depth */
-  m_dc = 0;
-  m_cf = 0;
-}
-
-/*! return a reference to the one and only instance of this class. 
- */
-FTVHelp *FTVHelp::getInstance()
-{
-  if (m_theInstance==0) m_theInstance = new FTVHelp;
-  return m_theInstance;
-}
-
-/*! This will create a folder tree view table of contents file (tree.js).
- *  \sa finalize()
- */
-void FTVHelp::initialize()
-{
-  /* open the contents file */
-  QCString fName = Config_getString("HTML_OUTPUT") + "/tree.js";
-  m_cf = new QFile(fName);
-  if (!m_cf->open(IO_WriteOnly))
-  {
-    err("Could not open file %s for writing\n",fName.data());
-    exit(1);
-  }
-  /* Write the header of the contents file */
-  m_cts.setDevice(m_cf);
-#if QT_VERSION >= 200
-  m_cts.setEncoding(QTextStream::Latin1);
-#endif
-
-  m_cts << "foldersTree = gFld(\"<b>";
-  if (Config_getString("PROJECT_NAME").isEmpty())
-  {
-    m_cts << "Root";
-  }
-  else
-  {
-    m_cts << Config_getString("PROJECT_NAME");
-  }
-  m_cts << "</b>\", \"\", \"\")\n";
-}
-
-/*! Finalizes the FTV help. This will finish and close the
- *  contents file (index.js).
- *  \sa initialize()
- */
-void FTVHelp::finalize()
-{
-  m_cts.unsetDevice();
-  m_cf->close();
-  delete m_cf;
-  generateFolderTreeViewData();
-}
-
-/*! Increase the level of the contents hierarchy. 
- *  This will start a new sublist in contents file.
- *  \sa decContentsDepth()
- */
-int FTVHelp::incContentsDepth()
-{
-  //int i; for (i=0;i<m_dc+1;i++) m_cts << "  ";
-  return ++m_dc;
-}
-
-/*! Decrease the level of the contents hierarchy.
- *  This will end the current sublist.
- *  \sa incContentsDepth()
- */
-int FTVHelp::decContentsDepth()
-{
-  //int i; for (i=0;i<m_dc;i++) m_cts << "  ";
-  return --m_dc;
-}
-
-/*! Add a list item to the contents file.
- *  \param isDir TRUE if the item is a directory, FALSE if it is a text
- *  \param ref  the URL of to the item.
- *  \param file the file containing the definition of the item
- *  \param anchor the anchor within the file.
- *  \param name the name of the item.
- */
-void FTVHelp::addContentsItem(bool isDir,
-                              const char *ref,
-                              const char *file,
-                              const char *anchor, 
-                              const char *name
-                             )
-{
-  int i; for (i=0;i<m_dc;i++) m_cts << "  ";
-  QCString parent;
-  QCString tagName = ref;
-  QCString tagDir;
-  if (ref)
-  {
-    tagName += ":";
-    QCString *s = Doxygen::tagDestinationDict[ref];
-    if (s)
-    {
-      tagDir = *s + "/";
-      tagName += tagDir;
-    }
-  }
-  if (m_dc==0) parent="foldersTree"; else parent.sprintf("aux%d",m_dc); 
-  if (isDir) // directory entry
-  {
-    m_cts << "aux" << m_dc+1 << " = insFld(" << parent << ", gFld(\"" 
-          << name << "\", \"" << tagName << "\", ";
-    if (file)      // file optional param
-    {
-      m_cts << "\"" << tagDir << file << Doxygen::htmlFileExtension << "\"))";
+      t << "Root";
     }
     else
     {
-      m_cts << "\"\"))";
+      t << projName;
     }
+    t << "</h3>\n";
+    t << "      <div style=\"display: block;\">\n";
+
+    generateTree(t,m_indentNodes[0],0);
+
+    t << "      </div>\n";
+    t << "    </div>\n";
+    t << "  </body>\n";
+    t << "</html>\n";
   }
-  else // text entry
-  {
-    m_cts << "     insDoc(" << parent << ", gLnk(\"" 
-          << name << "\", \"" << tagName << "\", ";
-    if (file)      // ref optional param
-    {
-      m_cts << "\"" << tagDir << file << Doxygen::htmlFileExtension;
-      if (anchor) m_cts << "#" << anchor;  
-      m_cts << "\"))";
-    }
-    else
-    {
-      m_cts << "\"\"))";
-    }
-  }
-  m_cts << "\n";
+  generateTreeViewImages();
+
 }
 
