@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -16,6 +16,7 @@
  *
  */
 
+#include <qdir.h>
 #include "htmldocvisitor.h"
 #include "docparser.h"
 #include "language.h"
@@ -24,9 +25,26 @@
 #include "code.h"
 #include "dot.h"
 #include "message.h"
+#include "config.h"
+
+static QString htmlAttribsToString(const HtmlAttribList &attribs)
+{
+  QString result;
+  HtmlAttribListIterator li(attribs);
+  HtmlAttrib *att;
+  for (li.toFirst();(att=li.current());++li)
+  {
+    result+=" ";
+    result+=att->name;
+    if (!att->value.isEmpty()) result+="=\""+att->value+"\"";
+  }
+  return result;
+}
+
+//-------------------------------------------------------------------------
 
 HtmlDocVisitor::HtmlDocVisitor(QTextStream &t,BaseCodeDocInterface &ci) 
-  : m_t(t), m_ci(ci), m_insidePre(FALSE), m_hide(FALSE) 
+  : DocVisitor(DocVisitor_Html), m_t(t), m_ci(ci), m_insidePre(FALSE), m_hide(FALSE) 
 {
 }
 
@@ -43,7 +61,7 @@ void HtmlDocVisitor::visit(DocWord *w)
 void HtmlDocVisitor::visit(DocLinkedWord *w)
 {
   if (m_hide) return;
-  startLink(w->ref(),w->file(),w->anchor());
+  startLink(w->ref(),w->file(),w->relPath(),w->anchor());
   filter(w->word());
   endLink();
 }
@@ -75,12 +93,15 @@ void HtmlDocVisitor::visit(DocSymbol *s)
     case DocSymbol::Hash:    m_t << "#"; break;
     case DocSymbol::Percent: m_t << "%"; break;
     case DocSymbol::Copy:    m_t << "&copy;"; break;
+    case DocSymbol::Tm:      m_t << "&tm;"; break;
+    case DocSymbol::Reg:     m_t << "&reg;"; break;
     case DocSymbol::Apos:    m_t << "'"; break;
     case DocSymbol::Quot:    m_t << "\""; break;
     case DocSymbol::Uml:     m_t << "&" << s->letter() << "uml;"; break;
     case DocSymbol::Acute:   m_t << "&" << s->letter() << "acute;"; break;
     case DocSymbol::Grave:   m_t << "&" << s->letter() << "grave;"; break;
     case DocSymbol::Circ:    m_t << "&" << s->letter() << "circ;"; break;
+    case DocSymbol::Slash:   m_t << "&" << s->letter() << "slash;"; break;
     case DocSymbol::Tilde:   m_t << "&" << s->letter() << "tilde;"; break;
     case DocSymbol::Szlig:   m_t << "&szlig;"; break;
     case DocSymbol::Cedil:   m_t << "&" << s->letter() << "cedil;"; break;
@@ -94,7 +115,9 @@ void HtmlDocVisitor::visit(DocSymbol *s)
 void HtmlDocVisitor::visit(DocURL *u)
 {
   if (m_hide) return;
-  m_t << "<a href=\"" << u->url() << "\">";
+  m_t << "<a href=\"";
+  if (u->isEmail()) m_t << "mailto:";
+  m_t << u->url() << "\">";
   filter(u->url());
   m_t << "</a>";
 }
@@ -117,57 +140,104 @@ void HtmlDocVisitor::visit(DocStyleChange *s)
   switch (s->style())
   {
     case DocStyleChange::Bold:
-      if (s->enable()) m_t << "<b>";      else m_t << "</b> ";
+      if (s->enable()) m_t << "<b" << htmlAttribsToString(s->attribs()) << ">";      else m_t << "</b>";
       break;
     case DocStyleChange::Italic:
-      if (s->enable()) m_t << "<em>";     else m_t << "</em> ";
+      if (s->enable()) m_t << "<em" << htmlAttribsToString(s->attribs()) << ">";     else m_t << "</em>";
       break;
     case DocStyleChange::Code:
-      if (s->enable()) m_t << "<code>";   else m_t << "</code> ";
+      if (s->enable()) m_t << "<code" << htmlAttribsToString(s->attribs()) << ">";   else m_t << "</code>";
       break;
     case DocStyleChange::Subscript:
-      if (s->enable()) m_t << "<sub>";    else m_t << "</sub> ";
+      if (s->enable()) m_t << "<sub" << htmlAttribsToString(s->attribs()) << ">";    else m_t << "</sub>";
       break;
     case DocStyleChange::Superscript:
-      if (s->enable()) m_t << "<sup>";    else m_t << "</sup> ";
+      if (s->enable()) m_t << "<sup" << htmlAttribsToString(s->attribs()) << ">";    else m_t << "</sup>";
       break;
     case DocStyleChange::Center:
-      if (s->enable()) m_t << "<center>"; else m_t << "</center> ";
+      if (s->enable()) m_t << "<center" << htmlAttribsToString(s->attribs()) << ">"; else m_t << "</center>";
       break;
     case DocStyleChange::Small:
-      if (s->enable()) m_t << "<small>";  else m_t << "</small> ";
+      if (s->enable()) m_t << "<small" << htmlAttribsToString(s->attribs()) << ">";  else m_t << "</small>";
       break;
+    case DocStyleChange::Preformatted:
+      if (s->enable())
+      {
+        m_t << "<pre" << htmlAttribsToString(s->attribs()) << ">";
+        m_insidePre=TRUE;
+      }
+      else
+      {
+        m_insidePre=FALSE;
+        m_t << "</pre>";
+      }
+      break;
+    case DocStyleChange::Div:
+      if (s->enable()) m_t << "<div" << htmlAttribsToString(s->attribs()) << ">";  else m_t << "</div>";
+      break;
+    case DocStyleChange::Span:
+      if (s->enable()) m_t << "<span" << htmlAttribsToString(s->attribs()) << ">";  else m_t << "</span>";
+      break;
+
   }
 }
 
+
 void HtmlDocVisitor::visit(DocVerbatim *s)
 {
   if (m_hide) return;
   switch(s->type())
   {
     case DocVerbatim::Code: // fall though
-      m_t << "<div class=\"fragment\"><pre>"; 
-      parseCode(m_ci,s->context(),s->text(),FALSE,0);
-      m_t << "</pre></div>"; 
+      m_t << "<pre><div class=\"fragment\">"; 
+      parseCode(m_ci,s->context(),s->text().latin1(),s->isExample(),s->exampleFile());
+      m_t << "</div></pre>"; 
       break;
     case DocVerbatim::Verbatim: 
-      m_t << "<div class=\"fragment\"><pre>";
+      m_t << "<pre><div class=\"fragment\">";
       filter(s->text());
-      m_t << "</pre></div>"; 
+      m_t << "</div></pre>"; 
       break;
     case DocVerbatim::HtmlOnly: 
       m_t << s->text(); 
       break;
+    case DocVerbatim::ManOnly: 
     case DocVerbatim::LatexOnly: 
+    case DocVerbatim::XmlOnly: 
       /* nothing */ 
       break;
+     
+    case DocVerbatim::Dot:
+      {
+        static int dotindex = 1;
+        QCString fileName(4096);
+
+        fileName.sprintf("%s%d", 
+            (Config_getString("HTML_OUTPUT")+"/inline_dotgraph_").data(), 
+            dotindex++
+           );
+        QFile file(fileName);
+        if (!file.open(IO_WriteOnly))
+        {
+          err("Could not open file %s for writing\n",fileName.data());
+        }
+        file.writeBlock( s->text(), s->text().length() );
+        file.close();
+
+        m_t << "<div align=\"center\">" << endl;
+        writeDotFile(fileName);
+        m_t << "</div>" << endl;
+
+        file.remove();
+      }
+      break;
   }
 }
 
 void HtmlDocVisitor::visit(DocAnchor *anc)
 {
   if (m_hide) return;
-  m_t << "<a name=\"" /*<< anc->file() << "#"*/ << anc->anchor() << "\"/></a>";
+  m_t << "<a class=\"anchor\" name=\"" << anc->anchor() << "\"></a>";
 }
 
 void HtmlDocVisitor::visit(DocInclude *inc)
@@ -176,9 +246,18 @@ void HtmlDocVisitor::visit(DocInclude *inc)
   switch(inc->type())
   {
     case DocInclude::Include: 
-      m_t << "<div class=\"fragment\"><pre>";
-      parseCode(m_ci,inc->context(),inc->text(),FALSE,0);
-      m_t << "</pre></div>"; 
+      m_t << "<pre><div class=\"fragment\">";
+      parseCode(m_ci,inc->context(),inc->text().latin1(),inc->isExample(),inc->exampleFile());
+      m_t << "</div></pre>"; 
+    case DocInclude::IncWithLines:
+      { 
+         m_t << "<pre><div class=\"fragment\">";
+         QFileInfo cfi( inc->file() );
+         FileDef fd( cfi.dirPath(), cfi.fileName() );
+         parseCode(m_ci,inc->context(),inc->text().latin1(),inc->isExample(),inc->exampleFile(), &fd);
+         m_t << "</div></pre>"; 
+      }
+      break;
       break;
     case DocInclude::DontInclude: 
       break;
@@ -186,9 +265,9 @@ void HtmlDocVisitor::visit(DocInclude *inc)
       m_t << inc->text(); 
       break;
     case DocInclude::VerbInclude: 
-      m_t << "<div class=\"fragment\"><pre>";
+      m_t << "<pre><div class=\"fragment\">";
       filter(inc->text());
-      m_t << "</pre></div>"; 
+      m_t << "</div></pre>"; 
       break;
   }
 }
@@ -199,41 +278,42 @@ void HtmlDocVisitor::visit(DocIncOperator *op)
   //    op->type(),op->isFirst(),op->isLast(),op->text().data());
   if (op->isFirst()) 
   {
-    m_t << "<div class=\"fragment\"><pre>";
-    m_hide = TRUE;
+    if (!m_hide) m_t << "<pre><div class=\"fragment\">";
+    pushEnabled();
+    m_hide=TRUE;
   }
   if (op->type()!=DocIncOperator::Skip) 
   {
-    parseCode(m_ci,op->context(),op->text(),FALSE,0);
+    popEnabled();
+    if (!m_hide) parseCode(m_ci,op->context(),op->text().latin1(),op->isExample(),op->exampleFile());
+    pushEnabled();
+    m_hide=TRUE;
   }
   if (op->isLast())  
   {
-    m_hide = FALSE;
-    m_t << "</pre></div>"; 
+    popEnabled();
+    if (!m_hide) m_t << "</div></pre>"; 
   }
   else
   {
-    m_t << endl;
+    if (!m_hide) m_t << endl;
   }
 }
 
 void HtmlDocVisitor::visit(DocFormula *f)
 {
   if (m_hide) return;
-  if (f->text().at(0)=='\\') m_t << "<p><center>" << endl;
-  m_t << "<img align=";
-#if !defined(_WIN32)
-  m_t << "\"top\"";     // assume Unix users use Netscape 4.x which does
-  // not seem to support align == "middle" :-((
-#else
-  m_t << "\"middle\"";  // assume Windows users use IE or HtmlHelp which on
-  // displays formulas nicely with align == "middle" 
-#endif
+  bool bDisplay = f->text().at(0)=='\\';
+  if (bDisplay) m_t << "<p class=\"formulaDsp\">" << endl;
+  m_t << "<img class=\"formula" << (bDisplay ? "Dsp" : "Inl");
+  m_t << "\" alt=\"";
+  filterQuotedCdataAttr(f->text());
+  m_t << "\"";
+  /// @todo cache image dimensions on formula generation and give height/width
+  /// for faster preloading and better rendering of the page
   m_t << " src=\"" << f->name() << ".png\">";
-  if (f->text().at(0)=='\\') 
-    m_t << endl << "</center><p>" << endl;
-  else
-    m_t << " ";
+  if (bDisplay)
+    m_t << endl << "<p>" << endl;
 }
 
 void HtmlDocVisitor::visit(DocIndexEntry *)
@@ -246,63 +326,86 @@ void HtmlDocVisitor::visit(DocIndexEntry *)
 
 void HtmlDocVisitor::visitPre(DocAutoList *l)
 {
+  if (m_hide) return;
   if (l->isEnumList())
   {
-    m_t << "<ol>\n";
+    m_t << "<ol>";
   }
   else
   {
-    m_t << "<ul>\n";
+    m_t << "<ul>";
   }
+  if (!l->isPreformatted()) m_t << "\n";
 }
 
 void HtmlDocVisitor::visitPost(DocAutoList *l)
 {
+  if (m_hide) return;
   if (l->isEnumList())
   {
-    m_t << "</ol>\n";
+    m_t << "</ol>";
   }
   else
   {
-    m_t << "</ul>\n";
+    m_t << "</ul>";
   }
+  if (!l->isPreformatted()) m_t << "\n";
 }
 
 void HtmlDocVisitor::visitPre(DocAutoListItem *)
 {
+  if (m_hide) return;
   m_t << "<li>";
 }
 
 void HtmlDocVisitor::visitPost(DocAutoListItem *) 
 {
+  if (m_hide) return;
   m_t << "</li>";
 }
 
 void HtmlDocVisitor::visitPre(DocPara *) 
 {
+  if (m_hide) return;
+
+  // TODO:
+  // Paragraph should be surrounded by <p>..</p>, but
+  // 
+  // A list item (li), description data (dd), or table data (td) should
+  // only have paragraph markers if there are multiple paragraphs (otherwise
+  // the output looks ugly).
+  //
+  // A list or table should be placed outside the paragraph context,
+  // so the current paragraph should be ended and restarted. To avoid
+  // empty paragraphs, it has to be checked if the list or table is the
+  // first or last child within the paragraph.
+  
 }
 
 void HtmlDocVisitor::visitPost(DocPara *p)
 {
+  if (m_hide) return;
   if (!p->isLast() &&            // omit <p> for last paragraph
       !(p->parent() &&           // and for parameter sections
         p->parent()->kind()==DocNode::Kind_ParamSect
-       )
-     ) m_t << "\n<p>\n";
+       ) 
+     ) 
+  {
+    m_t << "<p>\n";
+  }
 }
 
 void HtmlDocVisitor::visitPre(DocRoot *)
 {
-  //m_t << "<hr><h4><font color=\"red\">New parser:</font></h4>\n";
 }
 
 void HtmlDocVisitor::visitPost(DocRoot *)
 {
-  //m_t << "<hr><h4><font color=\"red\">Old parser:</font></h4>\n";
 }
 
 void HtmlDocVisitor::visitPre(DocSimpleSect *s)
 {
+  if (m_hide) return;
   m_t << "<dl compact><dt><b>";
   switch(s->type())
   {
@@ -335,11 +438,12 @@ void HtmlDocVisitor::visitPre(DocSimpleSect *s)
     case DocSimpleSect::Attention:
       m_t << theTranslator->trAttention(); break;
     case DocSimpleSect::User: break;
+    case DocSimpleSect::Rcs: break;
     case DocSimpleSect::Unknown:  break;
   }
 
   // special case 1: user defined title
-  if (s->type()!=DocSimpleSect::User)
+  if (s->type()!=DocSimpleSect::User && s->type()!=DocSimpleSect::Rcs)
   {
     m_t << ":</b></dt><dd>";
   }
@@ -347,6 +451,7 @@ void HtmlDocVisitor::visitPre(DocSimpleSect *s)
 
 void HtmlDocVisitor::visitPost(DocSimpleSect *)
 {
+  if (m_hide) return;
   m_t << "</dd></dl>\n";
 }
 
@@ -356,36 +461,45 @@ void HtmlDocVisitor::visitPre(DocTitle *)
 
 void HtmlDocVisitor::visitPost(DocTitle *)
 {
+  if (m_hide) return;
   m_t << "</b></dt><dd>";
 }
 
-void HtmlDocVisitor::visitPre(DocSimpleList *)
+void HtmlDocVisitor::visitPre(DocSimpleList *sl)
 {
-  m_t << "<ul>\n";
+  if (m_hide) return;
+  m_t << "<ul>";
+  if (!sl->isPreformatted()) m_t << "\n";
 }
 
-void HtmlDocVisitor::visitPost(DocSimpleList *)
+void HtmlDocVisitor::visitPost(DocSimpleList *sl)
 {
-  m_t << "</ul>\n";
+  if (m_hide) return;
+  m_t << "</ul>";
+  if (!sl->isPreformatted()) m_t << "\n";
 }
 
 void HtmlDocVisitor::visitPre(DocSimpleListItem *)
 {
+  if (m_hide) return;
   m_t << "<li>";
 }
 
-void HtmlDocVisitor::visitPost(DocSimpleListItem *) 
+void HtmlDocVisitor::visitPost(DocSimpleListItem *li) 
 {
-  m_t << "</li>\n";
+  if (m_hide) return;
+  m_t << "</li>";
+  if (!li->isPreformatted()) m_t << "\n";
 }
 
 void HtmlDocVisitor::visitPre(DocSection *s)
 {
+  if (m_hide) return;
   m_t << "<h" << s->level()+1 << ">";
-  m_t << "<a name=\"" << s->anchor();
-  m_t << "\"></a>" << endl;
+  m_t << "<a class=\"anchor\" name=\"" << s->anchor();
+  m_t << "\">" << endl;
   filter(s->title());
-  m_t << "</h" << s->level()+1 << ">\n";
+  m_t << "</a></h" << s->level()+1 << ">\n";
 }
 
 void HtmlDocVisitor::visitPost(DocSection *) 
@@ -394,140 +508,188 @@ void HtmlDocVisitor::visitPost(DocSection *)
 
 void HtmlDocVisitor::visitPre(DocHtmlList *s)
 {
+  if (m_hide) return;
   if (s->type()==DocHtmlList::Ordered) 
-    m_t << "<ol>\n"; 
+    m_t << "<ol" << htmlAttribsToString(s->attribs()) << ">\n"; 
   else 
-    m_t << "<ul>\n";
+    m_t << "<ul" << htmlAttribsToString(s->attribs()) << ">\n";
 }
 
 void HtmlDocVisitor::visitPost(DocHtmlList *s) 
 {
+  if (m_hide) return;
   if (s->type()==DocHtmlList::Ordered) 
-    m_t << "</ol>\n"; 
+    m_t << "</ol>"; 
   else 
-    m_t << "</ul>\n";
+    m_t << "</ul>";
+  if (!s->isPreformatted()) m_t << "\n";
 }
 
-void HtmlDocVisitor::visitPre(DocHtmlListItem *)
+void HtmlDocVisitor::visitPre(DocHtmlListItem *i)
 {
-  m_t << "<li>\n";
+  if (m_hide) return;
+  m_t << "<li" << htmlAttribsToString(i->attribs()) << ">";
+  if (!i->isPreformatted()) m_t << "\n";
 }
 
 void HtmlDocVisitor::visitPost(DocHtmlListItem *) 
 {
+  if (m_hide) return;
   m_t << "</li>\n";
 }
 
-void HtmlDocVisitor::visitPre(DocHtmlPre *)
+void HtmlDocVisitor::visitPre(DocHtmlDescList *dl)
 {
-  m_t << "<pre>\n";
-  m_insidePre=TRUE;
-}
-
-void HtmlDocVisitor::visitPost(DocHtmlPre *) 
-{
-  m_insidePre=FALSE;
-  m_t << "</pre>\n";
-}
-
-void HtmlDocVisitor::visitPre(DocHtmlDescList *)
-{
-  m_t << "<dl>\n";
+  if (m_hide) return;
+  m_t << "<dl" << htmlAttribsToString(dl->attribs()) << ">\n";
 }
 
 void HtmlDocVisitor::visitPost(DocHtmlDescList *) 
 {
+  if (m_hide) return;
   m_t << "</dl>\n";
 }
 
-void HtmlDocVisitor::visitPre(DocHtmlDescTitle *)
+void HtmlDocVisitor::visitPre(DocHtmlDescTitle *dt)
 {
-  m_t << "<dt>";
+  if (m_hide) return;
+  m_t << "<dt" << htmlAttribsToString(dt->attribs()) << ">";
 }
 
 void HtmlDocVisitor::visitPost(DocHtmlDescTitle *) 
 {
+  if (m_hide) return;
   m_t << "</dt>\n";
 }
 
-void HtmlDocVisitor::visitPre(DocHtmlDescData *)
+void HtmlDocVisitor::visitPre(DocHtmlDescData *dd)
 {
-  m_t << "<dd>";
+  if (m_hide) return;
+  m_t << "<dd" << htmlAttribsToString(dd->attribs()) << ">";
 }
 
 void HtmlDocVisitor::visitPost(DocHtmlDescData *) 
 {
+  if (m_hide) return;
   m_t << "</dd>\n";
 }
 
-void HtmlDocVisitor::visitPre(DocHtmlTable *)
+void HtmlDocVisitor::visitPre(DocHtmlTable *t)
 {
-  m_t << "<table border=\"1\" cellspacing=\"3\" cellpadding=\"3\">\n";
+  if (m_hide) return;
+  bool hasBorder      = FALSE;
+  bool hasCellSpacing = FALSE;
+  bool hasCellPadding = FALSE;
+
+  HtmlAttribListIterator li(t->attribs());
+  HtmlAttrib *att;
+  for (li.toFirst();(att=li.current());++li)
+  {
+    if      (att->name=="border")      hasBorder=TRUE;
+    else if (att->name=="cellspacing") hasCellSpacing=TRUE;
+    else if (att->name=="cellpadding") hasCellPadding=TRUE;
+  }
+  m_t << "<table" << htmlAttribsToString(t->attribs());
+  if (!hasBorder)      m_t << " border=\"1\"";
+  if (!hasCellSpacing) m_t << " cellspacing=\"3\"";
+  if (!hasCellPadding) m_t << " cellpadding=\"3\"";
+  m_t << ">\n";
 }
 
 void HtmlDocVisitor::visitPost(DocHtmlTable *) 
 {
+  if (m_hide) return;
   m_t << "</table>\n";
 }
 
-void HtmlDocVisitor::visitPre(DocHtmlRow *)
+void HtmlDocVisitor::visitPre(DocHtmlRow *tr)
 {
-  m_t << "<tr>\n";
+  if (m_hide) return;
+  m_t << "<tr" << htmlAttribsToString(tr->attribs()) << ">\n";
 }
 
 void HtmlDocVisitor::visitPost(DocHtmlRow *) 
 {
+  if (m_hide) return;
   m_t << "</tr>\n";
 }
 
 void HtmlDocVisitor::visitPre(DocHtmlCell *c)
 {
-  if (c->isHeading()) m_t << "<th>"; else m_t << "<td>";
+  if (m_hide) return;
+  if (c->isHeading()) 
+  {
+    m_t << "<th" << htmlAttribsToString(c->attribs()) << ">"; 
+  }
+  else 
+  {
+    m_t << "<td" << htmlAttribsToString(c->attribs()) << ">";
+  }
 }
 
 void HtmlDocVisitor::visitPost(DocHtmlCell *c) 
 {
+  if (m_hide) return;
   if (c->isHeading()) m_t << "</th>"; else m_t << "</td>";
 }
 
-void HtmlDocVisitor::visitPre(DocHtmlCaption *)
+void HtmlDocVisitor::visitPre(DocHtmlCaption *c)
 {
-  m_t << "<caption align=\"bottom\">";
+  if (m_hide) return;
+  bool hasAlign      = FALSE;
+  HtmlAttribListIterator li(c->attribs());
+  HtmlAttrib *att;
+  for (li.toFirst();(att=li.current());++li)
+  {
+    if (att->name=="align") hasAlign=TRUE;
+  }
+  m_t << "<caption" << htmlAttribsToString(c->attribs());
+  if (!hasAlign) m_t << " align=\"bottom\"";
+  m_t << ">";
 }
 
 void HtmlDocVisitor::visitPost(DocHtmlCaption *) 
 {
+  if (m_hide) return;
   m_t << "</caption>\n";
 }
 
 void HtmlDocVisitor::visitPre(DocInternal *)
 {
+  if (m_hide) return;
   m_t << "<p><b>" << theTranslator->trForInternalUseOnly() << "</b></p>" << endl;
   m_t << "<p>" << endl;
 }
 
 void HtmlDocVisitor::visitPost(DocInternal *) 
 {
+  if (m_hide) return;
   m_t << "</p>" << endl;
 }
 
 void HtmlDocVisitor::visitPre(DocHRef *href)
 {
-  m_t << "<a href=\"" << href->url() << "\">";
+  if (m_hide) return;
+  m_t << "<a href=\"" << href->url()  << "\""
+      << htmlAttribsToString(href->attribs()) << ">";
 }
 
 void HtmlDocVisitor::visitPost(DocHRef *) 
 {
+  if (m_hide) return;
   m_t << "</a>";
 }
 
 void HtmlDocVisitor::visitPre(DocHtmlHeader *header)
 {
-  m_t << "<h" << header->level() << ">";
+  if (m_hide) return;
+  m_t << "<h" << header->level() 
+      << htmlAttribsToString(header->attribs()) << ">";
 }
 
 void HtmlDocVisitor::visitPost(DocHtmlHeader *header) 
 {
+  if (m_hide) return;
   m_t << "</h" << header->level() << ">\n";
 }
 
@@ -535,7 +697,8 @@ void HtmlDocVisitor::visitPre(DocImage *img)
 {
   if (img->type()==DocImage::Html)
   {
-    QCString baseName=img->name();
+    if (m_hide) return;
+    QString baseName=img->name();
     int i;
     if ((i=baseName.findRev('/'))!=-1 || (i=baseName.findRev('\\'))!=-1)
     {
@@ -543,7 +706,7 @@ void HtmlDocVisitor::visitPre(DocImage *img)
     }
     m_t << "<div align=\"center\">" << endl;
     m_t << "<img src=\"" << img->name() << "\" alt=\"" 
-      << baseName << "\">" << endl;
+      << baseName << "\"" << htmlAttribsToString(img->attribs()) << ">" << endl;
     if (img->hasCaption())
     {
       m_t << "<p><strong>";
@@ -551,6 +714,7 @@ void HtmlDocVisitor::visitPre(DocImage *img)
   }
   else // other format -> skip
   {
+    pushEnabled();
     m_hide=TRUE;
   }
 }
@@ -559,6 +723,7 @@ void HtmlDocVisitor::visitPost(DocImage *img)
 {
   if (img->type()==DocImage::Html)
   {
+    if (m_hide) return;
     if (img->hasCaption())
     {
       m_t << "</strong></p>";
@@ -567,24 +732,15 @@ void HtmlDocVisitor::visitPost(DocImage *img)
   }
   else // other format
   {
-    m_hide=FALSE;
+    popEnabled();
   }
 }
 
 void HtmlDocVisitor::visitPre(DocDotFile *df)
 {
-  QCString baseName=df->file();
-  int i;
-  if ((i=baseName.findRev('/'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  } 
-  QCString outDir = Config_getString("HTML_OUTPUT");
-  writeDotGraphFromFile(df->file(),outDir,baseName,BITMAP);
+  if (m_hide) return;
+  writeDotFile(df->file());
   m_t << "<div align=\"center\">" << endl;
-  m_t << "<img src=\"" << baseName << "." 
-    << Config_getEnum("DOT_IMAGE_FORMAT") << "\" alt=\""
-    << baseName << "\">" << endl;
   if (df->hasCaption())
   { 
     m_t << "<p><strong>";
@@ -593,6 +749,7 @@ void HtmlDocVisitor::visitPre(DocDotFile *df)
 
 void HtmlDocVisitor::visitPost(DocDotFile *df) 
 {
+  if (m_hide) return;
   if (df->hasCaption())
   {
     m_t << "</strong></p>" << endl;
@@ -602,30 +759,39 @@ void HtmlDocVisitor::visitPost(DocDotFile *df)
 
 void HtmlDocVisitor::visitPre(DocLink *lnk)
 {
-  startLink(lnk->ref(),lnk->file(),lnk->anchor());
+  if (m_hide) return;
+  startLink(lnk->ref(),lnk->file(),lnk->relPath(),lnk->anchor());
 }
 
 void HtmlDocVisitor::visitPost(DocLink *) 
 {
+  if (m_hide) return;
   endLink();
 }
 
 void HtmlDocVisitor::visitPre(DocRef *ref)
 {
-  startLink(ref->ref(),ref->file(),ref->anchor());
+  if (m_hide) return;
+  if (!ref->file().isEmpty()) 
+  {
+    startLink(ref->ref(),ref->file(),ref->relPath(),ref->anchor());
+  }
   if (!ref->hasLinkText()) filter(ref->targetTitle());
 }
 
-void HtmlDocVisitor::visitPost(DocRef *) 
+void HtmlDocVisitor::visitPost(DocRef *ref) 
 {
-  endLink();
-  m_t << " ";
+  if (m_hide) return;
+  if (!ref->file().isEmpty()) endLink();
+  //m_t << " ";
 }
 
 void HtmlDocVisitor::visitPre(DocSecRefItem *ref)
 {
-  QCString refName=ref->file();
-  if (refName.right(Doxygen::htmlFileExtension.length())!=Doxygen::htmlFileExtension)
+  if (m_hide) return;
+  QString refName=ref->file();
+  if (refName.right(Doxygen::htmlFileExtension.length())!=
+      QString(Doxygen::htmlFileExtension))
   {
     refName+=Doxygen::htmlFileExtension;
   }
@@ -635,31 +801,46 @@ void HtmlDocVisitor::visitPre(DocSecRefItem *ref)
 
 void HtmlDocVisitor::visitPost(DocSecRefItem *) 
 {
+  if (m_hide) return;
   m_t << "</a> ";
 }
 
 void HtmlDocVisitor::visitPre(DocSecRefList *)
 {
+  if (m_hide) return;
   m_t << "<multicol cols=3>" << endl;
   m_t << "<ul>" << endl;
 }
 
 void HtmlDocVisitor::visitPost(DocSecRefList *) 
 {
+  if (m_hide) return;
   m_t << "</ul>" << endl;
   m_t << "</multicol>" << endl;
 }
 
-void HtmlDocVisitor::visitPre(DocLanguage *)
+void HtmlDocVisitor::visitPre(DocLanguage *l)
 {
+  QString langId = Config_getEnum("OUTPUT_LANGUAGE");
+  if (l->id().lower()!=langId.lower())
+  {
+    pushEnabled();
+    m_hide = TRUE;
+  }
 }
 
-void HtmlDocVisitor::visitPost(DocLanguage *) 
+void HtmlDocVisitor::visitPost(DocLanguage *l) 
 {
+  QString langId = Config_getEnum("OUTPUT_LANGUAGE");
+  if (l->id().lower()!=langId.lower())
+  {
+    popEnabled();
+  }
 }
 
 void HtmlDocVisitor::visitPre(DocParamSect *s)
 {
+  if (m_hide) return;
   m_t << "<dl compact><dt><b>";
   switch(s->type())
   {
@@ -679,31 +860,53 @@ void HtmlDocVisitor::visitPre(DocParamSect *s)
 
 void HtmlDocVisitor::visitPost(DocParamSect *)
 {
+  if (m_hide) return;
   m_t << "  </table>" << endl;
   m_t << "</dl>" << endl;
 }
 
 void HtmlDocVisitor::visitPre(DocParamList *pl)
 {
-  m_t << "    <tr><td valign=top><em>";
+  if (m_hide) return;
+  m_t << "    <tr><td>";
+  if (pl->direction()!=DocParamSect::Unspecified)
+  {
+    m_t << "<tt>[";
+    if (pl->direction()==DocParamSect::In)
+    {
+      m_t << "in";
+    }
+    else if (pl->direction()==DocParamSect::Out)
+    {
+      m_t << "out";
+    }
+    else if (pl->direction()==DocParamSect::InOut)
+    {
+      m_t << "in,out";
+    }
+    m_t << "]</tt>&nbsp;";
+  }
+  m_t << "</td><td valign=top><em>";
   QStrListIterator li(pl->parameters());
   const char *s;
   bool first=TRUE;
   for (li.toFirst();(s=li.current());++li)
   {
     if (!first) m_t << ","; else first=FALSE;
-    m_t << s;
+    filter(s);
   }
   m_t << "</em>&nbsp;</td><td>";
 }
 
 void HtmlDocVisitor::visitPost(DocParamList *)
 {
+  if (m_hide) return;
   m_t << "</td></tr>" << endl;
 }
 
 void HtmlDocVisitor::visitPre(DocXRefItem *x)
 {
+  if (m_hide) return;
   m_t << "<dl compact><dt><b><a class=\"el\" href=\"" 
     << x->file() << Doxygen::htmlFileExtension << "#" << x->anchor() << "\">";
   filter(x->title());
@@ -712,16 +915,19 @@ void HtmlDocVisitor::visitPre(DocXRefItem *x)
 
 void HtmlDocVisitor::visitPost(DocXRefItem *)
 {
+  if (m_hide) return;
   m_t << "</dd></dl>" << endl;
 }
 
 void HtmlDocVisitor::visitPre(DocInternalRef *ref)
 {
-  startLink(0,ref->file(),ref->anchor());
+  if (m_hide) return;
+  startLink(0,ref->file(),ref->relPath(),ref->anchor());
 }
 
 void HtmlDocVisitor::visitPost(DocInternalRef *) 
 {
+  if (m_hide) return;
   endLink();
   m_t << " ";
 }
@@ -734,6 +940,14 @@ void HtmlDocVisitor::visitPost(DocCopy *)
 {
 }
 
+void HtmlDocVisitor::visitPre(DocText *)
+{
+}
+
+void HtmlDocVisitor::visitPost(DocText *)
+{
+}
+
 void HtmlDocVisitor::filter(const char *str)
 { 
   if (str==0) return;
@@ -752,7 +966,45 @@ void HtmlDocVisitor::filter(const char *str)
   }
 }
 
-void HtmlDocVisitor::startLink(const QCString &ref,const QCString &file,const QCString &anchor)
+/// Escape basic entities to produce a valid CDATA attribute value,
+/// assume that the outer quoting will be using the double quote &quot;
+void HtmlDocVisitor::filterQuotedCdataAttr(const char* str)
+{
+  if (str==0) return;
+  const char *p=str;
+  char c;
+  while (*p)
+  {
+    c=*p++;
+    switch(c)
+    {
+      case '&':  m_t << "&amp;"; break;
+      case '"':  m_t << "&quot;"; break;
+       // For SGML compliance, and given the SGML declaration for HTML syntax,
+       // it's enough to replace these two, provided that the declaration
+       // for the HTML version we generate (and as supported by the browser)
+       // specifies that all the other symbols used in rawVal are
+       // within the right charachter class (i.e., they're not
+       // some multinational weird charachters not in the BASESET).
+       // We assume that 1) the browser will support whatever is remaining
+       // in the formula and 2) the TeX formulae are generally governed
+       // by even stricter charachter restrictions so it should be enough.
+       //
+       // On some incompliant browsers, additional translation of
+       // '>' and '<' into "&gt;" and "&lt;", respectively, might be needed;
+       // but I'm unaware of particular modern (last 4 years) versions
+       // with such problems, so let's not do it for performance.
+       // Also, some brousers will (wrongly) not process the entity references
+       // inside the attribute value and show the &...; form instead,  
+       // so we won't create entites unless necessary to minimize clutter there.
+       // --vassilii 
+      default:   m_t << c;
+    }
+  }
+}
+
+void HtmlDocVisitor::startLink(const QString &ref,const QString &file,
+                               const QString &relPath,const QString &anchor)
 {
   QCString *dest;
   if (!ref.isEmpty()) // link to entity imported via tag file
@@ -769,7 +1021,11 @@ void HtmlDocVisitor::startLink(const QCString &ref,const QCString &file,const QC
   m_t << "href=\"";
   if (!ref.isEmpty())
   {
-    if ((dest=Doxygen::tagDestinationDict[ref])) m_t << *dest << "/";
+    if ((dest=Doxygen::tagDestinationDict[ref])) m_t << relPath << *dest << "/";
+  }
+  else
+  {
+    m_t << relPath;
   }
   if (!file.isEmpty()) m_t << file << Doxygen::htmlFileExtension;
   if (!anchor.isEmpty()) m_t << "#" << anchor;
@@ -781,3 +1037,35 @@ void HtmlDocVisitor::endLink()
   m_t << "</a>";
 }
 
+void HtmlDocVisitor::pushEnabled()
+{
+  m_enabled.push(new bool(m_hide));
+}
+
+void HtmlDocVisitor::popEnabled()
+{
+  bool *v=m_enabled.pop();
+  ASSERT(v!=0);
+  m_hide = *v;
+  delete v;
+}
+
+void HtmlDocVisitor::writeDotFile(const QString &fileName)
+{
+  QString baseName=fileName;
+  int i;
+  if ((i=baseName.findRev('/'))!=-1)
+  {
+    baseName=baseName.right(baseName.length()-i-1);
+  } 
+  QString outDir = Config_getString("HTML_OUTPUT");
+  writeDotGraphFromFile(fileName,outDir,baseName,BITMAP);
+  QString mapName = baseName+".map";
+  QString mapFile = fileName+".map";
+  m_t << "<img src=\"" << baseName << "." 
+    << Config_getEnum("DOT_IMAGE_FORMAT") << "\" alt=\""
+    << baseName << "\" border=\"0\" usemap=\"#" << mapName << "\">" << endl;
+  QString imap = getDotImageMapFromFile(fileName,outDir);
+  m_t << "<map name=\"" << mapName << "\">" << imap << "</map>" << endl;
+}
+
