@@ -2,7 +2,7 @@
  *
  * $Id: htmlgen.cpp,v 1.56 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -32,73 +32,242 @@
 #include "htmlhelp.h"
 #include "docparser.h"
 #include "htmldocvisitor.h"
+#include "index.h"
+#include "pagedef.h"
+#include "debug.h"
 
 // #define GROUP_COLOR "#ff8080"
 
 //#define DBG_HTML(x) x;
 #define DBG_HTML(x) 
 
-
-/*
-    changed default stylesheet, startIndexKey(), startIndexValue()
-    02 jan 2002, jh
-*/
-
 static const char *defaultStyleSheet = 
-"H1 { text-align: center; }\n"
+"H1 {\n"
+"	text-align: center;\n"
+"	font-family: Geneva, Arial, Helvetica, sans-serif;\n"
+"}\n"
+"H2 {\n"
+"	font-family: Geneva, Arial, Helvetica, sans-serif;\n"
+"}\n"
 "CAPTION { font-weight: bold }\n"
-"A.qindex {}\n"
-"A.qindexRef {}\n"
+"DIV.qindex {\n"
+"	width: 100%;\n"
+"	background-color: #eeeeff;\n"
+"	border: 1px solid #B0B0B0;\n"
+"	text-align: center;\n"
+"	margin: 2px;\n"
+"	padding: 2px;\n"
+"	line-height: 120%;\n"
+"}\n"
+"A.qindex {\n"
+"       text-decoration: none;\n"
+"       font-weight: bold;\n"
+"       color: #1A419D;\n"
+"       padding: 2px;\n"
+"}\n"
+"A.qindex:visited {\n"
+"       text-decoration: none;\n"
+"       font-weight: bold;\n"
+"       color: #1A419D\n"
+"       padding: 2px;\n"
+"}\n"
+"A.qindex:hover {\n"
+"	text-decoration: none;\n"
+"	background-color: #ddddff;\n"
+"	padding: 2px;\n"
+"}\n"
+"A.qindexHL {\n"
+"	text-decoration: none;\n"
+"	font-weight: bold;\n"
+"	background-color: #6666cc;\n"
+"	color: #ffffff;\n"
+"	padding: 2px 6px;\n"
+"	border: 1px double #9295C2;\n"
+"}\n"
+"A.qindexHL:hover {\n"
+"	text-decoration: none;\n"
+"	background-color: #6666cc;\n"
+"	color: #ffffff;\n"
+"	padding: 2px 6px;\n"
+"}\n"
+"A.qindexHL:visited { text-decoration: none; background-color: #6666cc; color: #ffffff }\n"
 "A.el { text-decoration: none; font-weight: bold }\n"
 "A.elRef { font-weight: bold }\n"
-"A.code { text-decoration: none; font-weight: normal; color: #4444ee }\n"
-"A.codeRef { font-weight: normal; color: #4444ee }\n"
+"A.code { text-decoration: none; font-weight: normal; color: #1A419D}\n"
+"A.codeRef { font-weight: normal; color: #1A419D}\n"
 "A:hover { text-decoration: none; background-color: #f2f2ff }\n"
 "DL.el { margin-left: -1cm }\n"
-"DIV.fragment { width: 100%; border: none; background-color: #eeeeee }\n"
+"PRE.fragment {\n"
+"	border: 1px solid #CCCCCC;\n"
+"	background-color: #f5f5f5;\n"
+"	margin-top: 4px;\n"
+"	margin-bottom: 4px;\n"
+"	margin-left: 2px;\n"
+"	margin-right: 8px;\n"
+"	padding-left: 6px;\n"
+"	padding-right: 6px;\n"
+"	padding-top: 4px;\n"
+"	padding-bottom: 4px;\n"
+"}\n"
+"DIV.fragment {\n"
+"	border: 1px solid #CCCCCC;\n"
+"	background-color: #f5f5f5;\n"
+"	padding: 6px;\n"
+"}\n"
 "DIV.ah { background-color: black; font-weight: bold; color: #ffffff; margin-bottom: 3px; margin-top: 3px }\n"
-"TD.md { background-color: #f2f2ff; font-weight: bold; }\n"
-"TD.mdname1 { background-color: #f2f2ff; font-weight: bold; color: #602020; }\n"
-"TD.mdname { background-color: #f2f2ff; font-weight: bold; color: #602020; width: 600px; }\n"
-"DIV.groupHeader { margin-left: 16px; margin-top: 12px; margin-bottom: 6px; font-weight: bold }\n"
+"TD.md { background-color: #F4F4FB; font-weight: bold; }\n"
+"TD.mdname1 { background-color: #F4F4FB; font-weight: bold; color: #602020; }\n"
+"TD.mdname { background-color: #F4F4FB; font-weight: bold; color: #602020; width: 600px; }\n"
+"DIV.groupHeader {\n"
+"       margin-left: 16px;\n"
+"       margin-top: 12px;\n"
+"       margin-bottom: 6px;\n"
+"       font-weight: bold;\n"
+"	font-family: Geneva, Arial, Helvetica, sans-serif;\n"
+"}\n"
 "DIV.groupText { margin-left: 16px; font-style: italic; font-size: smaller }\n"
-"BODY { background: white }\n"
-"TD.indexkey { \n"
-"   background-color: #eeeeff; \n"
-"   font-weight: bold; \n"
-"   padding-right  : 10px; \n"
-"   padding-top    : 2px; \n"
-"   padding-left   : 10px; \n"
-"   padding-bottom : 2px; \n"
-"   margin-left    : 0px; \n"
-"   margin-right   : 0px; \n"
-"   margin-top     : 2px; \n"
-"   margin-bottom  : 2px  \n"
+"BODY {\n"
+"	background: white;\n"
+"	color: black;\n"
+"	margin-right: 20px;\n"
+"	margin-left: 20px;\n"
 "}\n"
-"TD.indexvalue { \n"
-"   background-color: #eeeeff; \n"
-"   font-style: italic; \n"
-"   padding-right  : 10px; \n"
-"   padding-top    : 2px; \n"
-"   padding-left   : 10px; \n"
-"   padding-bottom : 2px; \n"
-"   margin-left    : 0px; \n"
-"   margin-right   : 0px; \n"
-"   margin-top     : 2px; \n"
-"   margin-bottom  : 2px  \n"
+"TD.indexkey {\n"
+"	background-color: #eeeeff;\n"
+"	font-weight: bold;\n"
+"	padding-right  : 10px;\n"
+"	padding-top    : 2px;\n"
+"	padding-left   : 10px;\n"
+"	padding-bottom : 2px;\n"
+"	margin-left    : 0px;\n"
+"	margin-right   : 0px;\n"
+"	margin-top     : 2px;\n"
+"	margin-bottom  : 2px;\n"
+"	border: 1px solid #CCCCCC;\n"
 "}\n"
-"span.keyword       { color: #008000 }\n"
-"span.keywordtype   { color: #604020 }\n"
-"span.keywordflow   { color: #e08000 }\n"
-"span.comment       { color: #800000 }\n"
-"span.preprocessor  { color: #806020 }\n"
-"span.stringliteral { color: #002080 }\n"
-"span.charliteral   { color: #008080 }\n";
-
+"TD.indexvalue {\n"
+"	background-color: #eeeeff;\n"
+"	font-style: italic;\n"
+"	padding-right  : 10px;\n"
+"	padding-top    : 2px;\n"
+"	padding-left   : 10px;\n"
+"	padding-bottom : 2px;\n"
+"	margin-left    : 0px;\n"
+"	margin-right   : 0px;\n"
+"	margin-top     : 2px;\n"
+"	margin-bottom  : 2px;\n"
+"	border: 1px solid #CCCCCC;\n"
+"}\n"
+"TR.memlist {\n"
+"   background-color: #f0f0f0; \n"
+"}\n"
+"P.formulaDsp { text-align: center; }\n"
+"IMG.formulaDsp { }\n"
+"IMG.formulaInl { vertical-align: middle; }\n"
+"SPAN.keyword       { color: #008000 }\n"
+"SPAN.keywordtype   { color: #604020 }\n"
+"SPAN.keywordflow   { color: #e08000 }\n"
+"SPAN.comment       { color: #800000 }\n"
+"SPAN.preprocessor  { color: #806020 }\n"
+"SPAN.stringliteral { color: #002080 }\n"
+"SPAN.charliteral   { color: #008080 }\n"
+".mdTable {\n"
+"	border: 1px solid #868686;\n"
+"	background-color: #F4F4FB;\n"
+"}\n"
+".mdRow {\n"
+"	padding: 8px 10px;\n"
+"}\n"
+".mdescLeft {\n"
+"	font-size: smaller;\n"
+"	font-style: italic;\n"
+"	background-color: #FAFAFA;\n"
+"	padding-left: 8px;\n"
+"	border-top: 1px none #E0E0E0;\n"
+"	border-right: 1px none #E0E0E0;\n"
+"	border-bottom: 1px none #E0E0E0;\n"
+"	border-left: 1px none #E0E0E0;\n"
+"	margin: 0px;\n"
+"}\n"
+".mdescRight {\n"
+"	font-size: smaller;\n"
+"	font-style: italic;\n"
+"	background-color: #FAFAFA;\n"
+"	padding-left: 4px;\n"
+"	border-top: 1px none #E0E0E0;\n"
+"	border-right: 1px none #E0E0E0;\n"
+"	border-bottom: 1px none #E0E0E0;\n"
+"	border-left: 1px none #E0E0E0;\n"
+"	margin: 0px;\n"
+"	padding-bottom: 0px;\n"
+"	padding-right: 8px;\n"
+"}\n"
+".memItemLeft {\n"
+"	padding: 1px 0px 0px 8px;\n"
+"	margin: 4px;\n"
+"	border-top-width: 1px;\n"
+"	border-right-width: 1px;\n"
+"	border-bottom-width: 1px;\n"
+"	border-left-width: 1px;\n"
+"	border-top-style: solid;\n"
+"	border-top-color: #E0E0E0;\n"
+"	border-right-color: #E0E0E0;\n"
+"	border-bottom-color: #E0E0E0;\n"
+"	border-left-color: #E0E0E0;\n"
+"	border-right-style: none;\n"
+"	border-bottom-style: none;\n"
+"	border-left-style: none;\n"
+"	background-color: #FAFAFA;\n"
+"	font-family: Geneva, Arial, Helvetica, sans-serif;\n"
+"	font-size: 12px;\n"
+"}\n"
+".memItemRight {\n"
+"	padding: 1px 0px 0px 8px;\n"
+"	margin: 4px;\n"
+"	border-top-width: 1px;\n"
+"	border-right-width: 1px;\n"
+"	border-bottom-width: 1px;\n"
+"	border-left-width: 1px;\n"
+"	border-top-style: solid;\n"
+"	border-top-color: #E0E0E0;\n"
+"	border-right-color: #E0E0E0;\n"
+"	border-bottom-color: #E0E0E0;\n"
+"	border-left-color: #E0E0E0;\n"
+"	border-right-style: none;\n"
+"	border-bottom-style: none;\n"
+"	border-left-style: none;\n"
+"	background-color: #FAFAFA;\n"
+"	font-family: Geneva, Arial, Helvetica, sans-serif;\n"
+"	font-size: 13px;\n"
+"}\n"
+".search     { color: #003399;\n"
+"              font-weight: bold;\n"
+"}\n"
+"FORM.search {\n"
+"              margin-bottom: 0px;\n"
+"              margin-top: 0px;\n"
+"}\n"
+"INPUT.search { font-size: 75%;\n"
+"               color: #000080;\n"
+"               font-weight: normal;\n"
+"               background-color: #eeeeff;\n"
+"}\n"
+"TD.tiny      { font-size: 75%;\n"
+"}\n"
+"a {\n"
+"	color: #252E78;\n"
+"}\n"
+"a:visited {\n"
+"	color: #3D2185;\n"
+"}\n";
 
 static QCString g_header;
 static QCString g_footer;
 
+const char search_script[]=
+#include "search_php.h"
+;
+
 HtmlGenerator::HtmlGenerator() : OutputGenerator()
 {
   dir=Config_getString("HTML_OUTPUT");
@@ -110,12 +279,6 @@ HtmlGenerator::~HtmlGenerator()
   //printf("HtmlGenerator::~HtmlGenerator()\n");
 }
 
-void HtmlGenerator::append(const OutputGenerator *g)
-{
-  t << g->getContents();
-  col+=((HtmlGenerator *)g)->col;
-}
-
 void HtmlGenerator::init()
 {
   QCString dname=Config_getString("HTML_OUTPUT");
@@ -126,8 +289,17 @@ void HtmlGenerator::init()
     exit(1);
   }
   writeLogo(dname);
-  if (!Config_getString("HTML_HEADER").isEmpty()) g_header=fileToString(Config_getString("HTML_HEADER"));
-  if (!Config_getString("HTML_FOOTER").isEmpty()) g_footer=fileToString(Config_getString("HTML_FOOTER"));
+  if (!Config_getString("HTML_HEADER").isEmpty()) 
+  {
+    g_header=fileToString(Config_getString("HTML_HEADER"));
+    //printf("g_header='%s'\n",g_header.data());
+  }
+  if (!Config_getString("HTML_FOOTER").isEmpty()) 
+  {
+    g_footer=fileToString(Config_getString("HTML_FOOTER"));
+    //printf("g_footer='%s'\n",g_footer.data());
+  }
+  createSubDirs(d);
 }
 
 void HtmlGenerator::writeStyleSheetFile(QFile &file)
@@ -137,23 +309,23 @@ void HtmlGenerator::writeStyleSheetFile(QFile &file)
 }
 
 static void writeDefaultHeaderFile(QTextStream &t, const char *title,
-                                   bool external)
+                                   const char *relPath,bool usePathCmd)
 {
-  t << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n"
-    "<html><head>" /*"<meta name=\"robots\" content=\"noindex\">\n"*/
+  t << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"
+    "<html><head>" 
     "<meta http-equiv=\"Content-Type\" content=\"text/html;charset=" 
          << theTranslator->idLanguageCharset() << "\">\n"
     "<title>"; 
   t << convertToHtml(title);
   t << "</title>\n";
   t << "<link ";
-  if (external) 
-    t << "doxygen=\"_doc:" << Config_getString("DOC_URL") 
-      << "\" href=\"" << Config_getString("DOC_URL") << "/";
-  else
-    t << "href=\"";
+  t << "href=\"";
   if (Config_getString("HTML_STYLESHEET").isEmpty())
   {
+    if (usePathCmd) 
+      t << "$relpath$";
+    else
+      t << relPath;
     t << "doxygen.css";
   }
   else
@@ -177,21 +349,18 @@ void HtmlGenerator::writeHeaderFile(QFile &file)
 #if QT_VERSION >= 200
   t.setEncoding(QTextStream::Latin1);
 #endif
-  writeDefaultHeaderFile(t,"$title",FALSE);
+  writeDefaultHeaderFile(t,"$title",relativePathToRoot(0),TRUE);
 }
 
 void HtmlGenerator::writeFooterFile(QFile &file)
 {
   QTextStream t(&file);
-  t << "<hr><address style=\"align: right;\"><small>\n";
+  t << "<hr size=\"1\"><address style=\"align: right;\"><small>\n";
   t << theTranslator->trGeneratedAt( "$datetime", "$projectname" );
-  t << " <a href=\"http://www.doxygen.org/index.html\">\n"
+  t << " <a href=\"http://www.doxygen.org/index.html\">"
     << "<img src=\"doxygen.png\" alt=\"doxygen\" " 
-    << "align=\"middle\" border=0 width=110 height=53>\n"
+    << "align=\"middle\" border=0>"
     << "</a> $doxygenversion";
-  //  << " " << theTranslator->trWrittenBy()
-  //  << " <a href=\"mailto:dimitri@stack.nl\">Dimitri van Heesch</a>,\n"
-  //  << " &copy;&nbsp;1997-2002"
   t << "</small></address>\n"
     << "</body>\n"
     << "</html>\n";
@@ -199,11 +368,13 @@ void HtmlGenerator::writeFooterFile(QFile &file)
 
 
 void HtmlGenerator::startFile(const char *name,const char *,
-                              const char *title,bool external)
+                              const char *title)
 {
   //printf("HtmlGenerator::startFile(%s)\n",name);
   QCString fileName=name;
   lastTitle=title;
+  relPath = relativePathToRoot(fileName);
+
   if (fileName.right(Doxygen::htmlFileExtension.length())!=Doxygen::htmlFileExtension) 
   {
     fileName+=Doxygen::htmlFileExtension;
@@ -213,87 +384,91 @@ void HtmlGenerator::startFile(const char *name,const char *,
   {
     HtmlHelp::getInstance()->addIndexFile(fileName);
   }
+  
+  QCString dispTitle = title;
+  QCString projName = Config_getString("PROJECT_NAME");
+  if (!projName.isEmpty())
+  {
+    dispTitle.prepend(projName+": ");
+  }
  
   lastFile = fileName;
   if (g_header.isEmpty()) 
   {
-    writeDefaultHeaderFile(t,title,external);
+    writeDefaultHeaderFile(t,dispTitle,relPath,FALSE);
   }
   else
   {
-    t << substituteKeywords(g_header,convertToHtml(lastTitle));
+    t << substituteKeywords(g_header,convertToHtml(lastTitle),relPath);
   }
   t << "<!-- " << theTranslator->trGeneratedBy() << " Doxygen " 
     << versionString << " -->" << endl;
 }
 
-void HtmlGenerator::startQuickIndexItem(const char *s,const char *l)
+//void HtmlGenerator::startQuickIndexItem(const char *s,const char *l)
+//{
+//  QCString *dest;
+//  if (s) 
+//  {
+//    t << "<a class=\"qindexRef\" ";
+//    t << "doxygen=\"" << s << ":";
+//    if ((dest=Doxygen::tagDestinationDict[s])) t << *dest;
+//    if (strcmp(s,"_cgi")!=0) t << "/"; // small hack to get the cgi binary link right
+//    t << "\" ";
+//  }
+//  else
+//  {
+//    t << "<a class=\"qindex\" ";
+//  }
+//  t << "href=\""; 
+//  if (s)
+//  {
+//    if ((dest=Doxygen::tagDestinationDict[s])) t << *dest;
+//    if (strcmp(s,"_cgi")!=0) t << "/";
+//  }
+//  t << l << "\">";
+//}
+//
+//void HtmlGenerator::endQuickIndexItem()
+//{
+//  t << "</a> &nbsp; ";
+//}
+
+static void writePageFooter(QTextStream &t,const QCString &lastTitle,
+                            const QCString relPath)
 {
-  QCString *dest;
-  if (s) 
+  if (g_footer.isEmpty())
   {
-    t << "<a class=\"qindexRef\" ";
-    t << "doxygen=\"" << s << ":";
-    if ((dest=Doxygen::tagDestinationDict[s])) t << *dest;
-    if (strcmp(s,"_cgi")!=0) t << "/"; // small hack to get the cgi binary link right
-    t << "\" ";
+    t << "<hr size=\"1\"><address style=\"align: right;\"><small>";
+    t << theTranslator->trGeneratedAt(
+        dateToString(TRUE),
+        Config_getString("PROJECT_NAME")
+        );
+    t << endl << "<a href=\"http://www.doxygen.org/index.html\">";
+    t << endl << "<img src=\"" << relPath << "doxygen.png\" alt=\"doxygen\" " 
+      << "align=\"middle\" border=0 >" << "</a> " << versionString << " ";
+    t << "</small></address>";
+    if (Debug::isFlagSet(Debug::Validate))
+    {
+      t << "<p><a href=\"http://validator.w3.org/check/referer\">"
+           "<img border=\"0\" src=\"http://www.w3.org/Icons/valid-html401\""
+           " height=\"31\" width=\"88\" alt=\"This page is Valid HTML 4.01 "
+           "Transitional!\"></a><a href=\"http://jigsaw.w3.org/css-validator/\">"
+           "<img style=\"border:0;width:88px;height:31px\" "
+           "src=\"http://jigsaw.w3.org/css-validator/images/vcss\" "
+           "alt=\"This page uses valid CSS!\"></a></p>";
+    }
+    t << "\n</body>\n</html>\n";
   }
   else
   {
-    t << "<a class=\"qindex\" ";
+    t << substituteKeywords(g_footer,convertToHtml(lastTitle),relPath);
   }
-  t << "href=\""; 
-  if (s)
-  {
-    if ((dest=Doxygen::tagDestinationDict[s])) t << *dest;
-    if (strcmp(s,"_cgi")!=0) t << "/";
-  }
-  t << l << "\">";
 }
 
-void HtmlGenerator::endQuickIndexItem()
+void HtmlGenerator::writeFooter()
 {
-  t << "</a> &nbsp; ";
-}
-
-void HtmlGenerator::writeFooter(int part,bool external)
-{
-  switch (part)
-  {
-    case 0:
-      if (g_footer.isEmpty())
-        t << "<hr><address style=\"align: right;\"><small>";
-      else
-        t << substituteKeywords(g_footer,convertToHtml(lastTitle));
-      break;
-    case 1:
-      if (g_footer.isEmpty())
-      {
-        t << endl << "<a href=\"http://www.doxygen.org/index.html\">";
-        t << endl << "<img ";
-        if (external)
-        {
-          t << "doxygen=\"_doc:" << Config_getString("DOC_URL") 
-            << "\" src=\"" << Config_getString("DOC_URL") << "/";
-        }
-        else
-        {
-          t << "src=\"";
-        }
-        t << "doxygen.png\" alt=\"doxygen\" " 
-          << "align=\"middle\" border=0 " << endl << 
-             "width=110 height=53></a>" << versionString <<" ";
-      }
-      break;
-    default:
-      if (g_footer.isEmpty())
-      {
-        //t << " <a href=\"mailto:dimitri@stack.nl\">Dimitri van Heesch</a>,\n &copy;&nbsp;1997-2002";
-        t << "</small></address>\n</body>\n</html>\n";
-      }
-      break;
-      
-  }
+  writePageFooter(t,lastTitle,relPath);
 }
 
 void HtmlGenerator::endFile()
@@ -313,10 +488,12 @@ void HtmlGenerator::endProjectNumber()
 
 void HtmlGenerator::writeStyleInfo(int part)
 {
+  //printf("writeStyleInfo(%d)\n",part);
   if (part==0)
   {
     if (Config_getString("HTML_STYLESHEET").isEmpty()) // write default style sheet
     {
+      //printf("write doxygen.css\n");
       startPlainFile("doxygen.css"); 
       
       // alternative, cooler looking titles
@@ -336,8 +513,11 @@ void HtmlGenerator::writeStyleInfo(int part)
       }
       else
       {
+        // convert style sheet to string
+        QCString fileStr = fileToString(cssname);
+        // write the string into the output dir
         startPlainFile(cssfi.fileName());
-        t << fileToString(cssname);
+        t << fileStr;
         endPlainFile();
       }
     }
@@ -347,7 +527,7 @@ void HtmlGenerator::writeStyleInfo(int part)
 void HtmlGenerator::startDoxyAnchor(const char *,const char *,
                                     const char *anchor, const char *name)
 {
-  t << "<a name=\"" << anchor << "\" doxytag=\"" << name << "\"></a>";
+  t << "<a class=\"anchor\" name=\"" << anchor << "\" doxytag=\"" << name << "\" ></a>";
 }
 
 void HtmlGenerator::endDoxyAnchor(const char *,const char *)
@@ -403,10 +583,6 @@ void HtmlGenerator::writeIndexItem(const char *ref,const char *f,
   {
     t << "</b>";
   }
-  //if (Config_getBool("GENERATE_HTMLHELP") && f)
-  //{
-  //  htmlHelp->addItem(name,((QCString)f)+htmlFileExtension);
-  //}
 }
 
 void HtmlGenerator::writeStartAnnoItem(const char *,const char *f,
@@ -417,10 +593,6 @@ void HtmlGenerator::writeStartAnnoItem(const char *,const char *f,
   t << "<a class=\"el\" href=\"" << f << Doxygen::htmlFileExtension << "\">";
   docify(name);
   t << "</a> ";
-  //if (Config_getBool("GENERATE_HTMLHELP") && f)
-  //{
-  //  htmlHelp->addItem(name, ((QCString)f)+htmlFileExtension);
-  //}
 }
 
 void HtmlGenerator::writeObjectLink(const char *ref,const char *f,
@@ -441,7 +613,11 @@ void HtmlGenerator::writeObjectLink(const char *ref,const char *f,
   t << "href=\"";
   if (ref)
   {
-    if ((dest=Doxygen::tagDestinationDict[ref])) t << *dest << "/";
+    if ((dest=Doxygen::tagDestinationDict[ref])) t << relPath << *dest << "/";
+  }
+  else
+  {
+    t << relPath;
   }
   if (f) t << f << Doxygen::htmlFileExtension;
   if (anchor) t << "#" << anchor;
@@ -468,7 +644,11 @@ void HtmlGenerator::writeCodeLink(const char *ref,const char *f,
   t << "href=\"";
   if (ref)
   {
-    if ((dest=Doxygen::tagDestinationDict[ref])) t << *dest << "/";
+    if ((dest=Doxygen::tagDestinationDict[ref])) t << relPath << *dest << "/";
+  }
+  else
+  {
+    t << relPath;
   }
   if (f) t << f << Doxygen::htmlFileExtension;
   if (anchor) t << "#" << anchor;
@@ -481,7 +661,7 @@ void HtmlGenerator::writeCodeLink(const char *ref,const char *f,
 void HtmlGenerator::startTextLink(const char *f,const char *anchor)
 {
   t << "<a href=\"";
-  if (f)   t << f << Doxygen::htmlFileExtension;
+  if (f)   t << relPath << f << Doxygen::htmlFileExtension;
   if (anchor) t << "#" << anchor;
   t << "\">"; 
 }
@@ -505,13 +685,6 @@ void HtmlGenerator::endHtmlLink()
   t << "</a>";
 }
 
-void HtmlGenerator::writeMailLink(const char *url)
-{
-  t << "<a href=\"mailto:" << url << "\">";
-  docify(url); 
-  t << "</a>";
-}
-
 void HtmlGenerator::startGroupHeader()
 {
   t << "<h2>";
@@ -533,12 +706,12 @@ void HtmlGenerator::startSection(const char *lab,const char *,SectionInfo::Secti
     case SectionInfo::Paragraph:     t << "<h5>"; break;
     default: ASSERT(0); break;
   }
-  t << "<a name=\"" << lab << "\">";
+  t << "<a class=\"anchor\" name=\"" << lab << "\">";
 }
 
 void HtmlGenerator::endSection(const char *,SectionInfo::SectionType type)
 {
-  t << "</a>" << endl;
+  t << "</a>";
   switch(type)
   {
     case SectionInfo::Page:          t << "</h1>"; break;
@@ -550,46 +723,6 @@ void HtmlGenerator::endSection(const char *,SectionInfo::SectionType type)
   }
 }
 
-void HtmlGenerator::writeSectionRef(const char *ref,const char *name,
-                                    const char *anchor,const char *title)
-{
-  QCString *dest;
-  //printf("writeSectionRef(%s,%s,%s,%s)\n",ref,name,anchor,title);
-  QCString refName=name;
-  if (refName.right(Doxygen::htmlFileExtension.length())!=Doxygen::htmlFileExtension) 
-  {
-    refName+=Doxygen::htmlFileExtension;
-  }
-  t << "<a "; 
-  if (ref) 
-  {
-    t << "doxygen=\"" << ref << ":";
-    if ((dest=Doxygen::tagDestinationDict[ref])) t << *dest << "/";
-    t << "\" ";
-  }
-  t << "href=\""; 
-  if (ref)
-  {
-    if ((dest=Doxygen::tagDestinationDict[ref])) t << *dest << "/";
-  }
-  t << refName << "#" << anchor << "\">";
-  docify(title);
-  t << "</a>";
-}
-
-void HtmlGenerator::writeSectionRefItem(const char *name,const char *lab,
-                                    const char *title)
-{
-  QCString refName=name;
-  if (refName.right(Doxygen::htmlFileExtension.length())!=Doxygen::htmlFileExtension) 
-  {
-    refName+=Doxygen::htmlFileExtension;
-  }
-  t << "<li><a href=\"" << refName << "#" << lab << "\">";
-  docify(title);
-  t << "</a>";
-}
-
 void HtmlGenerator::docify(const char *str)
 {
   if (str)
@@ -687,32 +820,6 @@ void HtmlGenerator::endClassDiagram(ClassDiagram &d,
   d.writeImage(t,dir,fileName);
 }
 
-//void HtmlGenerator::startColorFont(uchar red,uchar green,uchar blue)
-//{
-//  QCString colorString;
-//  colorString.sprintf("%02x%02x%02x",red,green,blue);
-//  t << "<font color=\"#" << colorString << "\">";
-//}
-//
-//void HtmlGenerator::endColorFont()
-//{
-//  t << "</font>";
-//}
-
-void HtmlGenerator::writeFormula(const char *n,const char *text)
-{
-  if (text && text[0]=='\\') t << "<p><center>" << endl;
-  t << "<img align=";
-#if !defined(_WIN32)
-  t << "\"top\"";            // assume Unix users use Netscape 4.x which does
-                             // not seem to support align == "middle" :-((
-#else
-  t << "\"middle\"";         // assume Windows users use IE or HtmlHelp which only
-                             // displays formulas nicely with align == "middle" 
-#endif
-  t << " src=\"" << n << ".png\">" << endl;
-  if (text && text[0]=='\\') t << "</center><p>" << endl;
-}
 
 void HtmlGenerator::startMemberList()  
 { 
@@ -750,9 +857,9 @@ void HtmlGenerator::startMemberItem(int annoType)
     t << "<tr>";
     switch(annoType)
     {
-      case 0:  t << "<td nowrap align=right valign=top>"; break;
-      case 1:  t << "<td nowrap>"; break;
-      default: t << "<td nowrap valign=top>"; break;
+      case 0:  t << "<td class=\"memItemLeft\" nowrap align=right valign=top>"; break;
+      case 1:  t << "<td class=\"memItemLeft\" nowrap>"; break;
+      default: t << "<td class=\"memItemLeft\" nowrap valign=top>"; break;
     }
   }
   else
@@ -761,12 +868,12 @@ void HtmlGenerator::startMemberItem(int annoType)
   }
 }
 
-void HtmlGenerator::endMemberItem(bool) 
+void HtmlGenerator::endMemberItem() 
 { 
   //DBG_HTML(t << "<!-- endMemberItem(" << (int)inGroup << "," << fileName << "," << headerName << " -->" << endl)
   if (Config_getBool("HTML_ALIGN_MEMBERS"))
   {
-    t << "</td></tr>"; 
+    t << "</td></tr>\n"; 
   }
   t << endl; 
 }
@@ -777,7 +884,7 @@ void HtmlGenerator::insertMemberAlign()
   DBG_HTML(t << "<!-- insertMemberAlign -->" << endl)
   if (Config_getBool("HTML_ALIGN_MEMBERS"))
   {
-    t << "&nbsp;</td><td valign=bottom>"; 
+    t << "&nbsp;</td><td class=\"memItemRight\" valign=bottom>"; 
   }
 }
 
@@ -786,11 +893,11 @@ void HtmlGenerator::startMemberDescription()
   DBG_HTML(t << "<!-- startMemberDescription -->" << endl)
   if (Config_getBool("HTML_ALIGN_MEMBERS"))
   {
-    t << "<tr><td>&nbsp;</td><td><font size=-1><em>"; 
+    t << "<tr><td class=\"mdescLeft\">&nbsp;</td><td class=\"mdescRight\">"; 
   }
   else
   {
-    t << "<dl class=\"el\"><dd><font size=-1><em>";
+    t << "<dl class=\"el\"><dd class=\"mdescRight\">";
   }
 }
 
@@ -799,11 +906,11 @@ void HtmlGenerator::endMemberDescription()
   DBG_HTML(t << "<!-- endMemberDescription -->" << endl)
   if (Config_getBool("HTML_ALIGN_MEMBERS"))
   {
-    t << "</em></font><br><br></td></tr>" << endl; 
+    t << "<br><br></td></tr>" << endl; 
   }
   else
   {
-    t << "<br><br></em></font></dl>";
+    t << "<br><br></dl>";
   }
 }
 
@@ -897,83 +1004,13 @@ void HtmlGenerator::endIndexValue(const char *,bool)
   t << "</td></tr>" << endl;
 }
 
-void HtmlGenerator::startAlphabeticalIndexList()
-{
-  t << "<table align=center width=\"95%\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">" << endl;
-}
-
-void HtmlGenerator::endAlphabeticalIndexList()
-{
-  t << "</table>" << endl;
-}
-
-void HtmlGenerator::writeIndexHeading(const char *s)
-{
-  t << "<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\"><tr><td><div class=\"ah\">&nbsp;&nbsp;" << s 
-    << "&nbsp;&nbsp;</td></tr></table>";
-}
-
-void HtmlGenerator::startImage(const char *name,const char *,bool hasCaption)
-{
-  QCString baseName=name;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1 || (i=baseName.findRev('\\'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1); 
-  }
-  t << "<div align=\"center\">" << endl;
-  t << "<img src=\"" << name << "\" alt=\"" << baseName << "\">" << endl;
-  if (hasCaption)
-  {
-    t << "<p><strong>";
-  } 
-}
-
-void HtmlGenerator::endImage(bool hasCaption)
-{
-  if (hasCaption)
-  {
-    t << "</strong></p>" << endl;
-  }
-  t << "</div>" << endl;
-}
-
-void HtmlGenerator::startDotFile(const char *name,bool hasCaption)
-{
-  QCString baseName=name;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1 || (i=baseName.findRev('\\'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1); 
-  }
-  QCString outDir = Config_getString("HTML_OUTPUT");
-  writeDotGraphFromFile(name,outDir,baseName,BITMAP);
-  t << "<div align=\"center\">" << endl;
-  t << "<img src=\"" << baseName << "." 
-    << Config_getEnum("DOT_IMAGE_FORMAT") << "\" alt=\"" 
-    << baseName << "\">" << endl;
-  if (hasCaption)
-  {
-    t << "<p><strong>";
-  } 
-}
-
-void HtmlGenerator::endDotFile(bool hasCaption)
-{
-  if (hasCaption)
-  {
-    t << "</strong></p>" << endl;
-  }
-  t << "</div>" << endl;
-}
-
 void HtmlGenerator::startMemberDoc(const char *,const char *,const char *,const char *) 
 { 
   DBG_HTML(t << "<!-- startMemberDoc -->" << endl;)
   t << "<p>" << endl;
-  t << "<table width=\"100%\" cellpadding=\"2\" cellspacing=\"0\" border=\"0\">" << endl;
+  t << "<table class=\"mdTable\" width=\"100%\" cellpadding=\"2\" cellspacing=\"0\">" << endl;
   t << "  <tr>" << endl;
-  t << "    <td class=\"md\">" << endl; 
+  t << "    <td class=\"mdRow\">" << endl; 
   t << "      <table cellpadding=\"0\" cellspacing=\"0\" border=\"0\">" << endl;
 }
 
@@ -991,26 +1028,33 @@ void HtmlGenerator::endMemberDocPrefixItem()
   t << "        </tr>" << endl;
 }
 
-void HtmlGenerator::startMemberDocName()
+void HtmlGenerator::startMemberDocName(bool align)
 {
   DBG_HTML(t << "<!-- startMemberDocName -->" << endl;)
   t << "        <tr>" << endl;
-  t << "          <td class=\"md\" nowrap valign=\"top\"> ";
+  t << "          <td class=\"md\" nowrap valign=\"top\"";
+  if (align) t << " align=\"right\"";
+  t << "> ";
 }
 
 void HtmlGenerator::endMemberDocName()
 {
   DBG_HTML(t << "<!-- endMemberDocName -->" << endl;)
+  t << "          </td>" << endl;
+}
+
+void HtmlGenerator::startParameterList(bool openBracket)
+{
+  DBG_HTML(t << "<!-- startParameterList -->" << endl;)
+  t << "          <td class=\"md\" valign=\"top\">";
+  if (openBracket)
+  {
+    t << "(&nbsp;";
+  }
   t << "</td>" << endl;
 }
 
-void HtmlGenerator::startParameterList()
-{
-  DBG_HTML(t << "<!-- startParameterList -->" << endl;)
-  t << "          <td class=\"md\" valign=\"top\">(&nbsp;</td>" << endl;
-}
-
-void HtmlGenerator::startParameterType(bool first)
+void HtmlGenerator::startParameterType(bool first,const char *key)
 {
   if (first)
   {
@@ -1021,7 +1065,9 @@ void HtmlGenerator::startParameterType(bool first)
   {
     DBG_HTML(t << "<!-- startParameterType -->" << endl;)
     t << "        <tr>" << endl;
-    t << "          <td></td>" << endl;
+    t << "          <td class=\"md\" nowrap align=\"right\">";
+    if (key) t << key;
+    t << "</td>" << endl;
     t << "          <td></td>" << endl;
     t << "          <td class=\"md\" nowrap>";
   }
@@ -1041,10 +1087,10 @@ void HtmlGenerator::startParameterName(bool oneArgOnly)
   {
     t << "1\" valign=\"top";
   }
-  t << "\" nowrap>&nbsp;";
+  t << "\" nowrap>"; //&nbsp;";
 }
 
-void HtmlGenerator::endParameterName(bool last,bool emptyList)
+void HtmlGenerator::endParameterName(bool last,bool emptyList,bool closeBracket)
 {
   DBG_HTML(t << "<!-- endParameterName -->" << endl;)
   if (last)
@@ -1052,7 +1098,9 @@ void HtmlGenerator::endParameterName(bool last,bool emptyList)
     if (emptyList)
     {
       t << "          </td>" << endl;
-      t << "          <td class=\"md\" valign=\"top\">)&nbsp;</td>" << endl;
+      t << "          <td class=\"md\" valign=\"top\">";
+      if (closeBracket) t << "&nbsp;)";
+      t << "&nbsp;</td>" << endl;
       t << "          <td class=\"md\" nowrap>";
     }
     else
@@ -1061,7 +1109,9 @@ void HtmlGenerator::endParameterName(bool last,bool emptyList)
       t << "        </tr>" << endl;
       t << "        <tr>" << endl;
       t << "          <td></td>" << endl;
-      t << "          <td class=\"md\">)&nbsp;</td>" << endl;
+      t << "          <td class=\"md\">";
+      if (closeBracket) t << ")";
+      t << "&nbsp;</td>" << endl;
       t << "          <td class=\"md\" colspan=\"2\">";
     }
   }
@@ -1079,10 +1129,13 @@ void HtmlGenerator::endParameterList()
   t << "        </tr>" << endl;
 }
 
-void HtmlGenerator::endMemberDoc()     
+void HtmlGenerator::endMemberDoc(bool hasArgs)     
 { 
   DBG_HTML(t << "<!-- endMemberDoc -->" << endl;)
-  t << endl;
+  if (!hasArgs)
+  {
+    t << "        </tr>" << endl;
+  }
   t << "      </table>" << endl;
   t << "    </td>" << endl;
   t << "  </tr>" << endl;
@@ -1095,7 +1148,7 @@ void HtmlGenerator::startDotGraph()
 
 void HtmlGenerator::endDotGraph(DotClassGraph &g)
 {
-  g.writeGraph(t,BITMAP,Config_getString("HTML_OUTPUT"));
+  g.writeGraph(t,BITMAP,dir,relPath);
 }
 
 void HtmlGenerator::startInclDepGraph()
@@ -1104,12 +1157,21 @@ void HtmlGenerator::startInclDepGraph()
 
 void HtmlGenerator::endInclDepGraph(DotInclDepGraph &g)
 {
-  g.writeGraph(t,BITMAP,Config_getString("HTML_OUTPUT"));
+  g.writeGraph(t,BITMAP,dir,relPath);
+}
+
+void HtmlGenerator::startCallGraph()
+{
+}
+
+void HtmlGenerator::endCallGraph(DotCallGraph &g)
+{
+  g.writeGraph(t,BITMAP,dir,relPath);
 }
 
 void HtmlGenerator::writeGraphicalHierarchy(DotGfxHierarchyTable &g)
 {
-  g.writeGraph(t,Config_getString("HTML_OUTPUT"));
+  g.writeGraph(t,dir);
 }
 
 void HtmlGenerator::startMemberGroupHeader(bool)
@@ -1232,19 +1294,6 @@ void HtmlGenerator::endParamList()
   t << "</dl>"; 
 }
 
-void HtmlGenerator::startSectionRefList()
-{
-  t << "<multicol cols=3>" << endl;
-  t << "<ul>" << endl;
-}
-
-void HtmlGenerator::endSectionRefList()
-{
-  t << "</ul>" << endl;
-  t << "</multicol>" << endl;
-}
-
-
 void HtmlGenerator::printDoc(DocNode *n)
 {
   HtmlDocVisitor *visitor = new HtmlDocVisitor(t,*this);
@@ -1252,3 +1301,248 @@ void HtmlGenerator::printDoc(DocNode *n)
   delete visitor; 
 }
 
+static void startQuickIndexItem(QTextStream &t,const char *l,
+                                bool hl,bool compact,bool &first,
+                                const QCString &relPath)
+{
+  if (!first && compact) t << " | ";
+  first=FALSE;
+  if (!compact) t << "<li>";
+  if (hl && compact)
+  {
+    t << "<a class=\"qindexHL\" ";
+  }
+  else
+  {
+    t << "<a class=\"qindex\" ";
+  }
+  t << "href=\"" << relPath << l << "\">";
+}
+
+static void endQuickIndexItem(QTextStream &t)
+{
+  t << "</a>";
+}
+
+static QCString fixSpaces(const QCString &s)
+{
+  return substitute(s," ","&nbsp;");
+}
+
+static void writeDefaultQuickLinks(QTextStream &t,bool compact,
+                                   HighlightedItem hli,const QCString &relPath)
+{
+  bool first=TRUE;
+  if (compact) 
+  {
+    t << "<div class=\"qindex\">"; 
+  }
+  else 
+  {
+    t << "<ul>";
+  }
+
+  if (Config_getBool("SEARCHENGINE"))
+  {
+    t << "  <form class=\"search\" action=\"" << relPath 
+      << "search.php\" method=\"get\">\n";
+  }
+
+  if (Config_getBool("GENERATE_TREEVIEW"))
+  {
+    startQuickIndexItem(t,"main"+Doxygen::htmlFileExtension,
+                        hli==HLI_Main,compact,first,relPath);
+  }
+  else
+  {
+    startQuickIndexItem(t,"index"+Doxygen::htmlFileExtension,
+                        hli==HLI_Main,compact,first,relPath);
+  }
+  t << fixSpaces(theTranslator->trMainPage());
+  endQuickIndexItem(t);
+
+  if (documentedGroups>0)
+  {
+    startQuickIndexItem(t,"modules"+Doxygen::htmlFileExtension,
+                        hli==HLI_Modules,compact,first,relPath);
+    t << fixSpaces(theTranslator->trModules());
+    endQuickIndexItem(t);
+  } 
+  if (documentedNamespaces>0)
+  {
+    startQuickIndexItem(t,"namespaces"+Doxygen::htmlFileExtension,
+                        hli==HLI_Namespaces,compact,first,relPath);
+    if (Config_getBool("OPTIMIZE_OUTPUT_JAVA"))
+    {
+      t << fixSpaces(theTranslator->trPackages());
+    }
+    else
+    {
+      t << theTranslator->trNamespaceList();
+    }
+    endQuickIndexItem(t);
+  }
+  if (hierarchyClasses>0)
+  {
+    startQuickIndexItem(t,"hierarchy"+Doxygen::htmlFileExtension,
+                           hli==HLI_Hierarchy,compact,first,relPath);
+    t << fixSpaces(theTranslator->trClassHierarchy());
+    endQuickIndexItem(t);
+  } 
+  if (annotatedClasses>0)
+  {
+    if (Config_getBool("ALPHABETICAL_INDEX"))
+    {
+      startQuickIndexItem(t,"classes"+Doxygen::htmlFileExtension,
+                          hli==HLI_Classes,compact,first,relPath);
+      t << fixSpaces(theTranslator->trAlphabeticalList());
+      endQuickIndexItem(t);
+    }
+    startQuickIndexItem(t,"annotated"+Doxygen::htmlFileExtension,
+                        hli==HLI_Annotated,compact,first,relPath);
+    t << fixSpaces(theTranslator->trCompoundList());
+    endQuickIndexItem(t);
+  } 
+  if (documentedHtmlFiles>0)
+  {
+    startQuickIndexItem(t,"files"+Doxygen::htmlFileExtension,
+                        hli==HLI_Files,compact,first,relPath);
+    t << fixSpaces(theTranslator->trFileList());
+    endQuickIndexItem(t);
+  } 
+  if (documentedNamespaceMembers[NMHL_All]>0)
+  {
+    startQuickIndexItem(t,"namespacemembers"+Doxygen::htmlFileExtension,
+                        hli==HLI_NamespaceMembers,compact,first,relPath);
+    t << fixSpaces(theTranslator->trNamespaceMembers());
+    endQuickIndexItem(t);
+  }
+  if (documentedClassMembers[CMHL_All]>0)
+  {
+    startQuickIndexItem(t,"functions"+Doxygen::htmlFileExtension,
+                        hli==HLI_Functions,compact,first,relPath);
+    t << fixSpaces(theTranslator->trCompoundMembers());
+    endQuickIndexItem(t);
+  } 
+  if (documentedFileMembers[FMHL_All]>0)
+  {
+    startQuickIndexItem(t,"globals"+Doxygen::htmlFileExtension,
+                        hli==HLI_Globals,compact,first,relPath);
+    t << fixSpaces(theTranslator->trFileMembers());
+    endQuickIndexItem(t);
+  } 
+  if (indexedPages>0)
+  {
+    startQuickIndexItem(t,"pages"+Doxygen::htmlFileExtension,
+                        hli==HLI_Pages,compact,first,relPath);
+    t << fixSpaces(theTranslator->trRelatedPages());
+    endQuickIndexItem(t);
+  } 
+  if (Doxygen::exampleSDict->count()>0)
+  {
+    startQuickIndexItem(t,"examples"+Doxygen::htmlFileExtension,
+                        hli==HLI_Examples,compact,first,relPath);
+    t << fixSpaces(theTranslator->trExamples());
+    endQuickIndexItem(t);
+  } 
+  if (Config_getBool("SEARCHENGINE"))
+  {
+    if (compact)
+    {
+      t << "  | ";
+    }
+    else
+    {
+      t << "<li>";
+    }
+    t << "<span class=\"search";
+    if (hli==HLI_Search) t << "HL";
+    QCString searchFor = fixSpaces(theTranslator->trSearchForIndex());
+    if (searchFor.at(0)=='S') searchFor="<u>S</u>"+searchFor.mid(1);
+    t << "\">" << searchFor << "&nbsp;";
+    if (hli!=HLI_Search)
+    {
+      t << "<input class=\"search\" type=\"text\" name=\"query\" value=\"\" size=\"20\" accesskey=\"s\"/>"
+           "</span></form>";
+    }
+  } 
+  if (hli!=HLI_Search)
+  {
+    if (compact) 
+    {
+      t << "</div>\n";
+    }
+    else 
+    {
+      t << "</ul>\n";
+    }
+  }
+}
+
+void HtmlGenerator::writeQuickLinks(bool compact,HighlightedItem hli)
+{
+  writeDefaultQuickLinks(t,compact,hli,relPath);
+}
+
+void HtmlGenerator::writeSearchPage()
+{
+  if (Config_getBool("SEARCHENGINE") && Config_getBool("GENERATE_HTML"))
+  {
+    QCString fileName = Config_getString("HTML_OUTPUT")+"/search.php";
+    QFile f(fileName);
+    if (f.open(IO_WriteOnly))
+    {
+      QTextStream t(&f);
+#if QT_VERSION >= 200
+      t.setEncoding(QTextStream::Latin1);
+#endif
+      if (g_header.isEmpty()) 
+      {
+        writeDefaultHeaderFile(t,"Search",0,FALSE);
+      }
+      else
+      {
+        t << substituteKeywords(g_header,"Search","");
+      }
+      t << "<!-- " << theTranslator->trGeneratedBy() << " Doxygen " 
+        << versionString << " -->" << endl;
+      if (!Config_getBool("DISABLE_INDEX")) 
+      { 
+        writeDefaultQuickLinks(t,TRUE,HLI_Search,"");
+      }
+
+      t << "\n<?php\n\n";
+      t << "function search_results()\n";
+      t << "{\n";
+      t << "  return \"" << theTranslator->trSearchResultsTitle() << "\";\n";
+      t << "}\n";
+      t << "\n";
+      t << "function matches_text($num)\n";
+      t << "{\n";
+      t << "  if ($num==0)\n";
+      t << "  {\n";
+      t << "    return \"" << theTranslator->trSearchResults(0) << "\";\n";
+      t << "  }\n";
+      t << "  else if ($num==1)\n";
+      t << "  {\n";
+      t << "    return \"" << theTranslator->trSearchResults(1) << "\";\n";
+      t << "  }\n";
+      t << "  else // $num>1\n";
+      t << "  {\n";
+      t << "    return \"" << theTranslator->trSearchResults(2) << "\";\n";
+      t << "  }\n";
+      t << "}\n";
+      t << "\n";
+      t << "function report_matches()\n";
+      t << "{\n";
+      t << " return \"" << theTranslator->trSearchMatches() << " \";\n";
+      t << "}\n";
+      t << "\n";
+      t << search_script;
+      t << "\n";
+      t << "?>\n";
+      writePageFooter(t,"Search","");
+    }
+  }
+}
+
