@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -43,11 +43,14 @@
 #ifdef LANG_JP
 #include "translator_jp.h"
 #endif
+#ifdef LANG_JE
+#include "translator_je.h"
+#endif
 #ifdef LANG_ES
 #include "translator_es.h"
 #endif
 #ifdef LANG_FI
-#include "translator_fi.h"
+//#include "translator_fi.h"
 #endif
 #ifdef LANG_RU
 #include "translator_ru.h"
@@ -64,6 +67,9 @@
 #ifdef LANG_HU
 #include "translator_hu.h"
 #endif
+#ifdef LANG_KE
+#include "translator_ke.h"
+#endif
 #ifdef LANG_KR
 #include "translator_kr.h"
 #endif
@@ -112,6 +118,16 @@
 
 Translator *theTranslator=0;
 
+static const char obsoleteMsg[] =
+        "---------\n"
+        "ERROR: The selected language is no longer supported!\n"
+        "If you want doxygen to produce output in this language \n"
+        "you are kindly requested to help bringing the documentation \n"
+        "up to date. Please read the development section of the manual \n"
+        "for more information or contact Petr Prikryl (Prikryl@skil.cz).\n"
+        "Thanks in advance!\n"
+        "---------\n";
+
 bool setTranslator(const char *langName)
 {
   if (L_EQUAL("english"))
@@ -174,7 +190,9 @@ bool setTranslator(const char *langName)
 #ifdef LANG_FI
   else if (L_EQUAL("finnish"))
   {
-    theTranslator=new TranslatorFinnish;
+    //theTranslator=new TranslatorFinnish;
+    theTranslator=new TranslatorEnglish;
+    err(obsoleteMsg);
   }
 #endif
 #ifdef LANG_RU
@@ -212,6 +230,10 @@ bool setTranslator(const char *langName)
   {
     theTranslator=new TranslatorKorean;
   }
+  else if (L_EQUAL("korean-en"))
+  {
+    theTranslator=new TranslatorKoreanEn;
+  }
 #endif
 #ifdef LANG_RO
   else if (L_EQUAL("romanian"))
@@ -285,11 +307,7 @@ bool setTranslator(const char *langName)
     theTranslator=new TranslatorCatalan;
   }
 #endif
-//  else if (L_EQUAL("japanese-sjis"))
-//  {
-//    theTranslator=new TranslatorJapaneseSjis;
-//  }
-#endif
+#endif // ENGLISH_ONLY
   else // use the default language (i.e. english)
   {
     theTranslator=new TranslatorEnglish;
