@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -15,7 +15,7 @@
  * input used in their production; they are not affected by this license.
  *
  */
-
+#include <qfileinfo.h> 
 #include "latexdocvisitor.h"
 #include "docparser.h"
 #include "language.h"
@@ -26,9 +26,9 @@
 #include "util.h"
 #include "message.h"
 
-static QCString escapeLabelName(const char *s)
+static QString escapeLabelName(const char *s)
 {
-  QCString result;
+  QString result;
   const char *p=s;
   char c;
   while ((c=*p++))
@@ -44,9 +44,21 @@ static QCString escapeLabelName(const char *s)
   return result;
 }
 
-QCString LatexDocVisitor::escapeMakeIndexChars(const char *s)
+const int maxLevels=5;
+static const char *secLabels[maxLevels] = 
+   { "section","subsection","subsubsection","paragraph","subparagraph" };
+
+static const char *getSectionName(int level)
 {
-  QCString result;
+  int l = level;
+  if (Config_getBool("COMPACT_LATEX")) l++;
+  if (Doxygen::insideMainPage) l--;
+  return secLabels[QMIN(maxLevels-1,l)];
+}
+
+QString LatexDocVisitor::escapeMakeIndexChars(const char *s)
+{
+  QString result;
   const char *p=s;
   char str[2]; str[1]=0;
   char c;
@@ -68,7 +80,7 @@ QCString LatexDocVisitor::escapeMakeIndexChars(const char *s)
 
 
 LatexDocVisitor::LatexDocVisitor(QTextStream &t,BaseCodeDocInterface &ci) 
-  : m_t(t), m_ci(ci), m_insidePre(FALSE), m_hide(FALSE) 
+  : DocVisitor(DocVisitor_Latex), m_t(t), m_ci(ci), m_insidePre(FALSE), m_insideItem(FALSE), m_hide(FALSE) 
 {
 }
 
@@ -87,7 +99,7 @@ void LatexDocVisitor::visit(DocLinkedWord *w)
   if (m_hide) return;
   startLink(w->ref(),w->file(),w->anchor());
   filter(w->word());
-  endLink();
+  endLink(w->ref(),w->file(),w->anchor());
 }
 
 void LatexDocVisitor::visit(DocWhiteSpace *w)
@@ -117,6 +129,8 @@ void LatexDocVisitor::visit(DocSymbol *s)
     case DocSymbol::Hash:    m_t << "\\#"; break;
     case DocSymbol::Percent: m_t << "\\%"; break;
     case DocSymbol::Copy:    m_t << "\\copyright"; break;
+    case DocSymbol::Tm:      m_t << "\\texttrademark"; break;
+    case DocSymbol::Reg:     m_t << "\\textregistered"; break;
     case DocSymbol::Apos:    m_t << "'"; break;
     case DocSymbol::Quot:    m_t << "''"; break;
     case DocSymbol::Uml:     
@@ -143,6 +157,11 @@ void LatexDocVisitor::visit(DocSymbol *s)
                              else                  
                                m_t << "\\^{" << s->letter() << "}"; 
                              break;
+    case DocSymbol::Slash:   if (tolower(s->letter())=='o')
+                               m_t << "\\" << s->letter();
+                             else
+                               m_t << s->letter();
+                             break;
     case DocSymbol::Tilde:   m_t << "\\~{"  << s->letter() << "}"; break;
     case DocSymbol::Szlig:   m_t << "\"s"; break;
     case DocSymbol::Cedil:   m_t << "\\c{" << s->letter() << "}"; break;
@@ -158,9 +177,13 @@ void LatexDocVisitor::visit(DocURL *u)
   if (m_hide) return;
   if (Config_getBool("PDF_HYPERLINKS"))
   {
-    m_t << "\\href{" << u->url() << "}";
+    m_t << "\\href{";
+    if (u->isEmail()) m_t << "mailto:";
+    m_t << u->url() << "}";
   }
-  m_t << "{\\tt " << u->url() << "}";
+  m_t << "{\\tt ";
+  filter(u->url());
+  m_t << "}";
 }
 
 void LatexDocVisitor::visit(DocLineBreak *)
@@ -181,13 +204,13 @@ void LatexDocVisitor::visit(DocStyleChange *s)
   switch (s->style())
   {
     case DocStyleChange::Bold:
-      if (s->enable()) m_t << "{\\bf ";      else m_t << "} ";
+      if (s->enable()) m_t << "{\\bf ";      else m_t << "}";
       break;
     case DocStyleChange::Italic:
-      if (s->enable()) m_t << "{\\em ";     else m_t << "} ";
+      if (s->enable()) m_t << "{\\em ";     else m_t << "\\/}";
       break;
     case DocStyleChange::Code:
-      if (s->enable()) m_t << "{\\tt ";   else m_t << "} ";
+      if (s->enable()) m_t << "{\\tt ";   else m_t << "}";
       break;
     case DocStyleChange::Subscript:
       if (s->enable()) m_t << "$_{\\mbox{";    else m_t << "}}$ ";
@@ -201,6 +224,20 @@ void LatexDocVisitor::visit(DocStyleChange *s)
     case DocStyleChange::Small:
       if (s->enable()) m_t << "\\footnotesize ";  else m_t << "\\normalsize ";
       break;
+    case DocStyleChange::Preformatted:
+      if (s->enable()) 
+      {
+        m_t << "\\small\\begin{alltt}";
+        m_insidePre=TRUE;
+      }
+      else
+      {
+        m_insidePre=FALSE;
+        m_t << "\\end{alltt}\\normalsize " << endl;
+      }
+      break;
+    case DocStyleChange::Div:  /* HTML only */ break;
+    case DocStyleChange::Span: /* HTML only */ break;
   }
 }
 
@@ -209,29 +246,56 @@ void LatexDocVisitor::visit(DocVerbatim *s)
   if (m_hide) return;
   switch(s->type())
   {
-    case DocVerbatim::Code: // fall though
+    case DocVerbatim::Code: 
       m_t << "\n\n\\footnotesize\\begin{verbatim}"; 
-      parseCode(m_ci,s->context(),s->text(),FALSE,0);
-      m_t << "\\end{verbatim}\\normalsize" << endl; 
+      parseCode(m_ci,s->context(),s->text().latin1(),s->isExample(),s->exampleFile());
+      m_t << "\\end{verbatim}\n\\normalsize" << endl; 
       break;
     case DocVerbatim::Verbatim: 
       m_t << "\n\n\\footnotesize\\begin{verbatim}"; 
       m_t << s->text();
-      m_t << "\\end{verbatim}\\normalsize" << endl; 
+      m_t << "\\end{verbatim}\n\\normalsize" << endl; 
       break;
     case DocVerbatim::HtmlOnly: 
+    case DocVerbatim::XmlOnly: 
+    case DocVerbatim::ManOnly: 
       /* nothing */ 
       break;
     case DocVerbatim::LatexOnly: 
       m_t << s->text(); 
       break;
+    case DocVerbatim::Dot: 
+      {
+        static int dotindex = 1;
+        QCString fileName(4096);
+
+        fileName.sprintf("%s%d", 
+            (Config_getString("LATEX_OUTPUT")+"/inline_dotgraph_").data(), 
+            dotindex++
+           );
+        QFile file(fileName);
+        if (!file.open(IO_WriteOnly))
+        {
+          err("Could not open file %s for writing\n",fileName.data());
+        }
+        file.writeBlock( s->text(), s->text().length() );
+        file.close();
+
+        m_t << "\\begin{center}\n";
+        startDotFile(fileName,"","",FALSE);
+        endDotFile(FALSE);
+        m_t << "\\end{center}\n";
+
+        file.remove();
+      }
+      break;
   }
 }
 
 void LatexDocVisitor::visit(DocAnchor *anc)
 {
   if (m_hide) return;
-  m_t << "\\label{" << anc->anchor() << "}" << endl;
+  m_t << "\\label{" << anc->file() << "_" << anc->anchor() << "}" << endl;
   if (!anc->file().isEmpty() && Config_getBool("PDF_HYPERLINKS")) 
   {
     m_t << "\\hypertarget{" << anc->file() << "_" << anc->anchor() 
@@ -244,10 +308,19 @@ void LatexDocVisitor::visit(DocInclude *inc)
   if (m_hide) return;
   switch(inc->type())
   {
+    case DocInclude::IncWithLines:
+      { 
+         m_t << "\n\n\\footnotesize\\begin{verbatim}"; 
+         QFileInfo cfi( inc->file() );
+         FileDef fd( cfi.dirPath(), cfi.fileName() );
+         parseCode(m_ci,inc->context(),inc->text().latin1(),inc->isExample(),inc->exampleFile(), &fd);
+         m_t << "\\end{verbatim}\n\\normalsize" << endl; 
+      }
+      break;    
     case DocInclude::Include: 
       m_t << "\n\n\\footnotesize\\begin{verbatim}"; 
-      parseCode(m_ci,inc->context(),inc->text(),FALSE,0);
-      m_t << "\\end{verbatim}\\normalsize" << endl; 
+      parseCode(m_ci,inc->context(),inc->text().latin1(),inc->isExample(),inc->exampleFile());
+      m_t << "\\end{verbatim}\n\\normalsize" << endl; 
       break;
     case DocInclude::DontInclude: 
       break;
@@ -256,7 +329,7 @@ void LatexDocVisitor::visit(DocInclude *inc)
     case DocInclude::VerbInclude: 
       m_t << "\n\n\\footnotesize\\begin{verbatim}"; 
       m_t << inc->text();
-      m_t << "\\end{verbatim}\\normalsize" << endl; 
+      m_t << "\\end{verbatim}\n\\normalsize" << endl; 
       break;
   }
 }
@@ -267,21 +340,25 @@ void LatexDocVisitor::visit(DocIncOperator *op)
   //    op->type(),op->isFirst(),op->isLast(),op->text().data());
   if (op->isFirst()) 
   {
-    m_t << "\n\n\\footnotesize\\begin{verbatim}"; 
+    if (!m_hide) m_t << "\n\n\\footnotesize\\begin{verbatim}"; 
+    pushEnabled();
     m_hide = TRUE;
   }
   if (op->type()!=DocIncOperator::Skip) 
   {
-    parseCode(m_ci,op->context(),op->text(),FALSE,0);
+    popEnabled();
+    if (!m_hide) parseCode(m_ci,op->context(),op->text().latin1(),op->isExample(),op->exampleFile());
+    pushEnabled();
+    m_hide=TRUE;
   }
   if (op->isLast())  
   {
-    m_hide = FALSE;
-    m_t << "\\end{verbatim}\\normalsize" << endl; 
+    popEnabled();
+    if (!m_hide) m_t << "\\end{verbatim}\n\\normalsize" << endl; 
   }
   else
   {
-    m_t << endl;
+    if (!m_hide) m_t << endl;
   }
 }
 
@@ -293,6 +370,7 @@ void LatexDocVisitor::visit(DocFormula *f)
 
 void LatexDocVisitor::visit(DocIndexEntry *i)
 {
+  if (m_hide) return;
   m_t << "\\index{" << escapeLabelName(i->entry()) << "@{";
   escapeMakeIndexChars(i->entry());
   m_t << "}}";
@@ -304,6 +382,7 @@ void LatexDocVisitor::visit(DocIndexEntry *i)
 
 void LatexDocVisitor::visitPre(DocAutoList *l)
 {
+  if (m_hide) return;
   if (l->isEnumList())
   {
     m_t << "\\begin{enumerate}" << endl;
@@ -316,6 +395,7 @@ void LatexDocVisitor::visitPre(DocAutoList *l)
 
 void LatexDocVisitor::visitPost(DocAutoList *l)
 {
+  if (m_hide) return;
   if (l->isEnumList())
   {
     m_t << "\\end{enumerate}" << endl;
@@ -328,6 +408,7 @@ void LatexDocVisitor::visitPost(DocAutoList *l)
 
 void LatexDocVisitor::visitPre(DocAutoListItem *)
 {
+  if (m_hide) return;
   m_t << "\\item ";
 }
 
@@ -341,6 +422,7 @@ void LatexDocVisitor::visitPre(DocPara *)
 
 void LatexDocVisitor::visitPost(DocPara *p)
 {
+  if (m_hide) return;
   if (!p->isLast() &&            // omit <p> for last paragraph
       !(p->parent() &&           // and for parameter sections
         p->parent()->kind()==DocNode::Kind_ParamSect
@@ -358,50 +440,57 @@ void LatexDocVisitor::visitPost(DocRoot *)
 
 void LatexDocVisitor::visitPre(DocSimpleSect *s)
 {
+  if (m_hide) return;
   m_t << "\\begin{Desc}\n\\item[";
   switch(s->type())
   {
     case DocSimpleSect::See: 
-      m_t << theTranslator->trSeeAlso(); break;
+      filter(theTranslator->trSeeAlso()); break;
     case DocSimpleSect::Return: 
-      m_t << theTranslator->trReturns(); break;
+      filter(theTranslator->trReturns()); break;
     case DocSimpleSect::Author: 
-      m_t << theTranslator->trAuthor(TRUE,TRUE); break;
+      filter(theTranslator->trAuthor(TRUE,TRUE)); break;
     case DocSimpleSect::Authors: 
-      m_t << theTranslator->trAuthor(TRUE,FALSE); break;
+      filter(theTranslator->trAuthor(TRUE,FALSE)); break;
     case DocSimpleSect::Version: 
-      m_t << theTranslator->trVersion(); break;
+      filter(theTranslator->trVersion()); break;
     case DocSimpleSect::Since: 
-      m_t << theTranslator->trSince(); break;
+      filter(theTranslator->trSince()); break;
     case DocSimpleSect::Date: 
-      m_t << theTranslator->trDate(); break;
+      filter(theTranslator->trDate()); break;
     case DocSimpleSect::Note: 
-      m_t << theTranslator->trNote(); break;
+      filter(theTranslator->trNote()); break;
     case DocSimpleSect::Warning:
-      m_t << theTranslator->trWarning(); break;
+      filter(theTranslator->trWarning()); break;
     case DocSimpleSect::Pre:
-      m_t << theTranslator->trPrecondition(); break;
+      filter(theTranslator->trPrecondition()); break;
     case DocSimpleSect::Post:
-      m_t << theTranslator->trPostcondition(); break;
+      filter(theTranslator->trPostcondition()); break;
     case DocSimpleSect::Invar:
-      m_t << theTranslator->trInvariant(); break;
+      filter(theTranslator->trInvariant()); break;
     case DocSimpleSect::Remark:
-      m_t << theTranslator->trRemarks(); break;
+      filter(theTranslator->trRemarks()); break;
     case DocSimpleSect::Attention:
-      m_t << theTranslator->trAttention(); break;
+      filter(theTranslator->trAttention()); break;
     case DocSimpleSect::User: break;
+    case DocSimpleSect::Rcs: break;
     case DocSimpleSect::Unknown:  break;
   }
 
   // special case 1: user defined title
-  if (s->type()!=DocSimpleSect::User)
+  if (s->type()!=DocSimpleSect::User && s->type()!=DocSimpleSect::Rcs)
   {
     m_t << ":]";
   }
+  else
+  {
+    m_insideItem=TRUE;
+  }
 }
 
 void LatexDocVisitor::visitPost(DocSimpleSect *)
 {
+  if (m_hide) return;
   m_t << "\\end{Desc}" << endl;
 }
 
@@ -411,21 +500,26 @@ void LatexDocVisitor::visitPre(DocTitle *)
 
 void LatexDocVisitor::visitPost(DocTitle *)
 {
+  if (m_hide) return;
+  m_insideItem=FALSE;
   m_t << "]";
 }
 
 void LatexDocVisitor::visitPre(DocSimpleList *)
 {
+  if (m_hide) return;
   m_t << "\\begin{itemize}" << endl;
 }
 
 void LatexDocVisitor::visitPost(DocSimpleList *)
 {
+  if (m_hide) return;
   m_t << "\\end{itemize}" << endl;
 }
 
 void LatexDocVisitor::visitPre(DocSimpleListItem *)
 {
+  if (m_hide) return;
   m_t << "\\item ";
 }
 
@@ -435,32 +529,14 @@ void LatexDocVisitor::visitPost(DocSimpleListItem *)
 
 void LatexDocVisitor::visitPre(DocSection *s)
 {
+  if (m_hide) return;
   if (Config_getBool("PDF_HYPERLINKS"))
   {
     m_t << "\\hypertarget{" << s->file() << "_" << s->anchor() << "}{}";
   }
-  if (Config_getBool("COMPACT_LATEX"))
-  {
-    switch(s->level())
-    {
-      case 1: m_t << "\\subsubsection{"; break;
-      case 2: m_t << "\\paragraph{";     break;
-      case 3: m_t << "\\subparagraph{";  break;
-      case 4: m_t << "\\subparagraph{";  break;
-    }
-  }
-  else
-  {
-    switch(s->level())
-    {
-      case 1: m_t << "\\subsection{";    break;
-      case 2: m_t << "\\subsubsection{"; break;
-      case 3: m_t << "\\paragraph{";     break;
-      case 4: m_t << "\\subparagraph{";  break;
-    }
-  }
+  m_t << "\\" << getSectionName(s->level()) << "{";
   filter(s->title());
-  m_t << "}\\label{" << s->anchor() << "}" << endl;
+  m_t << "}\\label{" << s->file() << "_" << s->anchor() << "}" << endl;
 }
 
 void LatexDocVisitor::visitPost(DocSection *) 
@@ -469,6 +545,7 @@ void LatexDocVisitor::visitPost(DocSection *)
 
 void LatexDocVisitor::visitPre(DocHtmlList *s)
 {
+  if (m_hide) return;
   if (s->type()==DocHtmlList::Ordered) 
     m_t << "\\begin{enumerate}" << endl; 
   else 
@@ -477,6 +554,7 @@ void LatexDocVisitor::visitPre(DocHtmlList *s)
 
 void LatexDocVisitor::visitPost(DocHtmlList *s) 
 {
+  if (m_hide) return;
   if (s->type()==DocHtmlList::Ordered) 
     m_t << "\\end{enumerate}" << endl; 
   else 
@@ -485,6 +563,7 @@ void LatexDocVisitor::visitPost(DocHtmlList *s)
 
 void LatexDocVisitor::visitPre(DocHtmlListItem *)
 {
+  if (m_hide) return;
   m_t << "\\item ";
 }
 
@@ -492,35 +571,41 @@ void LatexDocVisitor::visitPost(DocHtmlListItem *)
 {
 }
 
-void LatexDocVisitor::visitPre(DocHtmlPre *)
-{
-  m_t << "\\small\\begin{alltt}";
-  m_insidePre=TRUE;
-}
+//void LatexDocVisitor::visitPre(DocHtmlPre *)
+//{
+//  m_t << "\\small\\begin{alltt}";
+//  m_insidePre=TRUE;
+//}
 
-void LatexDocVisitor::visitPost(DocHtmlPre *) 
-{
-  m_insidePre=FALSE;
-  m_t << "\\end{alltt}\\normalsize " << endl;
-}
+//void LatexDocVisitor::visitPost(DocHtmlPre *) 
+//{
+//  m_insidePre=FALSE;
+//  m_t << "\\end{alltt}\\normalsize " << endl;
+//}
 
 void LatexDocVisitor::visitPre(DocHtmlDescList *)
 {
+  if (m_hide) return;
   m_t << "\\begin{description}" << endl;
 }
 
 void LatexDocVisitor::visitPost(DocHtmlDescList *) 
 {
+  if (m_hide) return;
   m_t << "\\end{description}" << endl;
 }
 
 void LatexDocVisitor::visitPre(DocHtmlDescTitle *)
 {
+  if (m_hide) return;
   m_t << "\\item[";
+  m_insideItem=TRUE;
 }
 
 void LatexDocVisitor::visitPost(DocHtmlDescTitle *) 
 {
+  if (m_hide) return;
+  m_insideItem=FALSE;
   m_t << "]";
 }
 
@@ -534,6 +619,7 @@ void LatexDocVisitor::visitPost(DocHtmlDescData *)
 
 void LatexDocVisitor::visitPre(DocHtmlTable *t)
 {
+  if (m_hide) return;
   if (t->hasCaption()) 
   {
     m_t << "\\begin{table}[h]";
@@ -543,23 +629,26 @@ void LatexDocVisitor::visitPre(DocHtmlTable *t)
 
 void LatexDocVisitor::visitPost(DocHtmlTable *t) 
 {
+  if (m_hide) return;
   if (t->hasCaption())
   {
     m_t << "\\end{table}\n";
   }
   else
   {
-    m_t << "\\\\\\hline\n\\end{TabularC}\n";
+    m_t << "\\end{TabularC}\n";
   }
 }
 
 void LatexDocVisitor::visitPre(DocHtmlCaption *)
 {
-  m_t << "\\\\\\hline\n\\end{TabularC}\n\\centering\n\\caption{";
+  if (m_hide) return;
+  m_t << "\\end{TabularC}\n\\centering\n\\caption{";
 }
 
 void LatexDocVisitor::visitPost(DocHtmlCaption *) 
 {
+  if (m_hide) return;
   m_t << "}\n";
 }
 
@@ -569,6 +658,7 @@ void LatexDocVisitor::visitPre(DocHtmlRow *)
 
 void LatexDocVisitor::visitPost(DocHtmlRow *) 
 {
+  if (m_hide) return;
   m_t << "\\\\\\hline\n";
 }
 
@@ -578,22 +668,26 @@ void LatexDocVisitor::visitPre(DocHtmlCell *)
 
 void LatexDocVisitor::visitPost(DocHtmlCell *c) 
 {
+  if (m_hide) return;
   if (!c->isLast()) m_t << "&";
 }
 
 void LatexDocVisitor::visitPre(DocInternal *)
 {
+  if (m_hide) return;
   m_t << "\\begin{Desc}" << endl 
-    << "\\item[" << theTranslator->trForInternalUseOnly() << "]" << endl;
+    << "\\item["; filter(theTranslator->trForInternalUseOnly()); m_t << "]" << endl;
 }
 
 void LatexDocVisitor::visitPost(DocInternal *) 
 {
+  if (m_hide) return;
   m_t << "\\end{Desc}" << endl;
 }
 
 void LatexDocVisitor::visitPre(DocHRef *href)
 {
+  if (m_hide) return;
   if (Config_getBool("PDF_HYPERLINKS"))
   {
     m_t << "\\href{";
@@ -605,33 +699,19 @@ void LatexDocVisitor::visitPre(DocHRef *href)
 
 void LatexDocVisitor::visitPost(DocHRef *) 
 {
+  if (m_hide) return;
   m_t << "}";
 }
 
 void LatexDocVisitor::visitPre(DocHtmlHeader *header)
 {
-  if (Config_getBool("COMPACT_LATEX"))
-  {
-    switch(header->level())
-    {
-      case 1: m_t << "\\subsection*{"; break;
-      case 2: m_t << "\\subsubsection*{"; break;
-      case 3: m_t << "\\paragraph*{"; break;
-    }
-  }
-  else
-  {
-    switch(header->level())
-    {
-      case 1: m_t << "\\section*{"; break;
-      case 2: m_t << "\\subsection*{"; break;
-      case 3: m_t << "\\subsubsection*{"; break;
-    }
-  }
+  if (m_hide) return;
+  m_t << "\\" << getSectionName(header->level()) << "*{";
 }
 
 void LatexDocVisitor::visitPost(DocHtmlHeader *) 
 {
+  if (m_hide) return;
   m_t << "}";
 }
 
@@ -639,6 +719,7 @@ void LatexDocVisitor::visitPre(DocImage *img)
 {
   if (img->type()==DocImage::Latex)
   {
+    if (m_hide) return;
     if (img->hasCaption())
     {
       m_t << "\\begin{figure}[H]" << endl;
@@ -648,7 +729,7 @@ void LatexDocVisitor::visitPre(DocImage *img)
     {
       m_t << "\\mbox{";
     }
-    QCString gfxName = img->name();
+    QString gfxName = img->name();
     if (gfxName.right(4)==".eps" || gfxName.right(4)==".pdf")
     {
       gfxName=gfxName.left(gfxName.length()-4);
@@ -670,6 +751,7 @@ void LatexDocVisitor::visitPre(DocImage *img)
   }
   else // other format -> skip
   {
+    pushEnabled();
     m_hide=TRUE;
   }
 }
@@ -678,6 +760,7 @@ void LatexDocVisitor::visitPost(DocImage *img)
 {
   if (img->type()==DocImage::Latex)
   {
+    if (m_hide) return;
     m_t << "}" << endl; // end mbox or caption
     if (img->hasCaption())
     {
@@ -687,94 +770,62 @@ void LatexDocVisitor::visitPost(DocImage *img)
   }
   else // other format
   {
-    m_hide=FALSE;
+    popEnabled();
   }
 }
 
 void LatexDocVisitor::visitPre(DocDotFile *df)
 {
-  QCString baseName=df->file();
-  int i;
-  if ((i=baseName.findRev('/'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  } 
-  if (baseName.right(4)==".eps" || baseName.right(4)==".pdf")
-  {
-    baseName=baseName.left(baseName.length()-4);
-  }
-  QCString outDir = Config_getString("LATEX_OUTPUT");
-  writeDotGraphFromFile(df->file(),outDir,baseName,EPS);
-  if (df->hasCaption())
-  {
-    m_t << "\\begin{figure}[H]" << endl;
-    m_t << "\\begin{center}" << endl;
-  }
-  else
-  {
-    m_t << "\\mbox{";
-  }
-  m_t << "\\includegraphics";
-  if (!df->width().isEmpty())
-  {
-    m_t << "[width=" << df->width() << "]";
-  }
-  else if (!df->height().isEmpty())
-  {
-    m_t << "[height=" << df->height() << "]";
-  }
-  m_t << "{" << baseName << "}";
-
-  if (df->hasCaption())
-  {
-    m_t << "\\caption{";
-  }
+  if (m_hide) return;
+  startDotFile(df->file(),df->width(),df->height(),df->hasCaption());
 }
 
 void LatexDocVisitor::visitPost(DocDotFile *df) 
 {
-  m_t << "}" << endl; // end mbox or caption
-  if (df->hasCaption())
-  {
-    m_t << "\\end{center}" << endl;
-    m_t << "\\end{figure}" << endl;
-  }
+  if (m_hide) return;
+  endDotFile(df->hasCaption());
 }
 
 void LatexDocVisitor::visitPre(DocLink *lnk)
 {
+  if (m_hide) return;
   startLink(lnk->ref(),lnk->file(),lnk->anchor());
 }
 
-void LatexDocVisitor::visitPost(DocLink *) 
+void LatexDocVisitor::visitPost(DocLink *lnk) 
 {
-  endLink();
+  if (m_hide) return;
+  endLink(lnk->ref(),lnk->file(),lnk->anchor());
 }
 
 void LatexDocVisitor::visitPre(DocRef *ref)
 {
-  startLink(ref->ref(),ref->file(),ref->anchor());
+  if (m_hide) return;
+  if (!ref->file().isEmpty()) startLink(ref->ref(),ref->file(),ref->anchor());
   if (!ref->hasLinkText()) filter(ref->targetTitle());
 }
 
-void LatexDocVisitor::visitPost(DocRef *) 
+void LatexDocVisitor::visitPost(DocRef *ref) 
 {
-  endLink();
-  m_t << " ";
+  if (m_hide) return;
+  if (!ref->file().isEmpty()) endLink(ref->ref(),ref->file(),ref->anchor());
 }
 
 void LatexDocVisitor::visitPre(DocSecRefItem *)
 {
+  if (m_hide) return;
   m_t << "\\item \\contentsline{section}{";
 }
 
 void LatexDocVisitor::visitPost(DocSecRefItem *ref) 
 {
-  m_t << "}{\\ref{" << ref->anchor() << "}}{}" << endl;
+  if (m_hide) return;
+  m_t << "}{\\ref{" << ref->file() << "_" << ref->anchor() << "}}{}" << endl;
 }
 
 void LatexDocVisitor::visitPre(DocSecRefList *)
 {
+  if (m_hide) return;
   m_t << "\\footnotesize" << endl;
   m_t << "\\begin{multicols}{2}" << endl;
   m_t << "\\begin{CompactList}" << endl;
@@ -782,31 +833,44 @@ void LatexDocVisitor::visitPre(DocSecRefList *)
 
 void LatexDocVisitor::visitPost(DocSecRefList *) 
 {
+  if (m_hide) return;
   m_t << "\\end{CompactList}" << endl;
   m_t << "\\end{multicols}" << endl;
   m_t << "\\normalsize" << endl;
 }
 
-void LatexDocVisitor::visitPre(DocLanguage *)
+void LatexDocVisitor::visitPre(DocLanguage *l)
 {
+  QString langId = Config_getEnum("OUTPUT_LANGUAGE");
+  if (l->id().lower()!=langId.lower())
+  {
+    pushEnabled();
+    m_hide = TRUE;
+  }
 }
 
-void LatexDocVisitor::visitPost(DocLanguage *) 
+void LatexDocVisitor::visitPost(DocLanguage *l) 
 {
+  QString langId = Config_getEnum("OUTPUT_LANGUAGE");
+  if (l->id().lower()!=langId.lower())
+  {
+    popEnabled();
+  }
 }
 
 void LatexDocVisitor::visitPre(DocParamSect *s)
 {
+  if (m_hide) return;
   m_t << "\\begin{Desc}" << endl;
   m_t << "\\item[";
   switch(s->type())
   {
     case DocParamSect::Param: 
-      m_t << theTranslator->trParameters(); break;
+      filter(theTranslator->trParameters()); break;
     case DocParamSect::RetVal: 
-      m_t << theTranslator->trReturnValues(); break;
+      filter(theTranslator->trReturnValues()); break;
     case DocParamSect::Exception: 
-      m_t << theTranslator->trExceptions(); break;
+      filter(theTranslator->trExceptions()); break;
     default:
       ASSERT(0);
   }
@@ -816,20 +880,42 @@ void LatexDocVisitor::visitPre(DocParamSect *s)
 
 void LatexDocVisitor::visitPost(DocParamSect *)
 {
+  if (m_hide) return;
   m_t << "\\end{description}" << endl;
   m_t << "\\end{Desc}" << endl;
 }
 
 void LatexDocVisitor::visitPre(DocParamList *pl)
 {
-  m_t << "\\item[{\\em ";
+  if (m_hide) return;
+  m_t << "\\item[";
+  if (pl->direction()!=DocParamSect::Unspecified)
+  {
+    m_t << "\\mbox{";
+    if (pl->direction()==DocParamSect::In)
+    {
+      m_t << "$\\leftarrow$";
+    }
+    else if (pl->direction()==DocParamSect::Out)
+    {
+      m_t << "$\\rightarrow$";
+    }
+    else if (pl->direction()==DocParamSect::InOut)
+    {
+      m_t << "$\\leftrightarrow$";
+    }
+    m_t << "} ";
+  }
+  m_t << "{\\em ";
   QStrListIterator li(pl->parameters());
   const char *s;
   bool first=TRUE;
   for (li.toFirst();(s=li.current());++li)
   {
     if (!first) m_t << ","; else first=FALSE;
-    m_t << s;
+    m_insideItem=TRUE;
+    filter(s);
+    m_insideItem=FALSE;
   }
   m_t << "}]";
 }
@@ -840,34 +926,39 @@ void LatexDocVisitor::visitPost(DocParamList *)
 
 void LatexDocVisitor::visitPre(DocXRefItem *x)
 {
+  if (m_hide) return;
   m_t << "\\begin{Desc}" << endl;
   m_t << "\\item[";
   if (Config_getBool("PDF_HYPERLINKS"))
   {
-    m_t << "\\hyperlink{" << x->file() << "_" << x->anchor() << "}{";
+    m_t << "\\hyperlink{" << stripPath(x->file()) << "_" << x->anchor() << "}{";
   }
   else
   {
     m_t << "{\\bf ";
   }
+  m_insideItem=TRUE;
   filter(x->title());
+  m_insideItem=FALSE;
   m_t << "}]";
 }
 
 void LatexDocVisitor::visitPost(DocXRefItem *)
 {
+  if (m_hide) return;
   m_t << "\\end{Desc}" << endl;
 }
 
 void LatexDocVisitor::visitPre(DocInternalRef *ref)
 {
+  if (m_hide) return;
   startLink(0,ref->file(),ref->anchor());
 }
 
-void LatexDocVisitor::visitPost(DocInternalRef *) 
+void LatexDocVisitor::visitPost(DocInternalRef *ref) 
 {
-  endLink();
-  m_t << " ";
+  if (m_hide) return;
+  endLink(0,ref->file(),ref->anchor());
 }
 
 void LatexDocVisitor::visitPre(DocCopy *)
@@ -878,17 +969,25 @@ void LatexDocVisitor::visitPost(DocCopy *)
 {
 }
 
-void LatexDocVisitor::filter(const char *str)
-{ 
-  filterLatexString(m_t,str,FALSE,m_insidePre);
+void LatexDocVisitor::visitPre(DocText *)
+{
 }
 
-void LatexDocVisitor::startLink(const QCString &ref,const QCString &file,const QCString &anchor)
+void LatexDocVisitor::visitPost(DocText *)
+{
+}
+
+void LatexDocVisitor::filter(const char *str)
+{ 
+  filterLatexString(m_t,str,FALSE,m_insidePre,m_insideItem);
+}
+
+void LatexDocVisitor::startLink(const QString &ref,const QString &file,const QString &anchor)
 {
   if (ref.isEmpty() && Config_getBool("PDF_HYPERLINKS"))
   {
     m_t << "\\hyperlink{";
-    if (!file.isEmpty()) m_t << file;
+    if (!file.isEmpty()) m_t << stripPath(file);
     if (!file.isEmpty() && !anchor.isEmpty()) m_t << "_";
     if (!anchor.isEmpty()) m_t << anchor;
     m_t << "}{";
@@ -900,8 +999,89 @@ void LatexDocVisitor::startLink(const QCString &ref,const QCString &file,const Q
   }
 }
 
-void LatexDocVisitor::endLink()
+void LatexDocVisitor::endLink(const QString &ref,const QString &file,const QString &anchor)
 {
   m_t << "}";
+  if (ref.isEmpty() && !Config_getBool("PDF_HYPERLINKS"))
+  {
+    m_t << "{\\rm ("; 
+    filter(theTranslator->trPageAbbreviation());
+    m_t << "\\,\\pageref{" << file;
+    if (!anchor.isEmpty()) m_t << "_" << anchor;
+    m_t << "})}";
+  }
+}
+
+void LatexDocVisitor::pushEnabled()
+{
+  m_enabled.push(new bool(m_hide));
+}
+
+void LatexDocVisitor::popEnabled()
+{
+  bool *v=m_enabled.pop();
+  ASSERT(v!=0);
+  m_hide = *v;
+  delete v;
+}
+
+void LatexDocVisitor::startDotFile(const QString &fileName,
+                                   const QString &width,
+                                   const QString &height,
+                                   bool hasCaption
+                                  )
+{
+  QString baseName=fileName;
+  int i;
+  if ((i=baseName.findRev('/'))!=-1)
+  {
+    baseName=baseName.right(baseName.length()-i-1);
+  } 
+  if (baseName.right(4)==".eps" || baseName.right(4)==".pdf")
+  {
+    baseName=baseName.left(baseName.length()-4);
+  }
+  if (baseName.right(4)==".dot")
+  {
+    baseName=baseName.left(baseName.length()-4);
+  }
+  QString outDir = Config_getString("LATEX_OUTPUT");
+  QString name = fileName;
+  writeDotGraphFromFile(name,outDir,baseName,EPS);
+  if (hasCaption)
+  {
+    m_t << "\\begin{figure}[H]" << endl;
+    m_t << "\\begin{center}" << endl;
+  }
+  else
+  {
+    m_t << "\\mbox{";
+  }
+  m_t << "\\includegraphics";
+  if (!width.isEmpty())
+  {
+    m_t << "[width=" << width << "]";
+  }
+  else if (!height.isEmpty())
+  {
+    m_t << "[height=" << height << "]";
+  }
+  m_t << "{" << baseName << "}";
+
+  if (hasCaption)
+  {
+    m_t << "\\caption{";
+  }
+}
+
+void LatexDocVisitor::endDotFile(bool hasCaption)
+{
+  if (m_hide) return;
+  m_t << "}" << endl; // end mbox or caption
+  if (hasCaption)
+  {
+    m_t << "\\end{center}" << endl;
+    m_t << "\\end{figure}" << endl;
+  }
 }
 
