@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -20,10 +20,11 @@
 #define _LATEXDOCVISITOR_H
 
 #include "docvisitor.h"
+#include <qstack.h>
 
 class QTextStream;
 class BaseCodeDocInterface;
-class QCString;
+class QString;
 
 /*! @brief Concrete visitor implementation for LaTeX output. */
 class LatexDocVisitor : public DocVisitor
@@ -76,8 +77,8 @@ class LatexDocVisitor : public DocVisitor
     void visitPost(DocHtmlList *s);
     void visitPre(DocHtmlListItem *);
     void visitPost(DocHtmlListItem *);
-    void visitPre(DocHtmlPre *);
-    void visitPost(DocHtmlPre *);
+    //void visitPre(DocHtmlPre *);
+    //void visitPost(DocHtmlPre *);
     void visitPre(DocHtmlDescList *);
     void visitPost(DocHtmlDescList *);
     void visitPre(DocHtmlDescTitle *);
@@ -122,6 +123,8 @@ class LatexDocVisitor : public DocVisitor
     void visitPost(DocInternalRef *);
     void visitPre(DocCopy *);
     void visitPost(DocCopy *);
+    void visitPre(DocText *);
+    void visitPost(DocText *);
 
   private:
 
@@ -130,10 +133,17 @@ class LatexDocVisitor : public DocVisitor
     //--------------------------------------
     
     void filter(const char *str);
-    void startLink(const QCString &ref,const QCString &file,
-                   const QCString &anchor);
-    void endLink();
-    QCString escapeMakeIndexChars(const char *s);
+    void startLink(const QString &ref,const QString &file,
+                   const QString &anchor);
+    void endLink(const QString &ref,const QString &file,
+                 const QString &anchor);
+    QString escapeMakeIndexChars(const char *s);
+    void startDotFile(const QString &fileName,const QString &width,
+                      const QString &height, bool hasCaption);
+    void endDotFile(bool hasCaption);
+
+    void pushEnabled();
+    void popEnabled();
 
     //--------------------------------------
     // state variables
@@ -142,7 +152,9 @@ class LatexDocVisitor : public DocVisitor
     QTextStream &m_t;
     BaseCodeDocInterface &m_ci;
     bool m_insidePre;
+    bool m_insideItem;
     bool m_hide;
+    QStack<bool> m_enabled;
 };
 
 #endif
