@@ -2,7 +2,7 @@
  *
  * $Id: latexgen.cpp,v 1.58 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -28,7 +28,7 @@
 #include "language.h"
 #include "version.h"
 #include "dot.h"
-#include "page.h"
+#include "pagedef.h"
 #include "docparser.h"
 #include "latexdocvisitor.h"
 
@@ -51,6 +51,7 @@
 //  return result;  
 //}
 
+
 static QCString escapeLabelName(const char *s)
 {
   QCString result;
@@ -113,24 +114,24 @@ LatexGenerator::~LatexGenerator()
 //  col=g.col;
 //}
 
-void LatexGenerator::append(const OutputGenerator *g)
-{
-  t << g->getContents();
-  col+=((LatexGenerator *)g)->col;
-  insideTabbing=insideTabbing || ((LatexGenerator *)g)->insideTabbing;
-  firstDescItem = ((LatexGenerator *)g)->firstDescItem;
-  insidePre = insidePre || ((LatexGenerator *)g)->insidePre;
-  //printf("LatexGenerator::append(%s) insideTabbing=%s\n", g->getContents().data(),
-  //    insideTabbing ? "TRUE" : "FALSE" );
-}
+//void LatexGenerator::append(const OutputGenerator *g)
+//{
+//  t << g->getContents();
+//  col+=((LatexGenerator *)g)->col;
+//  insideTabbing=insideTabbing || ((LatexGenerator *)g)->insideTabbing;
+//  firstDescItem = ((LatexGenerator *)g)->firstDescItem;
+//  insidePre = insidePre || ((LatexGenerator *)g)->insidePre;
+//  //printf("LatexGenerator::append(%s) insideTabbing=%s\n", g->getContents().data(),
+//  //    insideTabbing ? "TRUE" : "FALSE" );
+//}
 
-OutputGenerator *LatexGenerator::copy()
-{
-  LatexGenerator *result = new LatexGenerator;
-  result->insideTabbing=insideTabbing;
-  result->insidePre=insidePre;
-  return result;
-}
+//OutputGenerator *LatexGenerator::copy()
+//{
+//  LatexGenerator *result = new LatexGenerator;
+//  result->insideTabbing=insideTabbing;
+//  result->insidePre=insidePre;
+//  return result;
+//}
 
 void LatexGenerator::init()
 {
@@ -221,7 +222,9 @@ void LatexGenerator::init()
 
   t << endl
     << "clean:" << endl
-    << "\trm -f *.ps *.dvi *.aux *.toc *.idx *.ind *.ilg *.log *.out" << endl;
+    << "\trm -f *.ps *.dvi *.aux *.toc *.idx *.ind *.ilg *.log *.out refman.pdf" << endl;
+
+  createSubDirs(d);
 }
 
 static void writeDefaultHeaderPart1(QTextStream &t)
@@ -246,7 +249,10 @@ static void writeDefaultHeaderPart1(QTextStream &t)
     "\\usepackage{graphicx}\n"
     "\\usepackage{multicol}\n"
     "\\usepackage{float}\n"
-    "\\usepackage{alltt}\n";
+    "\\usepackage{textcomp}\n"
+    "\\usepackage{alltt}\n"
+    //"\\usepackage{ae,aecompl,aeguill}\n"
+    ;
   if (Config_getBool("PDF_HYPERLINKS")) 
   {
     t << "\\usepackage{times}" << endl;
@@ -256,6 +262,7 @@ static void writeDefaultHeaderPart1(QTextStream &t)
       << "            colorlinks=true," << endl
       << "            linkcolor=blue" << endl
       << "           ]{hyperref}" << endl
+      << "\\usepackage{pspicture}" << endl
       << "\\else" << endl
       << "\\usepackage[pdftex," << endl
       << "            pagebackref=true," << endl
@@ -273,7 +280,7 @@ static void writeDefaultHeaderPart1(QTextStream &t)
   {
     // The command is not empty.  Put it to the output.
     // if the command is empty, no output is needed.
-    t << sLanguageSupportCommand;
+    t << sLanguageSupportCommand << endl;
   }
 
   t << "\\usepackage{doxygen}\n";
@@ -286,7 +293,7 @@ static void writeDefaultHeaderPart1(QTextStream &t)
   }
   t << "\\makeindex\n"
     "\\setcounter{tocdepth}{1}\n"
-    "\\setlength{\\footrulewidth}{0.4pt}\n"
+    "\\renewcommand{\\footrulewidth}{0.4pt}\n"
     "\\begin{document}\n";
   if (theTranslator->idLanguage()=="greek") t << "\\selectlanguage{greek}\n";
   t << "\\begin{titlepage}\n"
@@ -326,8 +333,8 @@ static void writeDefaultStyleSheetPart1(QTextStream &t)
   t << "\\RequirePackage{calc}\n";
   t << "\\RequirePackage{array}\n";
   t << "\\pagestyle{fancyplain}\n";
-  t << "\\addtolength{\\headwidth}{\\marginparsep}\n";
-  t << "\\addtolength{\\headwidth}{\\marginparwidth}\n";
+  //t << "\\addtolength{\\headwidth}{\\marginparsep}\n";
+  //t << "\\addtolength{\\headwidth}{\\marginparwidth}\n";
   t << "\\newcommand{\\clearemptydoublepage}{\\newpage{\\pagestyle{empty}";
   t << "\\cleardoublepage}}\n";
   if (!Config_getBool("COMPACT_LATEX")) 
@@ -399,6 +406,7 @@ static void writeDefaultStyleSheetPart3(QTextStream &t)
   t << "\\setlength{\\parskip}{0.2cm}\n";
   t << "\\addtocounter{secnumdepth}{1}\n";
   t << "\\sloppy\n";
+  t << "\\usepackage[T1]{fontenc}\n";
 }
 
 void LatexGenerator::writeHeaderFile(QFile &f)
@@ -424,18 +432,19 @@ void LatexGenerator::writeStyleSheetFile(QFile &f)
   t << theTranslator->trGeneratedAt( dateToString(TRUE), projectName );
   t << " doxygen";
   //t << " " << theTranslator->trWrittenBy() << " ";
-  //t << "Dimitri van Heesch \\copyright~1997-2002";
+  //t << "Dimitri van Heesch \\copyright~1997-2004";
   writeDefaultStyleSheetPart2(t);
   t << theTranslator->trGeneratedAt( dateToString(TRUE), projectName );
   t << " doxygen";
   //t << " << theTranslator->trWrittenBy() << " ";
-  //t << "Dimitri van Heesch \\copyright~1997-2002";
+  //t << "Dimitri van Heesch \\copyright~1997-2004";
   writeDefaultStyleSheetPart3(t);
 }
 
-void LatexGenerator::startFile(const char *name,const char *,const char *,bool)
+void LatexGenerator::startFile(const char *name,const char *,const char *)
 {
   QCString fileName=name;
+  relPath = relativePathToRoot(fileName);
   if (fileName.right(4)!=".tex" && fileName.right(4)!=".sty") fileName+=".tex";
   startPlainFile(fileName);
 }
@@ -512,21 +521,6 @@ void LatexGenerator::startIndexSection(IndexSections is)
       if (compactLatex) t << "\\section"; else t << "\\chapter";
       t << "{"; //Annotated Page Index}\n"
       break;
-//    case isPackageDocumentation:
-//      {
-//        PackageSDict::Iterator pdi(Doxygen::packageDict);
-//        PackageDef *pd=pdi.toFirst();
-//        bool found=FALSE;
-//        while (pd && !found)
-//        {
-//          if (compactLatex) t << "\\section"; else t << "\\chapter";
-//          t << "{"; 
-//          found=TRUE;
-//          ++pdi;
-//          pd=pdi.current();
-//        }
-//      }
-//      break;
     case isModuleDocumentation:
       {
         GroupSDict::Iterator gli(Doxygen::groupSDict);
@@ -640,9 +634,6 @@ void LatexGenerator::endIndexSection(IndexSections is)
         t << "\\input{" << indexName << "}\n";
       }
       break;
-    //case isPackageIndex:
-    //  t << "}\n\\input{packages}\n";
-    //  break;
     case isModuleIndex:
       t << "}\n\\input{modules}\n";
       break;
@@ -661,27 +652,6 @@ void LatexGenerator::endIndexSection(IndexSections is)
     case isPageIndex:
       t << "}\n\\input{pages}\n";
       break;
-    //case isPackageDocumentation:
-    //  {
-    //    PackageSDict::Iterator pdi(Doxygen::packageDict);
-    //    PackageDef *pd=pdi.toFirst();
-    //    bool found=FALSE;
-    //    while (pd && !found)
-    //    {
-    //      t << "}\n\\input{" << pd->getOutputFileBase() << "}\n";
-    //      found=TRUE;
-    //      ++pdi;
-    //      pd=pdi.current();
-    //    }
-    //    while (pd)
-    //    {
-    //       if (compactLatex) t << "\\input"; else t << "\\include";
-    //     t << "{" << pd->getOutputFileBase() << "}\n";
-    //      ++pdi;
-    //      pd=pdi.current();
-    //    }
-    //  }
-    //  break;
     case isModuleDocumentation:
       {
         GroupSDict::Iterator gli(Doxygen::groupSDict);
@@ -784,15 +754,15 @@ void LatexGenerator::endIndexSection(IndexSections is)
       {
         t << "}\n";
         PageSDict::Iterator pdi(*Doxygen::exampleSDict);
-        PageInfo *pi=pdi.toFirst();
-        if (pi)
+        PageDef *pd=pdi.toFirst();
+        if (pd)
         {
-          t << "\\input{" << pi->getOutputFileBase() << "}\n";
+          t << "\\input{" << pd->getOutputFileBase() << "}\n";
         }
-        for (++pdi;(pi=pdi.current());++pdi)
+        for (++pdi;(pd=pdi.current());++pdi)
         {
           if (compactLatex) t << "\\input" ; else t << "\\include";
-          t << "{" << pi->getOutputFileBase() << "}\n";
+          t << "{" << pd->getOutputFileBase() << "}\n";
         }
       }
       break;
@@ -800,14 +770,14 @@ void LatexGenerator::endIndexSection(IndexSections is)
       {
         t << "}\n";
         PageSDict::Iterator pdi(*Doxygen::pageSDict);
-        PageInfo *pi=pdi.toFirst();
+        PageDef *pd=pdi.toFirst();
         bool first=TRUE;
-        for (pdi.toFirst();(pi=pdi.current());++pdi)
+        for (pdi.toFirst();(pd=pdi.current());++pdi)
         {
-          if (!pi->getGroupDef() && !pi->isReference())
+          if (!pd->getGroupDef() && !pd->isReference())
           {
             if (compactLatex || first) t << "\\input" ; else t << "\\include";
-            t << "{" << pi->getOutputFileBase() << "}\n";
+            t << "{" << pd->getOutputFileBase() << "}\n";
             first=FALSE;
           }
         }
@@ -836,14 +806,14 @@ void LatexGenerator::writeStyleInfo(int part)
       break;
     case 2:
       {
-        //t << " Dimitri van Heesch \\copyright~1997-2002";
+        //t << " Dimitri van Heesch \\copyright~1997-2004";
         t << "}]{}\n";
         writeDefaultStyleSheetPart2(t);
       }
       break;
     case 4:
       {
-        //t << " Dimitri van Heesch \\copyright~1997-2002";
+        //t << " Dimitri van Heesch \\copyright~1997-2004";
         writeDefaultStyleSheetPart3(t);
         endPlainFile();
       }
@@ -899,18 +869,18 @@ void LatexGenerator::endHtmlLink()
   t << "}";
 }
 
-void LatexGenerator::writeMailLink(const char *url)
-{
-  if (Config_getBool("PDF_HYPERLINKS"))
-  {
-    t << "\\href{mailto:";
-    t << url;
-    t << "}";
-  }
-  t << "{\\tt "; 
-  docify(url);
-  t << "}";
-}
+//void LatexGenerator::writeMailLink(const char *url)
+//{
+//  if (Config_getBool("PDF_HYPERLINKS"))
+//  {
+//    t << "\\href{mailto:";
+//    t << url;
+//    t << "}";
+//  }
+//  t << "{\\tt "; 
+//  docify(url);
+//  t << "}";
+//}
 
 void LatexGenerator::writeStartAnnoItem(const char *,const char *,
                                         const char *path,const char *name)
@@ -960,7 +930,7 @@ void LatexGenerator::startTextLink(const char *f,const char *anchor)
   if (Config_getBool("PDF_HYPERLINKS"))
   {
     t << "\\hyperlink{";
-    if (f) t << f;
+    if (f) t << stripPath(f);
     if (anchor) t << "_" << anchor; 
     t << "}{";
   }
@@ -981,7 +951,7 @@ void LatexGenerator::writeObjectLink(const char *ref, const char *f,
   if (!ref && Config_getBool("PDF_HYPERLINKS"))
   {
     t << "\\hyperlink{";
-    if (f) t << f;
+    if (f) t << stripPath(f);
     if (f && anchor) t << "_"; 
     if (anchor) t << anchor; 
     t << "}{";
@@ -1020,7 +990,7 @@ void LatexGenerator::startTitleHead(const char *fileName)
 {
   if (Config_getBool("PDF_HYPERLINKS") && fileName)
   {
-    t << "\\hypertarget{" << fileName << "}{" << endl;
+    t << "\\hypertarget{" << stripPath(fileName) << "}{" << endl;
   }
   if (Config_getBool("COMPACT_LATEX")) 
   {
@@ -1134,7 +1104,7 @@ void LatexGenerator::startMemberDoc(const char *clname,
   t << "{\\setlength{\\rightskip}{0pt plus 5cm}";
 }
 
-void LatexGenerator::endMemberDoc() 
+void LatexGenerator::endMemberDoc(bool) 
 { 
   t << "}";
   if (Config_getBool("COMPACT_LATEX")) t << "\\hfill";
@@ -1146,7 +1116,7 @@ void LatexGenerator::startDoxyAnchor(const char *fName,const char *,
   if (Config_getBool("PDF_HYPERLINKS"))
   {
     t << "\\hypertarget{";
-    if (fName) t << fName;
+    if (fName) t << stripPath(fName);
     if (anchor) t << "_" << anchor;
     t << "}{" << endl;
   }
@@ -1170,7 +1140,7 @@ void LatexGenerator::writeAnchor(const char *fName,const char *name)
   t << "\\label{" << name << "}" << endl; 
   if (fName && Config_getBool("PDF_HYPERLINKS"))
   {
-    t << "\\hypertarget{" << fName << "_" << name << "}{}" << endl;
+    t << "\\hypertarget{" << stripPath(fName) << "_" << name << "}{}" << endl;
   }
 }
 
@@ -1202,7 +1172,7 @@ void LatexGenerator::startSection(const char *lab,const char *,SectionInfo::Sect
 {
   if (Config_getBool("PDF_HYPERLINKS"))
   {
-    t << "\\hypertarget{" << lab << "}{}";
+    t << "\\hypertarget{" << stripPath(lab) << "}{}";
   }
   t << "\\";
   if (Config_getBool("COMPACT_LATEX"))
@@ -1238,125 +1208,17 @@ void LatexGenerator::endSection(const char *lab,SectionInfo::SectionType)
   t << "}\\label{" << lab << "}" << endl;
 }
 
-void LatexGenerator::writeSectionRef(const char *ref,const char *,
-                                     const char *lab,const char *text)
-{
-  if (ref) // external reference
-  {
-    docify(text);
-  }
-  else // local reference
-  {
-    if (text && Config_getBool("PDF_HYPERLINKS"))
-    {
-      t << "\\hyperlink{";
-      if (lab) t << lab; 
-      t << "}{";
-      docify(text);
-      t << "}";
-      //t << " {\\rm (p.\\,\\pageref{" << lab << "})}";
-    }
-    else
-    {
-      if (strcmp(lab,text)!=0) // lab!=text
-      {
-        // todo: don't hardcode p. here!
-        t << "{\\bf ";
-        docify(text);
-        t << "} {\\rm (p.\\,\\pageref{" << lab << "})}";
-      }
-      else
-      {
-        t << "\\ref{" << lab << "}";
-      }
-    }
-  }
-}
-
-void LatexGenerator::writeSectionRefItem(const char *,const char *lab,
-                                     const char *title)
-{
-  t << "\\item \\contentsline{section}{";
-  docify(title);
-  t << "}{\\ref{" << lab << "}}{}" << endl;
-}
-
-// TODO: remove this function
-//void LatexGenerator::writeSectionRefAnchor(const char *,const char *lab,
-//                                     const char *title)
-//{
-//  startBold();
-//  docify(title);
-//  endBold();
-//  t << " (p.\\,\\pageref{" << lab << "})" << endl;
-//}
-
 
 //void LatexGenerator::docifyStatic(QTextStream &t,const char *str)
 void LatexGenerator::docify(const char *str)
 {
-#if 0
-  //static bool isCzech         = theTranslator->idLanguage()=="czech";
-  static bool isJapanese      = theTranslator->idLanguage()=="japanese";
-  //static bool isKorean        = theTranslator->idLanguage()=="korean";
-  //static bool isRussian       = theTranslator->idLanguage()=="russian";
-  //static bool isUkrainian     = theTranslator->idLanguage()=="ukrainian";
-  //static bool isChinese       = theTranslator->idLanguage()=="chinese" || 
-  //                              theTranslator->idLanguage()=="chinese-traditional";
-  //static bool isLatin2        = theTranslator->idLanguageCharset()=="iso-8859-2";
-  //static bool isGreek         = theTranslator->idLanguage()=="greek";
-  if (str)
-  {
-    const unsigned char *p=(const unsigned char *)str;
-    unsigned char c;
-    unsigned char pc='\0';
-    bool multiByte = FALSE;
-    while (*p)
-    {
-      c=*p++;
-
-      if( isJapanese)
-      {
-        if (multiByte)
-        {
-          t << (char)c;
-          multiByte = FALSE;
-          pc = c;
-          continue;
-        }
-        if (c>=0x80)
-        {
-          multiByte = TRUE;
-          t << (char)c;
-          pc = c;
-          continue;
-        }
-      }
-
-      if (insidePre)
-      {
-        switch(c)
-        {
-          case '\\': t << "\\(\\backslash\\)"; break;
-          case '{':  t << "\\{"; break;
-          case '}':  t << "\\}"; break;
-          default: t << (char)c; break;
-        }
-      }
-      else
-      {
-        filterLatexChar(t,c,insideTabbing);
-      }
-      pc = c;
-    }
-  }
-#endif
   filterLatexString(t,str,insideTabbing,insidePre);
 }
 
 void LatexGenerator::codify(const char *str)
 {
-  static bool isJapanese      = theTranslator->idLanguage()=="japanese";
+  static bool isJapanese  = theTranslator->idLanguage()=="japanese" || 
+                            theTranslator->idLanguage()=="japanese-en";
 
   if (str)
   { 
@@ -1377,7 +1239,7 @@ void LatexGenerator::codify(const char *str)
           MultiByte = FALSE;
           continue;
         }
-        if ( c<0 ) // char in range [0x80..0xff]
+        if ((uchar)c>=0x80) // char in range [0x80..0xff]
         {
           t << (char)c;
           MultiByte = TRUE;
@@ -1420,42 +1282,42 @@ void LatexGenerator::endClassDiagram(ClassDiagram &d,
   d.writeFigure(t,dir,fileName);
 }
 
-void LatexGenerator::writeFormula(const char *,const char *text)
+
+void LatexGenerator::startAnonTypeScope(int indent)
 {
-  t << text;
+  if (indent==0)
+  {
+    t << "\\begin{tabbing}" << endl;
+    t << "xx\\=xx\\=xx\\=xx\\=xx\\=xx\\=xx\\=xx\\=xx\\=\\kill" << endl;
+    //printf("LatexGenerator::startMemberItem() insideTabbing=TRUE\n");
+    insideTabbing=TRUE;
+  }
 }
 
-void LatexGenerator::startMemberItem(int annType) 
+void LatexGenerator::endAnonTypeScope(int indent)
+{
+  if (indent==0)
+  {
+    t << endl << "\\end{tabbing}";
+    insideTabbing=FALSE;
+  }
+}
+
+void LatexGenerator::startMemberItem(int) 
 { 
+  //printf("LatexGenerator::startMemberItem(%d)\n",annType);
   if (!insideTabbing)
   {
     t << "\\item " << endl; 
-    switch(annType)
-    {
-      case 0: break;
-      case 1:
-      default: 
-        t << "\\begin{tabbing}" << endl;
-        t << "xx\\=xx\\=xx\\=xx\\=xx\\=xx\\=xx\\=xx\\=xx\\=\\kill" << endl;
-        //printf("LatexGenerator::startMemberItem() insideTabbing=TRUE\n");
-        insideTabbing=TRUE;
-        break;
-    }
   }
 }
 
-void LatexGenerator::endMemberItem(bool endItem) 
+void LatexGenerator::endMemberItem() 
 {
-  if (insideTabbing && endItem)
-  {
-    t << endl << "\\end{tabbing}";
-    //printf("LatexGenerator::endMemberItem() insideTabbing=FALSE\n");
-    insideTabbing=FALSE;
-  }
   if (insideTabbing)
   {
     t << "\\\\";
-  } 
+ } 
   t << endl; 
 }
 
@@ -1470,92 +1332,17 @@ void LatexGenerator::writeNonBreakableSpace(int)
 void LatexGenerator::startMemberList()  
 { 
   if (!insideTabbing)
+  {
     t << "\\begin{CompactItemize}" << endl; 
+  }
 }
 
 void LatexGenerator::endMemberList()    
 {
+  //printf("LatexGenerator::endMemberList(%d)\n",insideTabbing);
   if (!insideTabbing)
+  {
     t << "\\end{CompactItemize}"   << endl; 
-}
-
-void LatexGenerator::startImage(const char *name,const char *size,bool hasCaption)
-{
-  if (hasCaption)
-  {
-    t << "\\begin{figure}[H]" << endl;
-    t << "\\begin{center}" << endl;
-  }
-  else
-  {
-    t << "\\mbox{";
-  }
-  QCString gfxName = name;
-  if (gfxName.right(4)==".eps") gfxName.left(gfxName.length()-4);
-  //     "\\epsfig{file=" << name;
-  t << "\\includegraphics";
-  if (size) t << "[" << size << "]";
-  t << "{" << gfxName << "}";
-  if (hasCaption) 
-  {
-    t << "\\caption{";
-  }
-  else
-  {
-    t << "}" << endl;
-  }
-}
-
-void LatexGenerator::endImage(bool hasCaption)
-{
-  if (hasCaption)
-  {
-    t << "}" << endl;
-    t << "\\end{center}" << endl;
-    t << "\\end{figure}" << endl;
-  }
-}
-
-void LatexGenerator::startDotFile(const char *name,bool hasCaption)
-{
-  QCString baseName=name;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1 || (i=baseName.findRev('\\'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1); 
-  }
-  QCString outDir = Config_getString("LATEX_OUTPUT");
-  writeDotGraphFromFile(name,outDir,baseName,EPS);
-  if (hasCaption)
-  {
-    t << "\\begin{figure}[H]" << endl;
-    t << "\\begin{center}" << endl;
-  }
-  else
-    t << "\\mbox{";
-  t << "\\includegraphics";
-  if( Config_getBool("USE_PDFLATEX") )
-  {
-    t << "{" << baseName << ".pdf}";
-  }
-  else
-  {
-    t << "{" << baseName << ".eps}";
-  }
-
-  if (hasCaption) 
-    t << "\\caption{";
-  else
-    t << "}" << endl;
-}
-
-void LatexGenerator::endDotFile(bool hasCaption)
-{
-  if (hasCaption)
-  {
-    t << "}" << endl;
-    t << "\\end{center}" << endl;
-    t << "\\end{figure}" << endl;
   }
 }
 
@@ -1597,7 +1384,7 @@ void LatexGenerator::startDotGraph()
 
 void LatexGenerator::endDotGraph(DotClassGraph &g) 
 {
-  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"));
+  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),relPath);
 }
 
 void LatexGenerator::startInclDepGraph() 
@@ -1606,7 +1393,16 @@ void LatexGenerator::startInclDepGraph()
 
 void LatexGenerator::endInclDepGraph(DotInclDepGraph &g) 
 {
-  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"));
+  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),relPath);
+}
+
+void LatexGenerator::startCallGraph() 
+{
+}
+
+void LatexGenerator::endCallGraph(DotCallGraph &g) 
+{
+  g.writeGraph(t,EPS,Config_getString("LATEX_OUTPUT"),relPath);
 }
 
 void LatexGenerator::startDescription() 
@@ -1671,19 +1467,6 @@ void LatexGenerator::endParamList()
   t << "\\end{Desc}" << endl;
 }
 
-void LatexGenerator::startSectionRefList()
-{
-  t << "\\footnotesize" << endl;
-  t << "\\begin{multicols}{2}" << endl;
-  t << "\\begin{CompactList}" << endl;
-}
-
-void LatexGenerator::endSectionRefList()
-{
-  t << "\\end{CompactList}" << endl;
-  t << "\\end{multicols}" << endl;
-  t << "\\normalsize" << endl;
-}
 
 void LatexGenerator::printDoc(DocNode *n)
 {
