@@ -2,7 +2,7 @@
  *
  * $Id: htmlgen.cpp,v 1.17 1998/11/28 11:33:19 root Exp $
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -29,15 +29,15 @@
 #include "doxygen.h"
 #include <string.h>
 #include "docparser.h"
+#include "mandocvisitor.h"
 
 static QCString getExtension()
 {
   QCString ext = Config_getString("MAN_EXTENSION");
   if( ext.length() >= 2 &&
-      ext.data()[0] == '.' &&
-      isdigit( ext.data()[1] ) )
+      ext.data()[0] == '.')
   {
-    ext = ext.mid(1, 1);
+    ext = ext.mid(1, ext.length()-1);
   }
   else
   {
@@ -61,21 +61,21 @@ ManGenerator::~ManGenerator()
 {
 }
 
-void ManGenerator::append(const OutputGenerator *g)
-{
-  QCString r=g->getContents();
-  if (upperCase)
-    t << r.upper();
-  else
-    t << r;
-  if (!r.isEmpty())
-    firstCol = r.at(r.length()-1)=='\n';
-  else
-    firstCol = ((ManGenerator *)g)->firstCol;
-  col+=((ManGenerator *)g)->col;
-  inHeader=((ManGenerator *)g)->inHeader;
-  paragraph=FALSE;
-}
+//void ManGenerator::append(const OutputGenerator *g)
+//{
+//  QCString r=g->getContents();
+//  if (upperCase)
+//    t << r.upper();
+//  else
+//    t << r;
+//  if (!r.isEmpty())
+//    firstCol = r.at(r.length()-1)=='\n';
+//  else
+//    firstCol = ((ManGenerator *)g)->firstCol;
+//  col+=((ManGenerator *)g)->col;
+//  inHeader=((ManGenerator *)g)->inHeader;
+//  paragraph=FALSE;
+//}
 
 void ManGenerator::init()
 {
@@ -94,6 +94,7 @@ void ManGenerator::init()
     err("Could not create output directory %s/man%s\n",manOutput.data(),ext.data());
     exit(1);
   }
+  createSubDirs(d);
 }
 
 static QCString buildFileName(const char *name)
@@ -136,8 +137,7 @@ static QCString buildFileName(const char *name)
   return fileName;
 }
 
-void ManGenerator::startFile(const char *,const char *manName,
-                             const char *,bool)
+void ManGenerator::startFile(const char *,const char *manName,const char *)
 {
   startPlainFile( buildFileName( manName ) );
   firstCol=TRUE;
@@ -153,6 +153,8 @@ void ManGenerator::endTitleHead(const char *,const char *name)
 {
   t << ".TH \"" << name << "\" " << getExtension() << " \"" 
     << dateToString(FALSE) << "\" \"";
+  if (!Config_getString("PROJECT_NUMBER").isEmpty())
+    t << "Version " << Config_getString("PROJECT_NUMBER") << "\" \"";
   if (Config_getString("PROJECT_NAME").isEmpty()) 
     t << "Doxygen";
   else
@@ -212,10 +214,10 @@ void ManGenerator::endHtmlLink()
 {
 }
 
-void ManGenerator::writeMailLink(const char *url)
-{
-  docify(url);
-}
+//void ManGenerator::writeMailLink(const char *url)
+//{
+//  docify(url);
+//}
 
 void ManGenerator::startGroupHeader()
 {
@@ -403,7 +405,7 @@ void ManGenerator::startDoxyAnchor(const char *,const char *manName,
     linkfile.close();
 }
 
-void ManGenerator::endMemberDoc()
+void ManGenerator::endMemberDoc(bool)
 {
     t << "\"";
 }
@@ -450,11 +452,11 @@ void ManGenerator::startDescItem()
   firstCol=FALSE;
 }
 
-void ManGenerator::endDescTitle()
-{
-  endBold();
-  paragraph=TRUE;
-}
+//void ManGenerator::endDescTitle()
+//{
+//  endBold();
+//  paragraph=TRUE;
+//}
 
 void ManGenerator::writeDescItem()
 {
@@ -472,21 +474,32 @@ void ManGenerator::endDescItem()
   firstCol=TRUE;
 }
 
-void ManGenerator::startMemberItem(int annType) 
+void ManGenerator::startAnonTypeScope(int indentLevel)
+{
+  if (indentLevel==0)
+  {
+    insideTabbing=TRUE;
+  }
+}
+
+void ManGenerator::endAnonTypeScope(int indentLevel)
+{
+  if (indentLevel==0)
+  {
+    insideTabbing=FALSE;
+  }
+}
+
+
+void ManGenerator::startMemberItem(int) 
 { 
   if (firstCol && !insideTabbing) t << ".in +1c\n";
   t << "\n.ti -1c\n.RI \""; 
   firstCol=FALSE;
-  if (annType!=0) insideTabbing=TRUE;
 }
 
-void ManGenerator::endMemberItem(bool endItem) 
+void ManGenerator::endMemberItem() 
 { 
-  if (endItem)
-  {
-    insideTabbing=FALSE;
-    t << "\"\n.br\n.RI \"";
-  }
   t << "\"\n.br"; 
 }
 
@@ -613,7 +626,11 @@ void ManGenerator::endParamList()
 {
 }
 
-void ManGenerator::printDoc(DocNode *)
+void ManGenerator::printDoc(DocNode *n)
 {
+  ManDocVisitor *visitor = new ManDocVisitor(t,*this);
+  n->accept(visitor);
+  delete visitor; 
+  firstCol=FALSE;
 }
 
