@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -21,6 +21,7 @@
 #include "qtbc.h"
 #include <qlist.h>
 #include <qdict.h>
+#include <qstack.h>
 
 #include "entry.h"
 #include "definition.h"
@@ -60,7 +61,8 @@ class MemberDef : public Definition
       Slot,
       Friend,
       DCOP,
-      Property
+      Property,
+      Event
     };
 
     MemberDef(const char *defFileName,int defLine,
@@ -72,13 +74,14 @@ class MemberDef : public Definition
     DefType definitionType() { return TypeMember; }
     
     QCString getOutputFileBase() const;
+    QCString getReference() const;
+    QCString anchor() const;
     const char *declaration() const       { return decl; }
     const char *definition() const        { return def; }
     const char *typeString() const        { return type; }
     const char *argsString() const        { return args; }
     const char *excpString() const        { return exception; }     
     const char *bitfieldString() const    { return bitfields; }     
-    QCString anchor() const;
     const QCString &initializer() const   { return init; }
     int initializerLines() const          { return initLines; }
     int  getMemberSpecifiers() const      { return memSpec; }
@@ -96,6 +99,7 @@ class MemberDef : public Definition
     const char *getGroupFileName() const     { return groupFileName; }
     int getGroupStartLine() const            { return groupStartLine; }
     bool getGroupHasDocs() const             { return groupHasDocs; }
+    QCString qualifiedName();
 
     // direct kind info 
     Protection protection() const         { return prot; }
@@ -114,16 +118,24 @@ class MemberDef : public Definition
     bool isFriend() const                 { return mtype==Friend;      }
     bool isDCOP() const                   { return mtype==DCOP;        }
     bool isProperty() const               { return mtype==Property;    }
+    bool isEvent() const                  { return mtype==Event;       }
     bool isRelated() const                { return related; }
     bool isStatic() const                 { return stat; }
     bool isInline() const                 { return (memSpec&Entry::Inline)!=0; }
     bool isExplicit() const               { return (memSpec&Entry::Explicit)!=0; }
     bool isMutable() const                { return (memSpec&Entry::Mutable)!=0; }
+    bool isGettable() const               { return (memSpec&Entry::Gettable)!=0; }
+    bool isSettable() const               { return (memSpec&Entry::Settable)!=0; }
+    bool isReadable() const               { return (memSpec&Entry::Readable)!=0; }
+    bool isWritable() const               { return (memSpec&Entry::Writable)!=0; }
+    bool isImplementation() const         { return m_implOnly; }
     bool isExternal() const               { return explExt; }
+    bool isObjCMethod() const;
     bool isConstructor() const;
     bool isDestructor() const;
     bool hasOneLineInitializer() const;
     bool hasMultiLineInitializer() const;
+    bool protectionVisible() const;
 
     // output info
     bool isLinkableInProject() const;
@@ -169,14 +181,24 @@ class MemberDef : public Definition
     MemberDef *reimplements() const;
     MemberList *reimplementedBy() const;
     
+    // in-body documentation
+    void setInbodyDocumentation(const char *docs,const char *file,int line);
+    int inbodyLine() const { return m_inbodyLine; }
+    QCString inbodyFile() const { return m_inbodyFile; }
+    const QCString &inbodyDocumentation() const { return m_inbodyDocs; }
+    
+    // For function documentation that can also be found in a class's related func section.
+    void setRelatedAlso(ClassDef *cd)     { m_relatedAlso=cd; }
+    ClassDef *relatedAlso() const         { return m_relatedAlso; }
+    
     // enumeration specific members
     void insertEnumField(MemberDef *md);
     void setEnumScope(MemberDef *md);
     MemberDef *getEnumScope() const          { return enumScope; }
-    void setEnumDecl(OutputList &ed);
     //void setEnumUsed()                       { eUsed=TRUE; }
     //bool enumUsed() const                    { return eUsed; }
-    OutputList *enumDecl() const             { return enumDeclList; }
+    //void setEnumDecl(OutputList &ed);
+    //OutputList *enumDecl() const             { return enumDeclList; }
     MemberList *enumFieldList() const        { return enumFields; }
     void setDocumentedEnumValues(bool value) { docEnumValues=value; }
     bool hasDocumentedEnumValues() const     { return docEnumValues; }
@@ -196,16 +218,8 @@ class MemberDef : public Definition
     // argument related members
     ArgumentList *argumentList() const { return defArgList; }
     ArgumentList *declArgumentList() const { return declArgList; }
-    void setArgumentList(ArgumentList *al) 
-    { 
-      if (defArgList) delete defArgList;
-      defArgList = al;
-    }
-    void setDeclArgumentList(ArgumentList *al)
-    {
-      if (declArgList) delete declArgList;
-      declArgList = al;
-    }
+    void setArgumentList(ArgumentList *al);
+    void setDeclArgumentList(ArgumentList *al);
     ArgumentList *templateArguments() const { return tArgList; }
     void setDefinitionTemplateParameterLists(QList<ArgumentList> *lists);
     QList<ArgumentList> *definitionTemplateParameterLists() const 
@@ -219,15 +233,20 @@ class MemberDef : public Definition
     MemberGroup *getMemberGroup() const { return memberGroup; }
     void setMemberGroupId(int id) { grpId=id; }
     int getMemberGroupId() const { return grpId; }
+    void makeImplementationDetail() { m_implOnly=TRUE; } 
     
     // anonymous scope members
     void setFromAnonymousScope(bool b) { annScope=b; }    
     void setFromAnonymousMember(MemberDef *m) { annMemb=m; }    
     bool fromAnonymousScope() const { return annScope; }
     bool anonymousDeclShown() const { return annUsed; }
-    void setIndentDepth( int i) { indDepth=i; }
-    int  indentDepth() { return indDepth; }
+    //void setIndentDepth( int i) { indDepth=i; }
+    //int  indentDepth() { return indDepth; }
 
+    // callgraph related members
+    bool hasCallGraph() const { return m_hasCallGraph; }
+    void enableCallGraph(bool e);
+    
     bool visibleMemberGroup(bool hideNoHeader);
     MemberDef *templateMaster() const { return m_templateMaster; } 
 
@@ -239,19 +258,31 @@ class MemberDef : public Definition
     void setTemplateMaster(MemberDef *mt) { m_templateMaster=mt; }
     void addListReference(Definition *d);
 
-    QCString getBodyAnchor() const 
-    { 
-      return bodyMemb ? bodyMemb->anchor() : anchor(); 
-    }
-    void setBodyMember(MemberDef *md) { bodyMemb = md; }
+    //QCString getBodyAnchor() const 
+    //{ 
+    //  return bodyMemb ? bodyMemb->anchor() : anchor(); 
+    //}
+    //void setBodyMember(MemberDef *md) { bodyMemb = md; }
     void setDocsForDefinition(bool b) { docsForDefinition = b; }
+    void setGroupAlias(MemberDef *md) { groupAlias = md; }
+    MemberDef *getGroupAlias() const { return groupAlias; }
 
+    // cached typedef functions
+    bool isTypedefValCached() const { return m_isTypedefValCached; }
+    ClassDef *getCachedTypedefVal() const { return m_cachedTypedefValue; }
+    void cacheTypedefVal(ClassDef *val) { m_isTypedefValCached=TRUE; m_cachedTypedefValue=val; }
+    
     // declaration <-> definition relation
     void setMemberDefinition(MemberDef *md) { memDef=md; }
     void setMemberDeclaration(MemberDef *md) { memDec=md; }
     MemberDef *memberDefinition() const { return memDef; }
     MemberDef *memberDeclaration() const { return memDec; }
         
+    void writeEnumDeclaration(OutputList &typeDecl,
+            ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd);
+
+    void findSectionsInDocumentation();
+    
     bool visited;
     
   private:
@@ -263,6 +294,7 @@ class MemberDef : public Definition
     MemberList *redefinedBy;  // the list of members that redefine this one
     MemberDef  *memDef;       // member definition for this declaration
     MemberDef  *memDec;       // member declaration for this definition
+    ClassDef   *m_relatedAlso;  // points to class marked by relatedAlso
 
     ExampleSDict *exampleSDict; // a dictionary of all examples for quick access
 
@@ -278,7 +310,7 @@ class MemberDef : public Definition
     QCString decl;            // member declaration in class
     QCString def;             // member definition in code (fully qualified name)
     QCString anc;             // HTML anchor name
-    MemberDef *bodyMemb;      // Member containing the definition
+    MemberDef *groupAlias;    // Member containing the definition
     Specifier virt;           // normal/virtual/pure virtual
     Protection prot;          // protection type [Public/Protected/Private]
     bool    related;          // is this a member that is only related to a class
@@ -290,7 +322,8 @@ class MemberDef : public Definition
     bool annScope;            // member is part of an annoymous scope
     bool annUsed;             
     bool annShown;           
-    int  indDepth;            // indentation depth for this member if inside an annonymous scope
+    bool m_hasCallGraph;
+    //int  indDepth;            // indentation depth for this member if inside an annonymous scope
     int  maxInitLines;        // when the initializer will be displayed 
     int userInitLines;        // result of explicit \hideinitializer or \showinitializer
     MemberList *section;      // declation list containing this member 
@@ -323,6 +356,19 @@ class MemberDef : public Definition
     // disable copying of member defs
     MemberDef(const MemberDef &);
     MemberDef &operator=(const MemberDef &);
+    static int s_indentLevel;
+
+    bool m_isTypedefValCached;
+    ClassDef *m_cachedTypedefValue;
+    
+    // inbody documentation
+    int m_inbodyLine;
+    QCString m_inbodyFile;
+    QCString m_inbodyDocs;
+
+    // objective-c
+    bool m_implOnly; // function found in implementation but not 
+                     // in the interface
 };
 
 #endif
