@@ -2,7 +2,7 @@
  *
  * $Id: outputlist.h,v 1.47 2001/03/19 19:27:41 root Exp $
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -36,16 +36,13 @@ class ClassDiagram;
 class DotClassGraph;
 class DotInclDepGraph;
 class DotGfxHierarchyTable;
+class SectionDict;
 
 class OutputList : public OutputDocInterface
 {
   public:
     OutputList(bool);
     virtual ~OutputList();
-    OutputList(const OutputList *ol);
-    OutputList &operator=(const OutputList &ol);
-    OutputList &operator+=(const OutputList &ol);
-
 
     void add(const OutputGenerator *);
     
@@ -64,19 +61,11 @@ class OutputList : public OutputDocInterface
     //////////////////////////////////////////////////
 
     void parseDoc(const char *fileName,int startLine,
-                  const char *clName,MemberDef *md,const QCString &docStr);
+                  Definition *ctx,MemberDef *md,const QCString &docStr,
+                  bool indexWords,bool isExample,const char *exampleName=0);
+    void parseText(const QCString &textStr);
     
-    OutputDocInterface *clone()
-    {
-      return new OutputList(this);
-    }
-    void append(const OutputDocInterface *g)
-    {
-      operator+=(*(OutputList *)g);
-    }
 
-    //void writeIndex() 
-    //{ forall(&OutputGenerator::writeIndex); }
     void startIndexSection(IndexSections is)
     { forall(&OutputGenerator::startIndexSection,is); }
     void endIndexSection(IndexSections is)
@@ -87,11 +76,10 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::endProjectNumber); }
     void writeStyleInfo(int part) 
     { forall(&OutputGenerator::writeStyleInfo,part); }
-    void startFile(const char *name,const char *manName,
-                   const char *title,bool external)
-    { forall(&OutputGenerator::startFile,name,manName,title,external); }
-    void writeFooter(int fase,bool external)
-    { forall(&OutputGenerator::writeFooter,fase,external); }
+    void startFile(const char *name,const char *manName,const char *title)
+    { forall(&OutputGenerator::startFile,name,manName,title); }
+    void writeFooter()
+    { forall(&OutputGenerator::writeFooter); }
     void endFile() 
     { forall(&OutputGenerator::endFile); }
     void startTitleHead(const char *fileName) 
@@ -122,16 +110,6 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startItemList); }
     void endItemList() 
     { forall(&OutputGenerator::endItemList); }
-    void startEnumList() 
-    { forall(&OutputGenerator::startEnumList); }
-    void endEnumList() 
-    { forall(&OutputGenerator::endEnumList); }
-    void startAlphabeticalIndexList()
-    { forall(&OutputGenerator::startAlphabeticalIndexList); }
-    void endAlphabeticalIndexList()
-    { forall(&OutputGenerator::endAlphabeticalIndexList); }
-    void writeIndexHeading(const char *s)
-    { forall(&OutputGenerator::writeIndexHeading,s); }
     void writeIndexItem(const char *ref,const char *file,const char *text)
     { forall(&OutputGenerator::writeIndexItem,ref,file,text); }
     void docify(const char *s)
@@ -152,8 +130,6 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startHtmlLink,url); }
     void endHtmlLink()
     { forall(&OutputGenerator::endHtmlLink); }
-    void writeMailLink(const char *url)
-    { forall(&OutputGenerator::writeMailLink,url); } 
     void writeStartAnnoItem(const char *type,const char *file, 
                             const char *path,const char *name)
     { forall(&OutputGenerator::writeStartAnnoItem,type,file,path,name); }
@@ -185,10 +161,14 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startMemberList); }
     void endMemberList() 
     { forall(&OutputGenerator::endMemberList); }
+    void startAnonTypeScope(int i1) 
+    { forall(&OutputGenerator::startAnonTypeScope,i1); }
+    void endAnonTypeScope(int i1) 
+    { forall(&OutputGenerator::endAnonTypeScope,i1); }
     void startMemberItem(int i1) 
     { forall(&OutputGenerator::startMemberItem,i1); }
-    void endMemberItem(bool b2) 
-    { forall(&OutputGenerator::endMemberItem,b2); }
+    void endMemberItem() 
+    { forall(&OutputGenerator::endMemberItem); }
     void startMemberGroupHeader(bool b) 
     { forall(&OutputGenerator::startMemberGroupHeader,b); }
     void endMemberGroupHeader()
@@ -211,22 +191,10 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startCodeFragment); }
     void endCodeFragment() 
     { forall(&OutputGenerator::endCodeFragment); }
-    void startPreFragment() 
-    { forall(&OutputGenerator::startPreFragment); }
-    void endPreFragment() 
-    { forall(&OutputGenerator::endPreFragment); }
-    void startVerbatimFragment() 
-    { forall(&OutputGenerator::startVerbatimFragment); }
-    void endVerbatimFragment() 
-    { forall(&OutputGenerator::endVerbatimFragment); }
     void startCodeLine() 
     { forall(&OutputGenerator::startCodeLine); }
     void endCodeLine() 
     { forall(&OutputGenerator::endCodeLine); }
-    //void startLineNumber() 
-    //{ forall(&OutputGenerator::startLineNumber); }
-    //void endLineNumber() 
-    //{ forall(&OutputGenerator::endLineNumber); }
     void writeLineNumber(const char *ref,const char *file,const char *anchor,
                          int lineNumber) 
     { forall(&OutputGenerator::writeLineNumber,ref,file,anchor,lineNumber); }
@@ -239,8 +207,8 @@ class OutputList : public OutputDocInterface
     void startMemberDoc(const char *clName,const char *memName,
                         const char *anchor,const char *title)
     { forall(&OutputGenerator::startMemberDoc,clName,memName,anchor,title); }
-    void endMemberDoc() 
-    { forall(&OutputGenerator::endMemberDoc); }
+    void endMemberDoc(bool hasArgs) 
+    { forall(&OutputGenerator::endMemberDoc,hasArgs); }
     void startDoxyAnchor(const char *fName,const char *manName,
                          const char *anchor, const char *name)
     { forall(&OutputGenerator::startDoxyAnchor,fName,manName,anchor,name); }
@@ -252,8 +220,6 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::endCodeAnchor); }
     void writeLatexSpacing() 
     { forall(&OutputGenerator::writeLatexSpacing); }
-    //void writeLatexLabel(const char *scope,const char *anchor) 
-    //{ forall(&OutputGenerator::writeLatexLabel,scope,anchor); }
     void startDescription() 
     { forall(&OutputGenerator::startDescription); }
     void endDescription() 
@@ -278,56 +244,12 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startSmall); }
     void endSmall() 
     { forall(&OutputGenerator::endSmall); }
-    void startSubscript() 
-    { forall(&OutputGenerator::startSubscript); }
-    void endSubscript() 
-    { forall(&OutputGenerator::endSubscript); }
-    void startSuperscript() 
-    { forall(&OutputGenerator::startSuperscript); }
-    void endSuperscript() 
-    { forall(&OutputGenerator::endSuperscript); }
-    void startTable(bool hasCaption,int cols)
-    { forall(&OutputGenerator::startTable,hasCaption,cols); }
-    void endTable(bool hasCaption) 
-    { forall(&OutputGenerator::endTable,hasCaption); }
-    void startCaption()
-    { forall(&OutputGenerator::startCaption); }
-    void endCaption() 
-    { forall(&OutputGenerator::endCaption); }
-    void nextTableRow() 
-    { forall(&OutputGenerator::nextTableRow); }
-    void endTableRow() 
-    { forall(&OutputGenerator::endTableRow); }
-    void nextTableColumn() 
-    { forall(&OutputGenerator::nextTableColumn); }
-    void endTableColumn() 
-    { forall(&OutputGenerator::endTableColumn); }
     void lineBreak() 
     { forall(&OutputGenerator::lineBreak); }
     void startBold() 
     { forall(&OutputGenerator::startBold); }
     void endBold() 
     { forall(&OutputGenerator::endBold); }
-    void writeCopyright() 
-    { forall(&OutputGenerator::writeCopyright); }
-    void writeQuote() 
-    { forall(&OutputGenerator::writeQuote); }
-    void writeUmlaut(char c)
-    { forall(&OutputGenerator::writeUmlaut,c); }
-    void writeAcute(char c)
-    { forall(&OutputGenerator::writeAcute,c); }
-    void writeGrave(char c)
-    { forall(&OutputGenerator::writeGrave,c); }
-    void writeCirc(char c)
-    { forall(&OutputGenerator::writeCirc,c); }
-    void writeTilde(char c)
-    { forall(&OutputGenerator::writeTilde,c); }
-    void writeRing(char c)
-    { forall(&OutputGenerator::writeRing,c); }
-    void writeSharpS()
-    { forall(&OutputGenerator::writeSharpS); }
-    void writeCCedil(char c)
-    { forall(&OutputGenerator::writeCCedil,c); }
     void startMemberDescription() 
     { forall(&OutputGenerator::startMemberDescription); }
     void endMemberDescription() 
@@ -341,8 +263,6 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startParamList,t,title); }
     void endParamList() 
     { forall(&OutputGenerator::endParamList); }
-    void endDescTitle() 
-    { forall(&OutputGenerator::endDescTitle); }
     void writeDescItem() 
     { forall(&OutputGenerator::writeDescItem); }
     void startIndent() 
@@ -353,13 +273,6 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startSection,lab,title,t); }
     void endSection(const char *lab,SectionInfo::SectionType t)
     { forall(&OutputGenerator::endSection,lab,t); }
-    void writeSectionRef(const char *ref,const char *file,
-                         const char *anchor, const char *title)
-    { forall(&OutputGenerator::writeSectionRef,ref,file,anchor,title); }
-    void writeSectionRefItem(const char *page,const char *lab, const char *title)
-    { forall(&OutputGenerator::writeSectionRefItem,page,lab,title); }
-    //void writeSectionRefAnchor(const char *page,const char *lab, const char *title)
-    //{ forall(&OutputGenerator::writeSectionRefAnchor,page,lab,title); }
     void addIndexItem(const char *s1,const char *s2)
     { forall(&OutputGenerator::addIndexItem,s1,s2); }
     void writeSynopsis() 
@@ -368,31 +281,18 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startClassDiagram); }
     void endClassDiagram(ClassDiagram &d,const char *f,const char *n)
     { forall(&OutputGenerator::endClassDiagram,d,f,n); }
-    //void startColorFont(uchar r,uchar g,uchar b)
-    //{ forall(&OutputGenerator::startColorFont,r,g,b); }
-    //void endColorFont()
-    //{ forall(&OutputGenerator::endColorFont); }
     void startPageRef()
     { forall(&OutputGenerator::startPageRef); }
     void endPageRef(const char *c,const char *a)
     { forall(&OutputGenerator::endPageRef,c,a); }
-    void startQuickIndexItem(const char *s,const char *l)
-    { forall(&OutputGenerator::startQuickIndexItem,s,l); }
-    void endQuickIndexItem()
-    { forall(&OutputGenerator::endQuickIndexItem); }
-    void writeFormula(const char *n,const char *t)
-    { forall(&OutputGenerator::writeFormula,n,t); }
+    //void startQuickIndexItem(const char *s,const char *l)
+    //{ forall(&OutputGenerator::startQuickIndexItem,s,l); }
+    //void endQuickIndexItem()
+    //{ forall(&OutputGenerator::endQuickIndexItem); }
+    void writeQuickLinks(bool compact,HighlightedItem hli)
+    { forall(&OutputGenerator::writeQuickLinks,compact,hli); }
     void writeNonBreakableSpace(int num)
     { forall(&OutputGenerator::writeNonBreakableSpace,num); }
-    void startImage(const char *n,const char *s,bool c)
-    { forall(&OutputGenerator::startImage,n,s,c); }
-    void endImage(bool c)
-    { forall(&OutputGenerator::endImage,c); }
-    void startDotFile(const char *n,bool c)
-    { forall(&OutputGenerator::startDotFile,n,c); }
-    void endDotFile(bool c)
-    { forall(&OutputGenerator::endDotFile,c); }
-
     void startDescTable()
     { forall(&OutputGenerator::startDescTable); }
     void endDescTable()
@@ -413,6 +313,10 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::startInclDepGraph); }
     void endInclDepGraph(DotInclDepGraph &g)
     { forall(&OutputGenerator::endInclDepGraph,g); }
+    void startCallGraph()
+    { forall(&OutputGenerator::startCallGraph); }
+    void endCallGraph(DotCallGraph &g)
+    { forall(&OutputGenerator::endCallGraph,g); }
     void writeGraphicalHierarchy(DotGfxHierarchyTable &g)
     { forall(&OutputGenerator::writeGraphicalHierarchy,g); }
     void startTextBlock(bool dense=FALSE)
@@ -421,55 +325,32 @@ class OutputList : public OutputDocInterface
     { forall(&OutputGenerator::endTextBlock); }
     void lastIndexPage()
     { forall(&OutputGenerator::lastIndexPage); }
-
     void startMemberDocPrefixItem()
     { forall(&OutputGenerator::startMemberDocPrefixItem); }
     void endMemberDocPrefixItem()
     { forall(&OutputGenerator::endMemberDocPrefixItem); }
-    void startMemberDocName()
-    { forall(&OutputGenerator::startMemberDocName); }
+    void startMemberDocName(bool align)
+    { forall(&OutputGenerator::startMemberDocName,align); }
     void endMemberDocName()
     { forall(&OutputGenerator::endMemberDocName); }
-    void startParameterType(bool first)
-    { forall(&OutputGenerator::startParameterType,first); }
+    void startParameterType(bool first,const char *key)
+    { forall(&OutputGenerator::startParameterType,first,key); }
     void endParameterType()
     { forall(&OutputGenerator::endParameterType); }
     void startParameterName(bool one)
     { forall(&OutputGenerator::startParameterName,one); }
-    void endParameterName(bool last,bool one)
-    { forall(&OutputGenerator::endParameterName,last,one); }
-    void startParameterList()
-    { forall(&OutputGenerator::startParameterList); }
+    void endParameterName(bool last,bool one,bool bracket)
+    { forall(&OutputGenerator::endParameterName,last,one,bracket); }
+    void startParameterList(bool openBracket)
+    { forall(&OutputGenerator::startParameterList,openBracket); }
     void endParameterList()
     { forall(&OutputGenerator::endParameterList); }
-
     void startFontClass(const char *c)
     { forall(&OutputGenerator::startFontClass,c); }
     void endFontClass()
     { forall(&OutputGenerator::endFontClass); }
-
-    void startHtmlOnly()
-    { forall(&OutputGenerator::startHtmlOnly); }
-    void endHtmlOnly()
-    { forall(&OutputGenerator::endHtmlOnly); }
-    void startLatexOnly()
-    { forall(&OutputGenerator::startLatexOnly); }
-    void endLatexOnly()
-    { forall(&OutputGenerator::endLatexOnly); }
-
-    void startSectionRefList()
-    { forall(&OutputGenerator::startSectionRefList); }
-    void endSectionRefList()
-    { forall(&OutputGenerator::endSectionRefList); }
-
     void writeCodeAnchor(const char *name)
     { forall(&OutputGenerator::writeCodeAnchor,name); }
-#if 0
-    void startPlainFile(const char *name)
-    { forall(&OutputGenerator::startPlainFile,name); }
-    void endPlainFile() 
-    { forall(&OutputGenerator::endPlainFile); }
-#else // this is to work around a bug in the SGI MipsPro compiler
     void startPlainFile(const char *name)
     { 
       OutputGenerator *og=outputs->first();
@@ -488,7 +369,6 @@ class OutputList : public OutputDocInterface
         og=outputs->next();
       }
     }
-#endif
 
   private:
     void debug();
@@ -501,15 +381,19 @@ class OutputList : public OutputDocInterface
     FORALLPROTO1(int);
     FORALLPROTO1(DotClassGraph &);
     FORALLPROTO1(DotInclDepGraph &);
+    FORALLPROTO1(DotCallGraph &);
     FORALLPROTO1(DotGfxHierarchyTable &);
     FORALLPROTO1(SectionTypes);
 #if defined(HAS_BOOL_TYPE) || defined(Q_HAS_BOOL_TYPE)
     FORALLPROTO1(bool);
     FORALLPROTO2(bool,int);
     FORALLPROTO2(bool,bool);
+    FORALLPROTO3(bool,bool,bool);
     FORALLPROTO4(const char *,const char *,const char *,int);
 #endif
     FORALLPROTO2(int,bool);
+    FORALLPROTO2(bool,HighlightedItem);
+    FORALLPROTO2(bool,const char *);
     FORALLPROTO2(ParamListTypes,const char *);
     FORALLPROTO2(const char *,const char *);
     FORALLPROTO2(const char *,bool);
