@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -278,7 +278,7 @@ class TranslatorCatalan : public TranslatorAdapter_1_2_17
         result+="documentats ";
       }
       result+=" amb enllaos a ";
-      if (extractAll) 
+      if (!extractAll) 
       {
         if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
         {
@@ -644,6 +644,8 @@ class TranslatorCatalan : public TranslatorAdapter_1_2_17
         case ClassDef::Struct:     result+=" Estructura "; break;
         case ClassDef::Union:      result+=" Uni "; break;
         case ClassDef::Interface:  result+=" Interfcie "; break;
+        case ClassDef::Protocol:   result+=" Protocol "; break; // translate me!
+        case ClassDef::Category:   result+=" Category "; break; // translate me!
         case ClassDef::Exception:  result+=" Excepci "; break;
       }
       if (isTemplate) result+="Template ";
@@ -801,6 +803,8 @@ class TranslatorCatalan : public TranslatorAdapter_1_2_17
         case ClassDef::Struct:     result+="estructura"; break;
         case ClassDef::Union:      result+="uni"; break;
         case ClassDef::Interface:  result+="interfcie"; break;
+        case ClassDef::Protocol:   result+="protocol"; break; // translate me!
+        case ClassDef::Category:   result+="category"; break; // translate me!
         case ClassDef::Exception:  result+="excepci"; break;
       }
       result+=" es va generar a partir del";
