@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -251,7 +251,7 @@ class TranslatorGreek : public TranslatorAdapter_1_2_11
         result+="";
       }
       result+="   ";
-      if (extractAll) 
+      if (!extractAll) 
       {
         if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
         {
@@ -619,6 +619,8 @@ class TranslatorGreek : public TranslatorAdapter_1_2_11
         case ClassDef::Struct:     result+=" "; break;
         case ClassDef::Union:      result+=" "; break;
         case ClassDef::Interface:  result+=" "; break;
+        case ClassDef::Protocol:   result+=" Protocol"; break; // translate me!
+        case ClassDef::Category:   result+=" Category"; break; // translate me!
         case ClassDef::Exception:  result+=" "; break;
       }
       if (isTemplate) result+=" Template";
@@ -776,6 +778,8 @@ class TranslatorGreek : public TranslatorAdapter_1_2_11
         case ClassDef::Struct:     result+="  "; break;
         case ClassDef::Union:      result+="  "; break;
         case ClassDef::Interface:  result+="  "; break;
+        case ClassDef::Protocol:   result+="Protocol"; break; // translate me!
+        case ClassDef::Category:   result+="Category"; break; // translate me!
         case ClassDef::Exception:  result+="  "; break;
       }
       result+="   ";
