@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -46,6 +46,12 @@
 //
 // 2001/11/13
 // - Added strings for 1.2.13
+//
+// 2003/02/26
+// - Added strings for 1.2.18
+//
+// 2003/04/29
+// - Added strings for 1.3.0
 
 #ifndef TRANSLATOR_HR_H
 #define TRANSLATOR_HR_H
@@ -56,22 +62,30 @@ class TranslatorCroatian : public Translator
         /*! to avoid macro redefinition from translator_cz.h */
         inline QCString decode(const QCString& sInput)
         { 
-#ifdef _WIN32
-                return ISO88592ToWin1250(sInput);
-#else
-                return sInput;
-#endif
+          if (Config_getBool("USE_WINDOWS_ENCODING"))
+          {
+            return ISO88592ToWin1250(sInput);
+          }
+          else
+          {
+            return sInput;
+          }
         }
   
   public:
     QCString idLanguage()
     { return "croatian"; }
     QCString idLanguageCharset()
-#ifdef _WIN32
-        { return "windows-1250"; }
-#else
-        { return "iso-8859-2"; }
-#endif
+    {
+      if (Config_getBool("USE_WINDOWS_ENCODING"))
+      {
+        return "windows-1250"; 
+      }
+      else
+      {
+        return "iso-8859-2";
+      }
+    }
     QCString latexLanguageSupportCommand()
     { return "\\usepackage[croatian]{babel}\n"; }
     QCString trRelatedFunctions()
@@ -172,7 +186,7 @@ class TranslatorCroatian : public Translator
 	  else
 		  result+="lanova struktura s linkovima na ";
 		  
-      if (extractAll)
+      if (!extractAll)
 	  {
 		  result+="dokumentaciju svakog lana:";
       }
@@ -356,6 +370,8 @@ class TranslatorCroatian : public Translator
         case ClassDef::Struct: result+=" strukture "; break;
         case ClassDef::Union:  result+=" unije "; break;
         case ClassDef::Interface:  result+=" suelja (interface) "; break;
+        case ClassDef::Protocol:   result+=" protocol "; break; // translate me!
+        case ClassDef::Category:   result+=" category "; break; // translate me!
         case ClassDef::Exception:  result+=" iznimke (exception) "; break;
       }
           result += clName;
@@ -504,6 +520,8 @@ class TranslatorCroatian : public Translator
         case ClassDef::Struct:     result+="strukture"; break;
         case ClassDef::Union:      result+="unije"; break;
         case ClassDef::Interface:  result+="suelja (interface)"; break;
+        case ClassDef::Protocol:   result+="protocol"; break; // translate me!
+        case ClassDef::Category:   result+="category"; break; // translate me!
         case ClassDef::Exception:  result+="iznimke (exception)"; break;
       }
       result+=" je generirana iz " + trFile(FALSE, single) + ": ";
@@ -1062,6 +1080,138 @@ class TranslatorCroatian : public Translator
     {
       return "Popis zastarjelih metoda";
     }
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.18
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a header for declaration section of the events found in 
+     * a C# program
+     */
+    virtual QCString trEvents()
+    {
+      return decode("Dogaaji");
+    }
+    /*! Header used for the documentation section of a class' events. */
+    virtual QCString trEventDocumentation()
+    {
+      return decode("Dokumentacija dogaaja");
+    }
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a heading for a list of Java class types with package scope.
+     */
+    virtual QCString trPackageTypes()
+    { 
+      return "Tipovi u paketu";
+    }
+    /*! Used as a heading for a list of Java class functions with package 
+     * scope. 
+     */
+    virtual QCString trPackageMembers()
+    { 
+      return "Funkcije u paketu";
+    }
+    /*! Used as a heading for a list of static Java class functions with 
+     *  package scope.
+     */
+    virtual QCString trStaticPackageMembers()
+    { 
+      return decode("Statike funkcije u paketu");
+    }
+    /*! Used as a heading for a list of Java class variables with package 
+     * scope.
+     */
+    virtual QCString trPackageAttribs()
+    { 
+      return "Atributi u paketu";
+    }
+    /*! Used as a heading for a list of static Java class variables with 
+     * package scope.
+     */
+    virtual QCString trStaticPackageAttribs()
+    { 
+      return decode("Statiki atributi u paketu");
+    }
+	//////////////////////////////////////////////////////////////////////////
+	// new since 1.3.1
+	//////////////////////////////////////////////////////////////////////////
+
+	/*! Used in the quick index of a class/file/namespace member list page 
+	*  to link to the unfiltered list of all members.
+	*/
+	virtual QCString trAll()
+	{
+		return "Sve";
+	}
+	/*! Put in front of the call graph for a function. */
+	virtual QCString trCallGraph()
+	{
+		return "Ovo je dijagram poziva za ovu funkciju:";
+	}
+
+	//////////////////////////////////////////////////////////////////////////
+	// new since 1.3.3
+	//////////////////////////////////////////////////////////////////////////
+
+	/*! When the search engine is enabled this text is put in the header 
+	*  of each page before the field where one can enter the text to search 
+	*  for. 
+	*/
+	virtual QCString trSearchForIndex()
+	{
+		return decode("Trai");
+	}
+	/*! This string is used as the title for the page listing the search
+	*  results.
+	*/
+	virtual QCString trSearchResultsTitle()
+	{
+		return "Rezultati pretrage";
+	}
+	/*! This string is put just before listing the search results. The
+	*  text can be different depending on the number of documents found.
+	*  Inside the text you can put the special marker $num to insert
+	*  the number representing the actual number of search results.
+	*  The @a numDocuments parameter can be either 0, 1 or 2, where the 
+	*  value 2 represents 2 or more matches. HTML markup is allowed inside
+	*  the returned string.
+	*/
+	virtual QCString trSearchResults(int numDocuments)
+	{
+		if (numDocuments==0)
+		{
+			return decode("Nema dokumenta koji odgovaraju vaem upitu");
+		}
+		else if (numDocuments==1)
+		{
+			return decode("Naen <b>1</b> dokument koji odgovara vaem upitu.");
+		}
+		else if (numDocuments<5) 
+		{
+			// croatian handle numbers up to 5 differently 
+			// For those of you who like to read comments :
+			// There are only 4 men in the world in croatian.
+			// We say 1 ovjek (man), 2 ovjeka, 3 ovjeka , 4 ovjeka , but 5, 6, 7 ... ljudi (people)
+			// That's deep, isn't it :-)?
+			return decode("Naena <b>$num</b> dokumenta koji odgovaraju vaem upitu."
+				"Najbolji su prikazani prvi.");
+		}
+		else 
+		{
+			return decode("Naeno <b>$num</b> dokumenata koji odgovaraju vaem upitu."
+				"Najbolji su prikazani prvi.");
+		}
+	}
+	/*! This string is put before the list of matched words, for each search 
+	*  result. What follows is the list of words that matched the query.
+	*/
+	virtual QCString trSearchMatches()
+	{
+		return decode("Pronaeno:");
+	}
+
 };
 
 #endif
