@@ -2,7 +2,7 @@
  *
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -19,6 +19,10 @@
  *
  * Revision history
  *
+ *  2003/11: translated new items used since version 1.3.3
+ *  2003/06: translated new items used since version 1.3.1
+ *  2003/04: translated new items used since version 1.3
+ *  2002/11: translated new items used since version 1.2.18
  *  2002/08: translated new items used since version 1.2.17
  *  2002/07: translated new items used since version 1.2.16
  *  2002/06: modified trRelatedPagesDescription() method
@@ -57,7 +61,7 @@
  *
  * Il plurale dei termini inglesi non tradotti  stato reso con il singolare
  * della parola inglese secondo una convenzione spesso ritrovata nella documentazione
- * tecnica (ad es "lista dei file" e non "lista dei files")
+ * tecnica (ad es. "lista dei file" e non "lista dei files")
  *
  * Se avete suggerimenti sulla traduzione di alcuni termini o volete segnalare
  * eventuali sviste potete scrivermi all'indirizzo: alessandro@falappa.net
@@ -180,7 +184,7 @@ class TranslatorItalian : public Translator
     // quick reference sections
 
     /*! This is put above each page as a link to the list of all groups of
-     *  compounds or files (see the \group command).
+     *  compounds or files (see the \\group command).
      */
     QCString trModules()
     { return "Moduli"; }
@@ -640,6 +644,8 @@ class TranslatorItalian : public Translator
         case ClassDef::Struct: result+="la struct "; break;
         case ClassDef::Union:  result+="la union "; break;
         case ClassDef::Interface:  result+="l'interfaccia "; break;
+        case ClassDef::Protocol:   result+="protocol "; break; // translate me!
+        case ClassDef::Category:   result+="category "; break; // translate me!
         case ClassDef::Exception:  result+="exception "; break;
       }
       result+=(QCString)clName;
@@ -799,6 +805,8 @@ class TranslatorItalian : public Translator
         case ClassDef::Struct:     result+="struct"; break;
         case ClassDef::Union:      result+="union"; break;
         case ClassDef::Interface:  result+="interfaccia"; break;
+        case ClassDef::Protocol:   result+="protocol"; break; // translate me!
+        case ClassDef::Category:   result+="category"; break; // translate me!
         case ClassDef::Exception:  result+="exception"; break;
       }
       result+="  stata generata a partire ";
@@ -1370,6 +1378,132 @@ class TranslatorItalian : public Translator
     {
       return "Lista degli elementi deprecati";
     }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.18
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a header for declaration section of the events found in
+     * a C# program
+     */
+    virtual QCString trEvents()
+    {
+      return "Eventi";
+    }
+    /*! Header used for the documentation section of a class' events. */
+    virtual QCString trEventDocumentation()
+    {
+      return "Documentazione degli eventi";
+    }
+
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a heading for a list of Java class types with package scope.
+     */
+    virtual QCString trPackageTypes()
+    {
+      return "Tipi con visibilit di package";
+    }
+    /*! Used as a heading for a list of Java class functions with package
+     * scope.
+     */
+    virtual QCString trPackageMembers()
+    {
+      return "Funczioni con visibilit di package";
+    }
+    /*! Used as a heading for a list of static Java class functions with
+     *  package scope.
+     */
+    virtual QCString trStaticPackageMembers()
+    {
+      return "Funzioni statiche con visibilit di package";
+    }
+    /*! Used as a heading for a list of Java class variables with package
+     * scope.
+     */
+    virtual QCString trPackageAttribs()
+    {
+      return "Attributi con visibilit di package";
+    }
+    /*! Used as a heading for a list of static Java class variables with
+     * package scope.
+     */
+    virtual QCString trStaticPackageAttribs()
+    {
+      return "Attributi statici con visibilit di package";
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3.1
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used in the quick index of a class/file/namespace member list page
+     *  to link to the unfiltered list of all members.
+     */
+    virtual QCString trAll()
+    {
+      return "Tutto";
+    }
+    /*! Put in front of the call graph for a function. */
+    virtual QCString trCallGraph()
+    {
+      return "Questo  il grafo delle chiamate per questa funzione:";
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3.3
+//////////////////////////////////////////////////////////////////////////
+
+    /*! When the search engine is enabled this text is put in the header
+     *  of each page before the field where one can enter the text to search
+     *  for.
+     */
+    virtual QCString trSearchForIndex()
+    {
+      return "Cerca";
+    }
+    /*! This string is used as the title for the page listing the search
+     *  results.
+     */
+    virtual QCString trSearchResultsTitle()
+    {
+      return "Risultati della ricerca";
+    }
+    /*! This string is put just before listing the search results. The
+     *  text can be different depending on the number of documents found.
+     *  Inside the text you can put the special marker $num to insert
+     *  the number representing the actual number of search results.
+     *  The @a numDocuments parameter can be either 0, 1 or 2, where the
+     *  value 2 represents 2 or more matches. HTML markup is allowed inside
+     *  the returned string.
+     */
+    virtual QCString trSearchResults(int numDocuments)
+    {
+      if (numDocuments==0)
+      {
+        return "Nessun documento soddisfa la tua richiesta.";
+      }
+      else if (numDocuments==1)
+      {
+        return "Trovato <b>1</b> documento rispondente alla tua richiesta.";
+      }
+      else
+      {
+        return "Trovati <b>$num</b> documenti rispondenti alla tua richiesta. "
+               "Le corrispondenze migliori sono in testa.";
+      }
+    }
+    /*! This string is put before the list of matched words, for each search
+     *  result. What follows is the list of words that matched the query.
+     */
+    virtual QCString trSearchMatches()
+    {
+      return "Corrispondenze:";
+    }
+
 };
 
 #endif
