@@ -2,7 +2,7 @@
  *
  *
  *
- * Copyright (C) 1997-2000 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -29,50 +29,21 @@
 #ifndef TRANSLATOR_JP_H
 #define TRANSLATOR_JP_H
 
-class TranslatorJapaneseEn : public TranslatorEnglish
-{
-  public:
-    virtual QCString idLanguage()
-    { return "japanese"; }
-    virtual QCString latexLanguageSupportCommand()
-      {
-	return "";
-      }
-    /*! returns the name of the package that is included by LaTeX */
-    virtual QCString idLanguageCharset()
-    {
-#ifdef __CYGWIN__
-      return "Shift_JIS";
-#else
-      return "euc-jp";
-#endif
-    }
-    virtual QCString trRTFansicp()
-    {
-      return "932";
-    }
-
-    /*! Used as ansicpg for RTF fcharset
-     *  \see trRTFansicp() for a table of possible values.
-     */
-    virtual QCString trRTFCharSet()
-    {
-      return "128";
-    }
-};
-
-class TranslatorJapanese : public Translator
+class TranslatorJapanese : public TranslatorAdapter_1_3_3
 {
  private:
   /*! The decode() can change euc into sjis */
   inline QCString decode(const QCString & sInput)
+  {
+    if (Config_getBool("USE_WINDOWS_ENCODING"))
     {
-#ifdef __CYGWIN__
       return JapaneseEucToSjis(sInput);
-#else
-      return sInput;
-#endif
     }
+    else
+    {
+      return sInput;
+    }
+  }
   public:
     virtual QCString idLanguage()
     { return "japanese"; }
@@ -83,11 +54,14 @@ class TranslatorJapanese : public Translator
     /*! returns the name of the package that is included by LaTeX */
     virtual QCString idLanguageCharset()
     {
-#ifdef __CYGWIN__
-      return "Shift_JIS";
-#else
-      return "euc-jp";
-#endif
+      if (Config_getBool("USE_WINDOWS_ENCODING"))
+      {
+        return "Shift_JIS";
+      }
+      else
+      {
+        return "euc-jp";
+      }
     }
 
     /*! used in the compound documentation before a list of related functions. */
@@ -104,26 +78,35 @@ class TranslatorJapanese : public Translator
 
     /*! header that is put before the list of typedefs. */
     virtual QCString trMemberTypedefDocumentation()
-    { return decode("Typedef β"); }
+    { return decode(""); }
 
     /*! header that is put before the list of enumerations. */
     virtual QCString trMemberEnumerationDocumentation()
-    { return decode("Enum β"); }
+    { return decode(""); }
 
     /*! header that is put before the list of member functions. */
     virtual QCString trMemberFunctionDocumentation()
-    { return decode("᥽åɤβ"); }
+    { 
+	  if( Config_getBool("OPTIMIZE_OUTPUT_JAVA"))
+	  {
+		return decode("᥽å");
+	  }
+	  else
+	  {
+		return decode("ؿ");
+	  }
+	}
 
     /*! header that is put before the list of member attributes. */
     virtual QCString trMemberDataDocumentation()
     {
       if( Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
 	{
-	  return decode("¤Τβ");
+	  return decode("¤");
 	}
       else
 	{
-	  return decode("ѿβ");
+	  return decode("ѿ");
 	}
     }
 
@@ -172,7 +155,7 @@ class TranslatorJapanese : public Translator
     // quick reference sections
 
     /*! This is put above each page as a link to the list of all groups of
-     *  compounds or files (see the \group command).
+     *  compounds or files (see the \\group command).
      */
     virtual QCString trModules()
     { return decode("⥸塼"); }
@@ -258,11 +241,11 @@ class TranslatorJapanese : public Translator
     {
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
 	{
-	  return decode("ǡ¤βǤ");
+	  return decode("ǡ¤Ǥ");
 	}
       else
 	{
-	  return decode("饹¤ΡΡ󥿥եβǤ");
+	  return decode("饹¤ΡΡ󥿥եǤ");
 	}
     }
 
@@ -280,7 +263,7 @@ class TranslatorJapanese : public Translator
 	  result+=decode("饹Фΰǡ줾");
 	  if (extractAll) result+=decode("°Ƥ륯饹");
 	}
-      result+=decode("βإ󥯤Ƥޤ");
+      result+=decode("إ󥯤Ƥޤ");
       return result;
     }
 
@@ -296,7 +279,7 @@ class TranslatorJapanese : public Translator
 	{
 	  result+=decode("եФ");
 	}
-      result+=decode("Ǥ줾줬°Ƥեβإ󥯤Ƥޤ");
+      result+=decode("Ǥ줾줬°Ƥեإ󥯤Ƥޤ");
       return result;
     }
 
@@ -366,7 +349,7 @@ class TranslatorJapanese : public Translator
      *  the documentation of all groups.
      */
     virtual QCString trModuleDocumentation()
-    { return decode("⥸塼β"); }
+    { return decode("⥸塼"); }
 
     /*! This is used in LaTeX as the title of the chapter containing
      *  the documentation of all classes, structs and unions.
@@ -375,11 +358,11 @@ class TranslatorJapanese : public Translator
     {
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
 	{
-	  return decode("ǡ¤β");
+	  return decode("ǡ¤");
 	}
       else
 	{
-	  return decode("饹β");
+	  return decode("饹");
 	}
     }
 
@@ -387,19 +370,19 @@ class TranslatorJapanese : public Translator
      *  the documentation of all files.
      */
     virtual QCString trFileDocumentation()
-    { return decode("եβ"); }
+    { return decode("ե"); }
 
     /*! This is used in LaTeX as the title of the chapter containing
      *  the documentation of all examples.
      */
     virtual QCString trExampleDocumentation()
-    { return decode("β"); }
+    { return decode(""); }
 
     /*! This is used in LaTeX as the title of the chapter containing
      *  the documentation of all related pages.
      */
     virtual QCString trPageDocumentation()
-    { return decode("ڡβ"); }
+    { return decode("ڡ"); }
 
     /*! This is used in LaTeX as the title of the document */
     virtual QCString trReferenceManual()
@@ -421,13 +404,13 @@ class TranslatorJapanese : public Translator
      *  list of typedefs
      */
     virtual QCString trTypedefs()
-    { return decode("Typedef"); }
+    { return decode(""); }
 
     /*! This is used in the documentation of a file as a header before the
      *  list of enumerations
      */
     virtual QCString trEnumerations()
-    { return decode("Enum"); }
+    { return decode(""); }
 
     /*! This is used in the documentation of a file as a header before the
      *  list of (global) functions
@@ -445,48 +428,48 @@ class TranslatorJapanese : public Translator
      *  list of (global) variables
      */
     virtual QCString trEnumerationValues()
-      { return decode("Enum "); }
+      { return decode("󷿤"); }
     /*! This is used in the documentation of a file before the list of
      *  documentation blocks for defines
      */
     virtual QCString trDefineDocumentation()
-    { return decode("ޥβ"); }
+    { return decode("ޥ"); }
 
     /*! This is used in the documentation of a file/namespace before the list
      *  of documentation blocks for function prototypes
      */
     virtual QCString trFunctionPrototypeDocumentation()
-    { return decode("ؿץȥפβ"); }
+    { return decode("ؿץȥ"); }
 
     /*! This is used in the documentation of a file/namespace before the list
      *  of documentation blocks for typedefs
      */
     virtual QCString trTypedefDocumentation()
-    { return decode("Typedef β"); }
+    { return decode(""); }
 
     /*! This is used in the documentation of a file/namespace before the list
      *  of documentation blocks for enumeration types
      */
     virtual QCString trEnumerationTypeDocumentation()
-    { return decode("Enum β"); }
+    { return decode(""); }
 
     /*! This is used in the documentation of a file/namespace before the list
      *  of documentation blocks for enumeration values
      */
     virtual QCString trEnumerationValueDocumentation()
-    { return decode("Enum ͤβ"); }
+    { return decode("󷿤"); }
 
     /*! This is used in the documentation of a file/namespace before the list
      *  of documentation blocks for functions
      */
     virtual QCString trFunctionDocumentation()
-    { return decode("ؿβ"); }
+    { return decode("ؿ"); }
 
     /*! This is used in the documentation of a file/namespace before the list
      *  of documentation blocks for variables
      */
     virtual QCString trVariableDocumentation()
-    { return decode("ѿβ"); }
+    { return decode("ѿ"); }
 
     /*! This is used in the documentation of a file/namespace/group before
      *  the list of links to documented compounds
@@ -576,13 +559,13 @@ class TranslatorJapanese : public Translator
 
     /*! used as the title of page containing all the index of all namespaces. */
     virtual QCString trNamespaceList()
-    { return decode("ְ̾"); }
+    { return decode("͡ॹڡ"); }
 
     /*! used as an introduction to the namespace list */
     virtual QCString trNamespaceListDescription(bool /*extractAll*/)
     {
       QCString result=decode("");
-      result+=decode("֤̾ΰǤ");
+      result+=decode("͡ॹڡΰǤ");
       return result;
     }
 
@@ -600,7 +583,7 @@ class TranslatorJapanese : public Translator
      * related classes
      */
     virtual QCString trRelatedFunctionDocumentation()
-    { return decode("եɤȴϢؿβ"); }
+    { return decode("եɤȴϢؿ"); }
 
 //////////////////////////////////////////////////////////////////////////
 // new since 0.49-990425
@@ -618,24 +601,26 @@ class TranslatorJapanese : public Translator
         case ClassDef::Struct:     result+=decode("¤ "); break;
         case ClassDef::Union:      result+=decode(" "); break;
         case ClassDef::Interface:  result+=decode("󥿥ե"); break;
+        case ClassDef::Protocol:   result+=decode("Protocol"); break; // translate me!
+        case ClassDef::Category:   result+=decode("Category"); break; // translate me!
         case ClassDef::Exception:  result+=decode("㳰"); break; //TODO:fixme
       }
       if (isTemplate) result+=decode(" ƥץ졼");
-      result+=(QCString)clName+decode(" β");
+		result+=(QCString)clName;
       return result;
     }
 
     /*! used as the title of the HTML page of a file */
     virtual QCString trFileReference(const char *fileName)
     {
-      QCString result=decode("")+(QCString)fileName+decode(" β");
+      QCString result=decode("")+(QCString)fileName;
       return result;
     }
 
     /*! used as the title of the HTML page of a namespace */
     virtual QCString trNamespaceReference(const char *namespaceName)
     {
-      QCString result=decode("̾ ")+(QCString)namespaceName+decode(" β");
+      QCString result=decode("͡ॹڡ ")+(QCString)namespaceName;
       return result;
     }
 
@@ -720,17 +705,17 @@ class TranslatorJapanese : public Translator
 
     /*! This is put above each page as a link to all members of namespaces. */
     virtual QCString trNamespaceMembers()
-    { return decode("֥̾"); }
+    { return decode("͡ॹڡ"); }
 
     /*! This is an introduction to the page with all namespace members */
     virtual QCString trNamespaceMemberDescription(bool extractAll)
     {
 	QCString result=decode("");
-      result+=decode("֤̾ΰǤ줾");
+      result+=decode("͡ॹڡΰǤ줾");
       if (extractAll)
-	  result+=decode("֤̾β");
+	  result+=decode("Υ͡ॹڡ");
       else
-	  result+=decode("°Ƥ̾");
+	  result+=decode("°Ƥ͡ॹڡ");
       result+=decode("إ󥯤Ƥޤ");
       return result;
     }
@@ -738,13 +723,13 @@ class TranslatorJapanese : public Translator
      *  index of all namespaces.
      */
     virtual QCString trNamespaceIndex()
-    { return decode("ֺ̾"); }
+    { return decode("͡ॹڡ"); }
 
     /*! This is used in LaTeX as the title of the chapter containing
      *  the documentation of all namespaces.
      */
     virtual QCString trNamespaceDocumentation()
-    { return decode("֤̾β"); }
+    { return decode("͡ॹڡ"); }
 
 //////////////////////////////////////////////////////////////////////////
 // new since 0.49-990522
@@ -754,7 +739,7 @@ class TranslatorJapanese : public Translator
      *  namespaces in a file.
      */
     virtual QCString trNamespaces()
-    { return decode("̾"); }
+    { return decode("͡ॹڡ"); }
 
 //////////////////////////////////////////////////////////////////////////
 // new since 0.49-990728
@@ -774,9 +759,11 @@ class TranslatorJapanese : public Translator
         case ClassDef::Struct:     result+=decode("¤"); break;
         case ClassDef::Union:      result+=decode(""); break;
         case ClassDef::Interface:  result+=decode("󥿥ե"); break;
+        case ClassDef::Protocol:   result+=decode("Protocol"); break; // translate me!
+        case ClassDef::Category:   result+=decode("Category"); break; // translate me!
         case ClassDef::Exception:  result+=decode("㳰"); break; //TODO:fixme
       }
-      result+=decode("βϼΥե뤫ޤ:");
+      result+=decode("ϼΥե뤫ޤ:");
       return result;
     }
 
@@ -848,7 +835,7 @@ class TranslatorJapanese : public Translator
     /*! header that is put before the list of constructor/destructors. */
     virtual QCString trConstructorDocumentation()
     {
-      return decode("󥹥ȥ饯ȥǥȥ饯β");
+      return decode("󥹥ȥ饯ȥǥȥ饯");
     }
     /*! Used in the file documentation to point to the corresponding sources. */
     virtual QCString trGotoSourceCode()
@@ -975,7 +962,7 @@ class TranslatorJapanese : public Translator
 
     virtual QCString trReferencedBy()
     {
-      return decode("ƽ");
+      return decode("ȸ");
     }
     virtual QCString trRemarks()
     {
@@ -1103,7 +1090,7 @@ class TranslatorJapanese : public Translator
     /*! Used as a section header for IDL property documentation */
     virtual QCString trPropertyDocumentation()
     {
-      return decode("ץѥƥβ");
+      return decode("ץѥƥ");
     }
 
 
@@ -1151,7 +1138,7 @@ class TranslatorJapanese : public Translator
     /*! Used as a chapter title for Latex & RTF output */
     virtual QCString trPackageDocumentation()
     {
-      return decode("ѥåβ");
+      return decode("ѥå");
     }
     /*! Text shown before a multi-line define */
     virtual QCString trDefineValue()
@@ -1256,7 +1243,7 @@ class TranslatorJapanese : public Translator
     {
       first_capital = first_capital;
       singular = singular;
-      QCString result(decode("̾"));
+      QCString result(decode("͡ॹڡ"));
       return result;
     }
 
@@ -1385,7 +1372,80 @@ class TranslatorJapanese : public Translator
      */
     virtual QCString trDeprecatedList()
     {
-      return "侩";
+      return decode("侩");
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.18
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a header for declaration section of the events found in 
+     * a C# program
+     */
+    virtual QCString trEvents()
+    {
+      return decode("٥");
+    }
+    /*! Header used for the documentation section of a class' events. */
+    virtual QCString trEventDocumentation()
+    {
+      return decode("٥");
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a heading for a list of Java class types with package scope.
+     */
+    virtual QCString trPackageTypes()
+    { 
+      return decode("ѥåη");
+    }
+    /*! Used as a heading for a list of Java class functions with package 
+     * scope. 
+     */
+    virtual QCString trPackageMembers()
+    { 
+      return decode("ؿ");
+    }
+    /*! Used as a heading for a list of static Java class functions with 
+     *  package scope.
+     */
+    virtual QCString trStaticPackageMembers()
+    { 
+      return decode("ƥåؿ");
+    }
+    /*! Used as a heading for a list of Java class variables with package 
+     * scope.
+     */
+    virtual QCString trPackageAttribs()
+    { 
+      return decode("ѿ");
+    }
+    /*! Used as a heading for a list of static Java class variables with 
+     * package scope.
+     */
+    virtual QCString trStaticPackageAttribs()
+    { 
+      return decode("ƥåѿ");
+    }
+    
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3.1
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used in the quick index of a class/file/namespace member list page 
+     *  to link to the unfiltered list of all members.
+     */
+    virtual QCString trAll()
+    {
+      return decode("");
+    }
+    /*! Put in front of the call graph for a function. */
+    virtual QCString trCallGraph()
+    {
+      return decode("ؿθƤӽФ:");
     }
 };
 #endif
