@@ -2,7 +2,7 @@
  *
  * $Id: translator_no.h,v 1.11 2000/07/22 16:59:12 root Exp $
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -144,7 +144,7 @@ class TranslatorNorwegian : public TranslatorAdapter_1_2_2
     QCString result="Her flger en liste over alle ";
       if (!extractAll) result+="dokumenterte ";
       result+="klassemedlemmer med lenker til ";
-      if (extractAll) result+="klassedokumentasjonen for hvert medlem:";
+      if (!extractAll) result+="klassedokumentasjonen for hvert medlem:";
       else result+="klassene som de tilhrer:";
       return result;
     }
@@ -346,6 +346,8 @@ class TranslatorNorwegian : public TranslatorAdapter_1_2_2
         case ClassDef::Struct: result+=" struktur"; break;
         case ClassDef::Union:  result+=" union"; break;
         case ClassDef::Interface:  result+=" grensesnitt"; break;
+        case ClassDef::Protocol:  result+=" protocol"; break; // translate me!
+        case ClassDef::Category:  result+=" category"; break; // translate me!
         case ClassDef::Exception:  result+=" unntak"; break;
       }
       result+="referanse";
@@ -484,6 +486,8 @@ class TranslatorNorwegian : public TranslatorAdapter_1_2_2
         case ClassDef::Struct:     result+="denne strukt "; break;
         case ClassDef::Union:      result+="denne union "; break;
         case ClassDef::Interface:  result+="dette grensesnittet "; break;
+        case ClassDef::Protocol:   result+="protocol"; break; // translate me!
+        case ClassDef::Category:   result+="category"; break; // translate me!
         case ClassDef::Exception:  result+="unntak "; break;
       }
       result+="var generert fra flgende fil";
