@@ -2,7 +2,7 @@
  *
  * $Id: translator_pl.h,v 1.3 2001/03/19 19:27:42 root Exp $
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -13,17 +13,28 @@
  * Documents produced by Doxygen are derivative works derived from the
  * input used in their production; they are not affected by this license.
  *
- * Polish translation was updated to version 1.2.17 by
+ * Polish translation was updated to version 1.3 by
  * Piotr Kaminski (Piotr.Kaminski@ctm.gdynia.pl)
  */
 
 #ifndef TRANSLATOR_PL_H
 #define TRANSLATOR_PL_H
 
-#include "translator_adapter.h"
-
-class TranslatorPolish : public TranslatorAdapter_1_2_17
+class TranslatorPolish : public TranslatorAdapter_1_3
 {
+  private:
+    /*! to avoid macro redefinition from translator_pl.h */
+    inline QCString decode(const QCString& sInput)
+    { 
+      if (Config_getBool("USE_WINDOWS_ENCODING"))
+      {
+        return ISO88592ToWin1250(sInput);
+      }  
+      else
+      {   
+        return sInput;  
+      }  
+    }
   public:
 
     // --- Language control methods -------------------
@@ -52,58 +63,67 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
 
     /*! return the language charset. This will be used for the HTML output */
     virtual QCString idLanguageCharset()
-    { return "iso-8859-2"; }
-
+    {
+      if (Config_getBool("USE_WINDOWS_ENCODING"))
+      {
+        return "windows-1250";
+      }  
+      else
+      {   
+        return "iso-8859-2"; 
+      }  
+    }
+    
     // --- Language translation methods -------------------
 
     /*! used in the compound documentation before a list of related functions. */
     QCString trRelatedFunctions()
-    { return "Funkcje Powizane"; }
+    { return decode("Funkcje powizane"); }
 
     /*! subscript for the related functions. */
     QCString trRelatedSubscript()
-    { return "(Zauwa, e to nie s metody klas.)"; }
+    { return decode("(Zauwa, e to nie s metody klas.)"); }
 
     /*! header that is put before the detailed description of files, classes and namespaces. */
     QCString trDetailedDescription()
-    { return "Opis Szczegowy"; }
+    { return decode("Opis szczegowy"); }
 
     /*! header that is put before the list of typedefs. */
     QCString trMemberTypedefDocumentation()
-    { return "Dokumentacja Skadowych Definicji Typu"; }
+    { return decode("Dokumentacja skadowych definicji typu"); }
     
     /*! header that is put before the list of enumerations. */
     QCString trMemberEnumerationDocumentation()
-    { return "Dokumentacja Skadowych Wyliczanych"; }
+    { return decode("Dokumentacja skadowych wyliczanych"); }
     
     /*! header that is put before the list of member functions. */
     QCString trMemberFunctionDocumentation()
-    { return "Dokumentacja Funkcji Skadowych"; }
+    { return decode("Dokumentacja funkcji skadowych"); }
     
     /*! header that is put before the list of member attributes. */
     QCString trMemberDataDocumentation()
     { 
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        return "Dokumentacja Pl"; 
+        return decode("Dokumentacja pl"); 
       }
       else
       {
-        return "Dokumentacja Atrybutw Skadowych"; 
+        return decode("Dokumentacja atrybutw skadowych"); 
       }
     }
 
     /*! this is the text of a link put after brief descriptions. */
     QCString trMore() 
-    { return "Wicej..."; }
+    { return decode("Wicej..."); }
 
     /*! put in the class documentation */
     QCString trListOfAllMembers()
-    { return "Lista wszystkich skadowych."; }
+    { return decode("Lista wszystkich skadowych."); }
 
     /*! used as the title of the "list of all members" page of a class */
     QCString trMemberList()
-    { return "Lista Skadowych"; }
+    { return decode("Lista skadowych"); }
 
     /*! this is the first part of a sentence that is followed by a class name */
     QCString trThisIsTheListOfAllMembers()
@@ -111,13 +131,13 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
 
     /*! this is the remainder of the sentence after the class name */
     QCString trIncludingInheritedMembers()
-    { return ", uwzgldniajca wszystkie dziedziczone skadowe."; }
+    { return decode(", uwzgldniajca wszystkie dziedziczone skadowe."); }
     
     /*! this is put at the author sections at the bottom of man pages.
      *  parameter s is name of the project name.
      */
     QCString trGeneratedAutomatically(const char *s)
-    { QCString result="Wygenerowano automatycznie z kodu rdowego programem Doxygen";
+    { QCString result=decode("Wygenerowano automatycznie z kodu rdowego programem Doxygen");
       if (s) result+=(QCString)" dla "+s;
       result+="."; 
       return result;
@@ -129,7 +149,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     
     /*! put after an enum value in the list of all members */
     QCString trEnumValue()
-    { return "warto wyliczenia"; }
+    { return decode("warto wyliczenia"); }
     
     /*! put after an undocumented member in the list of all members */
     QCString trDefinedIn()
@@ -139,51 +159,51 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
      *  parameter f is the name of the include file.
      */
     QCString trVerbatimText(const char *f)
-    { return (QCString)"Tutaj znajduje si zawarto pliku nagwkowego "+f+"."; }
+    { return (QCString)decode("Tutaj znajduje si zawarto pliku nagwkowego ")+f+"."; }
     
     // quick reference sections
 
     /*! This is put above each page as a link to the list of all groups of 
-     *  compounds or files (see the \group command).
+     *  compounds or files (see the \\group command).
      */
     QCString trModules()
-    { return "Moduy"; }
+    { return decode("Moduy"); }
     
     /*! This is put above each page as a link to the class hierarchy */
     QCString trClassHierarchy()
-    { return "Hierarchia Klas"; }
+    { return "Hierarchia klas"; }
     
     /*! This is put above each page as a link to the list of annotated classes */
     QCString trCompoundList()
     { 
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        return "Struktury Danych";
+        return "Struktury danych";
       }
       else
       {
-        return "Lista Klas"; 
+        return "Lista klas"; 
       }
     }   
     
     /*! This is put above each page as a link to the list of documented files */
     QCString trFileList()
-    { return "Lista Plikw"; }
+    { return decode("Lista plikw"); }
 
     /*! This is put above each page as a link to the list of all verbatim headers */
     QCString trHeaderFiles()
-    { return "Pliki Nagwkowe"; }
+    { return decode("Pliki nagwkowe"); }
 
     /*! This is put above each page as a link to all members of compounds. */
     QCString trCompoundMembers()
     { 
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        return "Pola Danych"; 
+        return "Pola danych"; 
       }
       else
       {
-        return "Skadowe Klas"; 
+        return decode("Skadowe klas"); 
       }
     }
 
@@ -196,17 +216,17 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
       }
       else
       {
-        return "Skadowe Plikw"; 
+        return decode("Skadowe plikw"); 
       }
     }
 
     /*! This is put above each page as a link to all related pages. */
     QCString trRelatedPages()
-    { return "Dodatkowe Strony"; }
+    { return "Dodatkowe strony"; }
 
     /*! This is put above each page as a link to all examples. */
     QCString trExamples()
-    { return "Przykady"; }
+    { return decode("Przykady"); }
 
     /*! This is put above each page as a link to the search engine. */
     QCString trSearch()
@@ -214,8 +234,8 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
 
     /*! This is an introduction to the class hierarchy. */
     QCString trClassHierarchyDescription()
-    { return "Ta lista dziedziczenia posortowana jest z grubsza, "
-             "cho nie cakowicie, alfabetycznie:";
+    { return decode("Ta lista dziedziczenia posortowana jest z grubsza, "
+             "cho nie cakowicie, alfabetycznie:");
     }
 
     /*! This is an introduction to the list with all files. */
@@ -224,7 +244,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
       QCString result="Tutaj znajduje si lista wszystkich ";
       if (!extractAll) result+="udokumentowanych ";
       result+="plikw z ich krtkimi opisami:";
-      return result;
+      return decode(result);
     }
 
     /*! This is an introduction to the annotated compound list. */
@@ -233,12 +253,12 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
       
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        return "Tutaj znajduj si struktury danych wraz z ich krtkimi opisami:"; 
+        return decode("Tutaj znajduj si struktury danych wraz z ich krtkimi opisami:"); 
       }
       else
       {
-        return "Tutaj znajduj si klasy, struktury, "
-             "unie i interfejsy wraz z ich krtkimi opisami:"; 
+        return decode("Tutaj znajduj si klasy, struktury, "
+             "unie i interfejsy wraz z ich krtkimi opisami:"); 
       }
     }
 
@@ -259,7 +279,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
         result+="skadowych";
       }
       result+=" wraz z odnonikami do ";
-      if (extractAll) 
+      if (!extractAll) 
       {
         if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
         {
@@ -281,7 +301,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
           result+="klas, do ktrych dana skadowa naley:";
         }
       }
-      return result;
+      return decode(result);
     }
 
     /*! This is an introduction to the page with all file members. */
@@ -302,24 +322,24 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
         result+="plikw, do ktrych one nale:";
       else 
         result+="dokumentacji:";
-      return result;
+      return decode(result);
     }
 
     /*! This is an introduction to the page with the list of all header files. */
     QCString trHeaderFilesDescription()
-    { return "Tutaj znajduj si pliki nagwkowe tworzce API:"; }
+    { return decode("Tutaj znajduj si pliki nagwkowe tworzce API:"); }
 
     /*! This is an introduction to the page with the list of all examples */
     QCString trExamplesDescription()
-    { return "Tutaj znajduje si lista wszystkich przykadw:"; }
+    { return decode("Tutaj znajduje si lista wszystkich przykadw:"); }
 
     /*! This is an introduction to the page with the list of related pages */
     QCString trRelatedPagesDescription()
-    { return "Tutaj znajduje si lista wszystkich stron dokumentacji:"; }
+    { return decode("Tutaj znajduje si lista wszystkich stron dokumentacji:"); }
 
     /*! This is an introduction to the page with the list of class/file groups */
     QCString trModulesDescription()
-    { return "Tutaj znajduje si lista wszystkich grup:"; }
+    { return decode("Tutaj znajduje si lista wszystkich grup:"); }
 
     /*! This sentences is used in the annotated class/file lists if no brief
      * description is given. 
@@ -338,13 +358,13 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
      * index of all groups.
      */
     QCString trModuleIndex()
-    { return "Indeks Grup"; }
+    { return "Indeks grup"; }
 
     /*! This is used in LaTeX as the title of the chapter with the 
      * class hierarchy.
      */
     QCString trHierarchicalIndex()
-    { return "Indeks Hierarchiczny"; }
+    { return "Indeks hierarchiczny"; }
 
     /*! This is used in LaTeX as the title of the chapter with the 
      * annotated compound index.
@@ -353,11 +373,11 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     {
      if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       { 
-        return "Indeks Struktur Danych";
+        return "Indeks struktur danych";
       }
       else
       {
-        return "Indeks Klas"; 
+        return "Indeks klas"; 
       }
     }
 
@@ -365,13 +385,13 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
      * list of all files.
      */
     QCString trFileIndex() 
-    { return "Indeks Plikw"; }
+    { return decode("Indeks plikw"); }
 
     /*! This is used in LaTeX as the title of the chapter containing
      *  the documentation of all groups.
      */
     QCString trModuleDocumentation()
-    { return "Dokumentacja Grup"; }
+    { return "Dokumentacja grup"; }
 
     /*! This is used in LaTeX as the title of the chapter containing
      *  the documentation of all classes, structs and unions.
@@ -380,11 +400,11 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     { 
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        return "Dokumentacja Struktur Danych"; 
+        return "Dokumentacja struktur danych"; 
       }
       else
       {
-        return "Dokumentacja Klas"; 
+        return "Dokumentacja klas"; 
       }
     }
 
@@ -392,23 +412,23 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
      *  the documentation of all files.
      */
     QCString trFileDocumentation()
-    { return "Dokumentacja Plikw"; }
+    { return decode("Dokumentacja plikw"); }
 
     /*! This is used in LaTeX as the title of the chapter containing
      *  the documentation of all examples.
      */
     QCString trExampleDocumentation()
-    { return "Dokumentacja Przykadw"; }
+    { return decode("Dokumentacja przykadw"); }
 
     /*! This is used in LaTeX as the title of the chapter containing
      *  the documentation of all related pages.
      */
     QCString trPageDocumentation()
-    { return "Dokumentacja Stron"; }
+    { return "Dokumentacja stron"; }
 
     /*! This is used in LaTeX as the title of the document */
     QCString trReferenceManual()
-    { return "Podrcznik Referencji"; }
+    { return decode("Podrcznik"); }
     
     /*! This is used in the documentation of a file as a header before the 
      *  list of defines
@@ -420,13 +440,13 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
      *  list of function prototypes
      */
     QCString trFuncProtos()
-    { return "Prototypy Funkcji"; }
+    { return "Prototypy funkcji"; }
 
     /*! This is used in the documentation of a file as a header before the 
      *  list of typedefs
      */
     QCString trTypedefs()
-    { return "Definicje Typw"; }
+    { return decode("Definicje typw"); }
 
     /*! This is used in the documentation of a file as a header before the 
      *  list of enumerations
@@ -450,7 +470,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
      *  list of (global) variables
      */
     QCString trEnumerationValues()
-    { return "Wartoci Wylicze"; }
+    { return decode("Wartoci wylicze"); }
     
     /*! This is used in man pages as the author section. */
     QCString trAuthor()
@@ -460,43 +480,43 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
      *  documentation blocks for defines
      */
     QCString trDefineDocumentation()
-    { return "Dokumentacja Definicji"; }
+    { return "Dokumentacja definicji"; }
 
     /*! This is used in the documentation of a file/namespace before the list 
      *  of documentation blocks for function prototypes
      */
     QCString trFunctionPrototypeDocumentation()
-    { return "Dokumentacja Prototypw Funkcji"; }
+    { return decode("Dokumentacja prototypw funkcji"); }
 
     /*! This is used in the documentation of a file/namespace before the list 
      *  of documentation blocks for typedefs
      */
     QCString trTypedefDocumentation()
-    { return "Dokumentacja Definicji Typw"; }
+    { return decode("Dokumentacja definicji typw"); }
 
     /*! This is used in the documentation of a file/namespace before the list 
      *  of documentation blocks for enumeration types
      */
     QCString trEnumerationTypeDocumentation()
-    { return "Dokumentacja Typw Wyliczanych"; }
+    { return decode("Dokumentacja typw wyliczanych"); }
 
     /*! This is used in the documentation of a file/namespace before the list 
      *  of documentation blocks for enumeration values
      */
     QCString trEnumerationValueDocumentation()
-    { return "Dokumentacja Wartoci Wyliczanych"; }
+    { return decode("Dokumentacja wartoci wyliczanych"); }
 
     /*! This is used in the documentation of a file/namespace before the list 
      *  of documentation blocks for functions
      */
     QCString trFunctionDocumentation()
-    { return "Dokumentacja Funkcji"; }
+    { return "Dokumentacja funkcji"; }
 
     /*! This is used in the documentation of a file/namespace before the list 
      *  of documentation blocks for variables
      */
     QCString trVariableDocumentation()
-    { return "Dokumentacja Zmiennych"; }
+    { return "Dokumentacja zmiennych"; }
 
     /*! This is used in the documentation of a file/namespace/group before 
      *  the list of links to documented compounds
@@ -505,7 +525,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     { 
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        return "Struktury Danych"; 
+        return "Struktury danych"; 
       }
       else
       {
@@ -544,19 +564,19 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     
     /*! this text is generated when the \\internal command is used. */
     QCString trForInternalUseOnly()
-    { return "Tylko do uytku wewntrznego."; }
+    { return decode("Tylko do uytku wewntrznego."); }
 
     /*! this text is generated when the \\reimp command is used. */
     QCString trReimplementedForInternalReasons()
-    { return "Reimplementowana z wewntrzych przyczyn; nie dotyczy API."; }
+    { return decode("Reimplementowana z wewntrzych przyczyn; nie dotyczy API."); }
 
     /*! this text is generated when the \\warning command is used. */
     QCString trWarning()
-    { return "Ostrzeenie"; }
+    { return decode("Ostrzeenie"); }
 
     /*! this text is generated when the \\bug command is used. */
     QCString trBugsAndLimitations()
-    { return "Bdy i ograniczenia"; }
+    { return decode("Bdy i ograniczenia"); }
 
     /*! this text is generated when the \\version command is used. */
     QCString trVersion()
@@ -576,7 +596,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
 
     /*! this text is generated when the \\sa command is used. */
     QCString trSeeAlso()
-    { return "Zobacz rwnie"; }
+    { return decode("Zobacz rwnie"); }
 
     /*! this text is generated when the \\param command is used. */
     QCString trParameters()
@@ -584,7 +604,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
 
     /*! this text is generated when the \\exception command is used. */
     QCString trExceptions()
-    { return "Wyjtki"; }
+    { return decode("Wyjtki"); }
     
     /*! this text is used in the title page of a LaTeX document. */
     QCString trGeneratedBy()
@@ -594,7 +614,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     
     /*! used as the title of page containing all the index of all namespaces. */
     QCString trNamespaceList()
-    { return "Lista Przestrzeni Nazw"; }
+    { return "Lista przestrzeni nazw"; }
 
     /*! used as an introduction to the namespace list */
     QCString trNamespaceListDescription(bool extractAll)
@@ -602,7 +622,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
       QCString result="Tutaj znajduj si wszystkie ";
       if (!extractAll) result+="udokumentowane ";
       result+="przestrzenie nazw wraz z ich krtkimi opisami:";
-      return result;
+      return decode(result);
     }
 
     /*! used in the class documentation as a header before the list of all
@@ -619,7 +639,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
      * related classes 
      */
     QCString trRelatedFunctionDocumentation()
-    { return "Dokumentacja Przyjaci i Funkcji Zwizanych"; }
+    { return decode("Dokumentacja przyjaci i funkcji zwizanych"); }
     
 //////////////////////////////////////////////////////////////////////////
 // new since 0.49-990425
@@ -631,14 +651,16 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
                                     bool isTemplate)
     {
       QCString result="Dokumentacja";
-      if (isTemplate) result+=" Szablonu";
+      if (isTemplate) result+=" szablonu";
       switch(compType)
       {
-        case ClassDef::Class:      result+=" Klasy "; break;
-        case ClassDef::Struct:     result+=" Struktury "; break;
-        case ClassDef::Union:      result+=" Unii "; break;
-        case ClassDef::Interface:  result+=" Interfejsu "; break;
-        case ClassDef::Exception:  result+=" Wyjtku "; break;
+        case ClassDef::Class:      result+=" klasy "; break;
+        case ClassDef::Struct:     result+=" struktury "; break;
+        case ClassDef::Union:      result+=" unii "; break;
+        case ClassDef::Interface:  result+=" interfejsu "; break;
+        case ClassDef::Protocol:   result+=decode(" protocol "); break; // translate me!
+        case ClassDef::Category:   result+=decode(" category "); break; // translate me!
+        case ClassDef::Exception:  result+=decode(" wyjtku "); break;
       }
       result+=(QCString)clName;
       return result;
@@ -647,7 +669,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     /*! used as the title of the HTML page of a file */
     QCString trFileReference(const char *fileName)
     {
-      QCString result="Dokumentacja Pliku ";
+      QCString result="Dokumentacja pliku ";
       result+=fileName; 
       return result;
     }
@@ -655,32 +677,32 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     /*! used as the title of the HTML page of a namespace */
     QCString trNamespaceReference(const char *namespaceName)
     {
-      QCString result="Dokumentacja Przestrzeni Nazw ";
+      QCString result="Dokumentacja przestrzeni nazw ";
       result+=namespaceName;
       return result;
     }
     
     /* these are for the member sections of a class, struct or union */
     QCString trPublicMembers()
-    { return "Metody Publiczne"; }
+    { return "Metody publiczne"; }
     QCString trPublicSlots()
-    { return "Sloty Publiczne"; }
+    { return "Sloty publiczne"; }
     QCString trSignals()
-    { return "Sygnay"; }
+    { return decode("Sygnay"); }
     QCString trStaticPublicMembers()
-    { return "Statyczne Metody Publiczne"; }
+    { return "Statyczne metody publiczne"; }
     QCString trProtectedMembers()
-    { return "Metody Chronione"; }
+    { return "Metody chronione"; }
     QCString trProtectedSlots()
-    { return "Sloty Chronione"; }
+    { return "Sloty chronione"; }
     QCString trStaticProtectedMembers()
-    { return "Statyczne Metody Chronione"; }
+    { return "Statyczne metody chronione"; }
     QCString trPrivateMembers()
-    { return "Metody Prywatne"; }
+    { return "Metody prywatne"; }
     QCString trPrivateSlots()
-    { return "Sloty Prywatne"; }
+    { return "Sloty prywatne"; }
     QCString trStaticPrivateMembers()
-    { return "Statyczne Metody Prywatne"; }
+    { return "Statyczne metody prywatne"; }
     
     /*! this function is used to produce a comma-separated list of items.
      *  use generateMarker(i) to indicate where item i should be put.
@@ -741,7 +763,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
 
     /*! This is put above each page as a link to all members of namespaces. */
     QCString trNamespaceMembers()
-    { return "Skadowe Przestrzeni Nazw"; }
+    { return decode("Skadowe przestrzeni nazw"); }
 
     /*! This is an introduction to the page with all namespace members */
     QCString trNamespaceMemberDescription(bool extractAll)
@@ -753,19 +775,19 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
         result+="dokumentacji przestrzeni nazw dla kadej skadowej:";
       else 
         result+="przestrzeni nazw do ktrych skadowe te nale:";
-      return result;
+      return decode(result);
     }
     /*! This is used in LaTeX as the title of the chapter with the 
      *  index of all namespaces.
      */
     QCString trNamespaceIndex()
-    { return "Indeks Przestrzeni Nazw"; }
+    { return "Indeks przestrzeni nazw"; }
 
     /*! This is used in LaTeX as the title of the chapter containing
      *  the documentation of all namespaces.
      */
     QCString trNamespaceDocumentation()
-    { return "Dokumentacja Przestrzeni Nazw"; }
+    { return "Dokumentacja przestrzeni nazw"; }
 
 //////////////////////////////////////////////////////////////////////////
 // new since 0.49-990522
@@ -775,7 +797,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
      *  namespaces in a file.
      */
     QCString trNamespaces()
-    { return "Przestrzenie Nazw"; }
+    { return "Przestrzenie nazw"; }
 
 //////////////////////////////////////////////////////////////////////////
 // new since 0.49-990728
@@ -795,18 +817,20 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
         case ClassDef::Struct:     result+="j struktury"; break;
         case ClassDef::Union:      result+="j unii"; break;
         case ClassDef::Interface:  result+="go interfejsu"; break;
+        case ClassDef::Protocol:   result+=decode("protocol"); break; // translate me!
+        case ClassDef::Category:   result+=decode("category"); break; // translate me!
         case ClassDef::Exception:  result+="go wyjtku"; break;
       }
       result+=" zostaa wygenerowana z plik";
       if (single) result+="u:"; else result+="w:";
-      return result;
+      return decode(result);
     }
 
     /*! This is in the (quick) index as a link to the alphabetical compound
      * list.
      */
     QCString trAlphabeticalList()
-    { return "Lista Alfabetyczna"; }
+    { return "Lista alfabetyczna"; }
 
 //////////////////////////////////////////////////////////////////////////
 // new since 0.49-990901
@@ -814,12 +838,12 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
 
     /*! This is used as the heading text for the retval command. */
     QCString trReturnValues()
-    { return "Zwracane wartoci"; }
+    { return decode("Zwracane wartoci"); }
 
     /*! This is in the (quick) index as a link to the main page (index.html)
      */
     QCString trMainPage()
-    { return "Strona Gwna"; }
+    { return decode("Strona gwna"); }
 
     /*! This is used in references to page that are put in the LaTeX 
      *  documentation. It should be an abbreviation of the word page.
@@ -833,7 +857,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
 
     QCString trSources()
     {
-      return "rda";
+      return decode("rda");
     }
     QCString trDefinedAtLineInSourceFile()
     {
@@ -860,68 +884,68 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     /*! this text is put before a collaboration diagram */
     QCString trCollaborationDiagram(const char *clName)
     {
-      return (QCString)"Diagram wsppracy dla "+clName+":";
+      return (QCString)decode("Diagram wsppracy dla ")+clName+":";
     }
     /*! this text is put before an include dependency graph */
     QCString trInclDepGraph(const char *fName)
     {
-      return (QCString)"Wykres zalenoci zaczania dla "+fName+":";
+      return (QCString)decode("Wykres zalenoci zaczania dla ")+fName+":";
     }
     /*! header that is put before the list of constructor/destructors. */
     QCString trConstructorDocumentation()
     {
-      return "Dokumentacja Konstruktora i Destruktora"; 
+      return "Dokumentacja konstruktora i destruktora"; 
     }
     /*! Used in the file documentation to point to the corresponding sources. */
     QCString trGotoSourceCode()
     {
-      return "Id do kodu rdowego tego pliku.";
+      return decode("Id do kodu rdowego tego pliku.");
     }
     /*! Used in the file sources to point to the corresponding documentation. */
     QCString trGotoDocumentation()
     {
-      return "Id do dokumentacji tego pliku.";
+      return decode("Id do dokumentacji tego pliku.");
     }
     /*! Text for the \\pre command */
     QCString trPrecondition()
     {
-      return "Warunek wstpny";
+      return decode("Warunek wstpny");
     }
     /*! Text for the \\post command */
     QCString trPostcondition()
     {
-      return "Warunek kocowy";
+      return decode("Warunek kocowy");
     }
     /*! Text for the \\invariant command */
     QCString trInvariant()
     {
-      return "Inwariant";
+      return "Niezmiennik";
     }
     /*! Text shown before a multi-line variable/enum initialization */
     QCString trInitialValue()
     {
-      return "Inicjalizator:";
+      return decode("Warto pocztkowa:");
     }
     /*! Text used the source code in the file index */
     QCString trCode()
     {
-      return "kod rdowy";
+      return decode("kod rdowy");
     }
     QCString trGraphicalHierarchy()
     {
-      return "Graficzna Hierarchia Klas";
+      return "Graficzna hierarchia klas";
     }
     QCString trGotoGraphicalHierarchy()
     {
-      return "Id do graficznej hierarchi klas";
+      return decode("Id do graficznej hierarchi klas");
     }
     QCString trGotoTextualHierarchy()
     {
-      return "Id do tekstowej hierarchi klas";
+      return decode("Id do tekstowej hierarchi klas");
     }
     QCString trPageIndex()
     {
-      return "Indeks Stron";
+      return "Indeks stron";
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -934,46 +958,46 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     }
     QCString trPublicTypes()
     {
-      return "Typy Publiczne";
+      return "Typy publiczne";
     }
     QCString trPublicAttribs()
     {
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        return "Pola Danych";
+        return "Pola danych";
       }
       else
       {
-        return "Atrybuty Publiczne";
+        return "Atrybuty publiczne";
       }
     }
     QCString trStaticPublicAttribs()
     {
-      return "Statyczne Atrybuty Publiczne";
+      return "Statyczne atrybuty publiczne";
     }
     QCString trProtectedTypes()
     {
-      return "Typy Chronione";
+      return "Typy chronione";
     }
     QCString trProtectedAttribs()
     {
-      return "Atrybuty Chronione";
+      return "Atrybuty chronione";
     }
     QCString trStaticProtectedAttribs()
     {
-      return "Statyczne Atrybuty Chronione";
+      return "Statyczne atrybuty chronione";
     }
     QCString trPrivateTypes()
     {
-      return "Typy Prywatne";
+      return "Typy prywatne";
     }
     QCString trPrivateAttribs()
     {
-      return "Atrybuty Prywatne";
+      return "Atrybuty prywatne";
     }
     QCString trStaticPrivateAttribs()
     {
-      return "Statyczne Atrybuty Prywatne";
+      return "Statyczne atrybuty prywatne";
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -983,12 +1007,12 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     /*! Used as a marker that is put before a todo item */
     QCString trTodo()
     {
-      return "Do Zrobienia";
+      return "Do zrobienia";
     }
     /*! Used as the header of the todo list */
     QCString trTodoList()
     {
-      return "Lista Do Zrobienia";
+      return "Lista rzeczy do zrobienia";
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -997,11 +1021,11 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
 
     QCString trReferencedBy()
     {
-      return "Odwoania w";
+      return decode("Odwoania w");
     }
     QCString trRemarks()
     {
-      return "Spostrzeenia";
+      return decode("Spostrzeenia");
     }
     QCString trAttention()
     {
@@ -1009,8 +1033,8 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     }
     QCString trInclByDepGraph()
     {
-      return "Ten wykres pokazuje, ktre pliki bezporednio lub "
-             "porednio zaczaj ten plik:";
+      return decode("Ten wykres pokazuje, ktre pliki bezporednio lub "
+             "porednio zaczaj ten plik:");
     }
     QCString trSince()
     {
@@ -1024,12 +1048,12 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     /*! title of the graph legend page */
     QCString trLegendTitle()
     {
-      return "Legenda Wykresu";
+      return "Legenda wykresu";
     }
     /*! page explaining how the dot graph's should be interpreted */
     QCString trLegendDocs()
     {
-      return 
+      return decode(
         "Ta strona wyjania jak interpretowa wykresy, ktre s wygenerowane "
         "przez doxygen.<p>\n"
         "Rozwa nastpujcy przykad:\n"
@@ -1081,7 +1105,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
         "<li>Fioletowa przerywana strzaka jest uywana jeli klasa jest zawarta "
         "lub uyta przez inn klas. Strzaka jest podpisana zmienn(ymi) "
         "przez ktre wskazywana klasa lub struktura jest dostpna. \n"
-        "</ul>\n";
+        "</ul>\n");
     }
     /*! text for the link to the legend page */
     QCString trLegend()
@@ -1101,7 +1125,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     /*! Used as the header of the test list */
     QCString trTestList()
     {
-      return "Lista Testu";
+      return "Lista testu";
     }
 //////////////////////////////////////////////////////////////////////////
 // new since 1.2.1
@@ -1120,12 +1144,12 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     /*! Used as a section header for IDL properties */
     virtual QCString trProperties()
     {
-      return "Waciwoci";
+      return decode("Waciwoci");
     }
     /*! Used as a section header for IDL property documentation */
     virtual QCString trPropertyDocumentation()
     {
-      return "Dokumentacja Waciwoci";
+      return decode("Dokumentacja waciwoci");
     }
 //////////////////////////////////////////////////////////////////////////
 // new since 1.2.4
@@ -1156,27 +1180,27 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     /*! Title of the package index page */
     virtual QCString trPackageList()
     {
-      return "Lista Pakietw";
+      return decode("Lista Pakietw");
     }
     /*! The description of the package index page */
     virtual QCString trPackageListDescription()
     {
-      return "Oto lista pakietw wraz z krtkim opisem (o ile jest dostpny):";
+      return decode("Oto lista pakietw wraz z krtkim opisem (o ile jest dostpny):");
     }
     /*! The link name in the Quick links header for each page */
     virtual QCString trPackages()
     {
-      return "Pakiety";
+      return decode("Pakiety");
     }
     /*! Used as a chapter title for Latex & RTF output */
     virtual QCString trPackageDocumentation()
     {
-      return "Dokumentacja Pakietu";
+      return decode("Dokumentacja Pakietu");
     }
     /*! Text shown before a multi-line define */
     virtual QCString trDefineValue()
     {
-      return "Warto:";
+      return decode("Warto:");
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1186,12 +1210,12 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     /*! Used as a marker that is put before a \\bug item */
     virtual QCString trBug()
     {
-      return "Bd";
+      return decode("Bd");
     }
     /*! Used as the header of the bug list */
     virtual QCString trBugList()
     {
-      return "Lista bdw";
+      return decode("Lista bdw");
     }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1213,7 +1237,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     /*! Used as header RTF general index */
     virtual QCString trRTFGeneralIndex()
     {
-      return "Indeks";
+      return decode("Indeks");
     }
                                                                           
     /*! This is used for translation of the word that will possibly
@@ -1247,7 +1271,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
       QCString result((first_capital ? "Przestrze" : "przestrze"));
       result+=(singular ? "" : "nie");
       result+=" nazw";
-      return result; 
+      return decode(result); 
     }
 
     /*! This is used for translation of the word that will possibly
@@ -1267,7 +1291,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
      */
     virtual QCString trPage(bool first_capital, bool singular)
     { 
-      QCString result((first_capital ? "Ston" : "stron"));
+      QCString result((first_capital ? "Stron" : "stron"));
       result+=(singular ? "a" : "y");
       return result; 
     }
@@ -1280,7 +1304,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
     { 
       QCString result((first_capital ? "Skadow" : "skadow"));
       result+=(singular ? "a" : "e");
-      return result; 
+      return decode(result); 
     }
    
     /*! This is used for translation of the word that will possibly
@@ -1326,7 +1350,7 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
      */
     virtual QCString trReferences()
     {
-      return "Odwouje si do";
+      return decode("Odwouje si do");
     }
 
     
@@ -1354,9 +1378,39 @@ class TranslatorPolish : public TranslatorAdapter_1_2_17
      */
     virtual QCString trRTFTableOfContents()
     {
-      return "Spis Treci";
+      return decode("Spis treci");
     }
 
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.17
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as the header of the list of item that have been 
+     *  flagged deprecated 
+     */
+    virtual QCString trDeprecatedList()
+    {
+      return decode("Lista elementw do wycofania");
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.18
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a header for declaration section of the events found in 
+     * a C# program
+     */
+    virtual QCString trEvents()
+    {
+      return "Zdarzenia";
+    }
+    /*! Header used for the documentation section of a class' events. */
+    virtual QCString trEventDocumentation()
+    {
+      return decode("Dokumentacja zdarze");
+    }
+
+
 };
 
 #endif
