@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2000 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,9 +18,7 @@
  *
  * Nickolay Semyonov
  * Andrey V. Stolyarov released Feb 14, 2001
- * Alexandr V. Chelpanov <cav@cryptopro.ru> released Apr 21, 2001
- *
- *          email (. )
+ * Alexandr V. Chelpanov <cav@cryptopro.ru> released Sep 16, 2003
  *
  *     list  documentation .
  *
@@ -58,11 +56,14 @@ class TranslatorRussian : public Translator
      */
     inline QCString decode(const QCString & sInput)
     { 
-#ifdef _WIN32
-      return Koi8RToWindows1251(sInput);
-#else
-      return sInput;
-#endif
+      if (Config_getBool("USE_WINDOWS_ENCODING"))
+      {
+        return Koi8RToWindows1251(sInput);
+      }
+      else
+      {
+        return sInput;
+      }
     }
 
   public:
@@ -72,19 +73,29 @@ class TranslatorRussian : public Translator
 
     /* Used to get the command(s) for the language support. */
     virtual QCString latexLanguageSupportCommand()
-#ifdef _WIN32
-    { return "\\usepackage[cp1251]{inputenc}\n\\usepackage[russian]{babel}\n"; }
-#else
-    { return "\\usepackage[koi8-r]{inputenc}\n\\usepackage[russian]{babel}\n"; }
-#endif
+    {
+      if (Config_getBool("USE_WINDOWS_ENCODING"))
+      { 
+        return "\\usepackage[cp1251]{inputenc}\n\\usepackage[russian]{babel}\n"; 
+      }
+      else
+      { 
+        return "\\usepackage[koi8-r]{inputenc}\n\\usepackage[russian]{babel}\n"; 
+      }
+    }
 
     /*! return the language charset. This will be used for the HTML output */
     virtual QCString idLanguageCharset()
-#ifdef _WIN32
-    { return "Windows-1251"; }
-#else
-    { return "koi8-r"; }
-#endif
+    {
+      if (Config_getBool("USE_WINDOWS_ENCODING"))
+      { 
+        return "Windows-1251"; 
+      }
+      else
+      { 
+        return "koi8-r"; 
+      }
+    }
 
     // --- Language translation methods -------------------
 
@@ -277,7 +288,7 @@ class TranslatorRussian : public Translator
           result+="      ";
         else
           result+="     ";
-        if(extractAll)
+        if(!extractAll)
         {
           if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
             result+="     .";
@@ -649,6 +660,8 @@ class TranslatorRussian : public Translator
           case ClassDef::Struct: result+=""; break;
           case ClassDef::Union:  result+=""; break;
           case ClassDef::Interface:  result+=""; break;
+          case ClassDef::Protocol:   result+="protocol"; break; // translate me!
+          case ClassDef::Category:   result+="category"; break; // translate me!
           case ClassDef::Exception:  result+=""; break;
         }
       }
@@ -660,6 +673,8 @@ class TranslatorRussian : public Translator
           case ClassDef::Struct: result+=""; break;
           case ClassDef::Union:  result+=""; break;
           case ClassDef::Interface:  result+=""; break;
+          case ClassDef::Protocol:   result+="protocol"; break; // translate me!
+          case ClassDef::Category:   result+="category"; break; // translate me!
           case ClassDef::Exception:  result+=""; break;
         }
       }
@@ -823,6 +838,10 @@ class TranslatorRussian : public Translator
         case ClassDef::Interface:  result+="";
           if (single) result+=''; else result+="";
           break;
+        case ClassDef::Protocol:  result+="Protocol"; // translate me!
+          break;
+        case ClassDef::Category:  result+="Category"; // translate me!
+          break;
         case ClassDef::Exception:  result+="";
           if (single) result+=''; else result+='';
           break;
@@ -1417,6 +1436,139 @@ class TranslatorRussian : public Translator
     {
       return decode( "    " );
     }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.18
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a header for declaration section of the events found in 
+     * a C# program
+     */
+    virtual QCString trEvents()
+    {
+      return decode( "" );
+    }
+    /*! Header used for the documentation section of a class' events. */
+    virtual QCString trEventDocumentation()
+    {
+      return decode( "C" );
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a heading for a list of Java class types with package scope.
+     */
+    virtual QCString trPackageTypes()
+    { 
+      return decode( "    " );
+    }
+    /*! Used as a heading for a list of Java class functions with package 
+     * scope. 
+     */
+    virtual QCString trPackageMembers()
+    { 
+      return decode( "    " );
+    }
+    /*! Used as a heading for a list of static Java class functions with 
+     *  package scope.
+     */
+    virtual QCString trStaticPackageMembers()
+    { 
+      return decode( "     " );
+    }
+    /*! Used as a heading for a list of Java class variables with package 
+     * scope.
+     */
+    virtual QCString trPackageAttribs()
+    { 
+      return decode( "    " );
+    }
+    /*! Used as a heading for a list of static Java class variables with 
+     * package scope.
+     */
+    virtual QCString trStaticPackageAttribs()
+    { 
+      return decode( "     " );
+    }
+    
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3.1
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used in the quick index of a class/file/namespace member list page 
+     *  to link to the unfiltered list of all members.
+     */
+    virtual QCString trAll()
+    {
+      return decode( "" );
+    }
+    /*! Put in front of the call graph for a function. */
+    virtual QCString trCallGraph()
+    {
+      return decode( " :" );
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3.3
+//////////////////////////////////////////////////////////////////////////
+
+    /*! When the search engine is enabled this text is put in the header 
+     *  of each page before the field where one can enter the text to search 
+     *  for. 
+     */
+    virtual QCString trSearchForIndex()
+    {
+      return decode( "" );
+    }
+    /*! This string is used as the title for the page listing the search
+     *  results.
+     */
+    virtual QCString trSearchResultsTitle()
+    {
+      return decode( " " );
+    }
+    /*! This string is put just before listing the search results. The
+     *  text can be different depending on the number of documents found.
+     *  Inside the text you can put the special marker $num to insert
+     *  the number representing the actual number of search results.
+     *  The @a numDocuments parameter can be either 0, 1 or 2, where the 
+     *  value 2 represents 2 or more matches. HTML markup is allowed inside
+     *  the returned string.
+     */
+    virtual QCString trSearchResults(int numDocuments)
+    {
+      if (numDocuments==0)
+      {
+        return decode( " ,      ." );
+      }
+      else if( numDocuments == 1 )
+      {
+        return decode( " <b>$num</b> " ) + as_documents( numDocuments )
+          + ".";
+      }
+      else 
+      {
+        return decode( " <b>$num</b> " ) + as_documents( numDocuments )
+          + decode( ".    ." );
+      }
+    }
+    /*! This string is put before the list of matched words, for each search 
+     *  result. What follows is the list of words that matched the query.
+     */
+    virtual QCString trSearchMatches()
+    {
+      return decode( ":" );
+    }
+
+private:
+    QCString as_documents( int num )
+    {
+      if( num % 10 == 1 ) return "";
+      if( (num % 10) >= 2 && (num % 10) <= 4 ) return decode( "" );
+      return decode( "" );
+    }
 };
 
 #endif
