@@ -2,7 +2,7 @@
  *
  * $Id: util.h,v 1.49 2001/03/19 19:27:42 root Exp $
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,11 +18,15 @@
 #ifndef UTIL_H
 #define UTIL_H
 
+/*! \file util.h 
+ *  \brief A bunch of utility functions.
+ */
+
 #include "qtbc.h"
 #include <qlist.h>
 #include <qtextstream.h>
 #include <ctype.h>
-
+#include "sortdict.h"
 
 class ClassDef;
 class FileDef;
@@ -37,21 +41,23 @@ class ExampleSDict;
 class ClassSDict;
 class BaseClassList;
 class GroupDef;
-class NamespaceList;
+class NamespaceSDict;
 class ClassList;
 class MemberGroupSDict;
-class Definition;
 struct TagInfo;
 class MemberNameInfoSDict;
 struct ListItemInfo;
-class PageInfo;
+class PageDef;
+struct SectionInfo;
+class QDir;
+class Definition;
 
 //--------------------------------------------------------------------
 
 class TextGeneratorIntf
 {
   public:
-    virtual void writeString(const char *) const = 0;
+    virtual void writeString(const char *,bool) const = 0;
     virtual void writeBreak() const = 0;
     virtual void writeLink(const char *extRef,const char *file,
                       const char *anchor,const char *text
@@ -62,7 +68,7 @@ class TextGeneratorOLImpl : public TextGeneratorIntf
 {
   public:
     TextGeneratorOLImpl(OutputDocInterface &od);
-    void writeString(const char *s) const;
+    void writeString(const char *s,bool keepSpaces) const;
     void writeBreak() const;
     void writeLink(const char *extRef,const char *file,
                    const char *anchor,const char *text
@@ -73,13 +79,19 @@ class TextGeneratorOLImpl : public TextGeneratorIntf
 
 //--------------------------------------------------------------------
 
-extern void linkifyText(const TextGeneratorIntf &ol,const char *clName,const char *name,
-                        const char *text,bool autoBreak=FALSE,bool external=TRUE);
-
-extern void setAnchors(ClassDef *cd,char id,MemberList *ml,int groupId=-1);
-extern QCString fileToString(const char *name,bool filter=FALSE);
-extern QCString dateToString(bool);
-extern bool getDefs(const QCString &scopeName,
+void linkifyText(const TextGeneratorIntf &ol,
+                 Definition *scope,
+                 FileDef *fileScope,
+                 const char *name,
+                 const char *text,
+                 bool autoBreak=FALSE,
+                 bool external=TRUE,
+                 bool keepSpaces=FALSE
+                );
+void setAnchors(ClassDef *cd,char id,MemberList *ml,int groupId=-1);
+QCString fileToString(const char *name,bool filter=FALSE);
+QCString dateToString(bool);
+bool getDefs(const QCString &scopeName,
                     const QCString &memberName, 
                     const char *, 
                     MemberDef *&md, 
@@ -93,67 +105,61 @@ extern bool getDefs(const QCString &scopeName,
                    );
 
 
-extern bool resolveRef(/* in */  const char *scName,
-                       /* in */  const char *name,
-                       /* in */  bool inSeeBlock,
-                       /* out */ Definition **resContext,
-                       /* out */ MemberDef  **resMember
-                      );
+bool resolveRef(/* in */  const char *scName,
+                /* in */  const char *name,
+                /* in */  bool inSeeBlock,
+                /* out */ Definition **resContext,
+                /* out */ MemberDef  **resMember
+               );
 
-extern bool resolveLink(/* in */ const char *scName,
-                        /* in */ const char *lr,
-                        /* in */ bool inSeeBlock,
-                        /* out */ Definition **resContext,
-                        /* out */ PageInfo **resPageInfo,
-                        /* out */ QCString &resAnchor
-                       );
+bool resolveLink(/* in */  const char *scName,
+                 /* in */  const char *lr,
+                 /* in */  bool inSeeBlock,
+                 /* out */ Definition **resContext,
+                 /* out */ QCString &resAnchor
+                );
 
-extern bool generateRef(OutputDocInterface &od,const char *,
+bool generateRef(OutputDocInterface &od,const char *,
                         const char *,bool inSeeBlock,const char * =0);
-extern bool generateLink(OutputDocInterface &od,const char *,
+bool generateLink(OutputDocInterface &od,const char *,
                          const char *,bool inSeeBlock,const char *);
 
-extern void generateFileRef(OutputDocInterface &od,const char *,
+void generateFileRef(OutputDocInterface &od,const char *,
                              const char *linkTxt=0);
 void writePageRef(OutputDocInterface &od,const char *cn,const char *mn);
 
-extern bool matchArguments(ArgumentList *,ArgumentList *,
+bool matchArguments(ArgumentList *,ArgumentList *,
                            const char *cl=0,const char *ns=0,bool checkCV=TRUE,
-                           NamespaceList *usingNamespaces=0,
-                           ClassList *usingClasses=0);
-extern void mergeArguments(ArgumentList *,ArgumentList *);
-extern QCString substituteClassNames(const QCString &s);
-extern QCString substitute(const char *s,const char *src,const char *dst);
-extern QCString resolveDefines(const char *n);
-extern ClassDef *getClass(const char *key);
-extern ClassDef *getResolvedClass(Definition *scope,
-                                  const char *key,
-                                  bool *pIsTypeDef=0,
-                                  QCString *pTemplSpec=0);
-extern NamespaceDef *getResolvedNamespace(const char *key);
-extern FileDef *findFileDef(const FileNameDict *fnDict,const char *n,
+                           NamespaceSDict *usingNamespaces=0,
+                           SDict<Definition> *usingClasses=0);
+void mergeArguments(ArgumentList *,ArgumentList *,bool forceNameOverwrite=FALSE);
+QCString substituteClassNames(const QCString &s);
+QCString substitute(const char *s,const char *src,const char *dst);
+QCString resolveDefines(const char *n);
+ClassDef *getClass(const char *key);
+ClassDef *getResolvedClass(Definition *scope,
+                           FileDef *fileScope,
+                           const char *key,
+                           MemberDef **pTypeDef=0,
+                           QCString *pTemplSpec=0,
+                           bool mayBeUnlinkable=FALSE);
+NamespaceDef *getResolvedNamespace(const char *key);
+FileDef *findFileDef(const FileNameDict *fnDict,const char *n,
                 bool &ambig);
-extern QCString showFileDefMatches(const FileNameDict *fnDict,const char *n);
-extern int guessSection(const char *name);
-extern bool isId(char c);
-extern QCString removeRedundantWhiteSpace(const QCString &s);
-//extern void startTitle(OutputList &ol,const char *fileName);
-//extern void endTitle(OutputList &ol,const char *fileName,const char *name);
-//void startFile(OutputList &ol,const char *name,
-//               const char *title,bool external=FALSE);
-//void endFile(OutputList &ol,bool external=FALSE);
-//void writeQuickLinks(OutputList &ol,bool compact,bool external=FALSE);
+QCString showFileDefMatches(const FileNameDict *fnDict,const char *n);
+int guessSection(const char *name);
+bool isId(char c);
+QCString removeRedundantWhiteSpace(const QCString &s);
 QCString argListToString(ArgumentList *al);
 QCString tempArgListToString(ArgumentList *al);
 QCString generateMarker(int id);
 void writeExample(OutputList &ol,ExampleSDict *el);
-//void setFileNameForSections(QList<QCString> *anchorList,const char *fileName,
-//                            PageInfo *pi=0);
 QCString stripAnonymousNamespaceScope(const QCString &s);
 QCString stripFromPath(const QCString &path);
+QCString stripFromIncludePath(const QCString &path);
 bool rightScopeMatch(const QCString &scope, const QCString &name);
 bool leftScopeMatch(const QCString &scope, const QCString &name);
-QCString substituteKeywords(const QCString &s,const char *title);
+QCString substituteKeywords(const QCString &s,const char *title,const QCString &relPath="");
 int getPrefixIndex(const QCString &name);
 QCString removeAnonymousScopes(const QCString &s);
 QCString replaceAnonymousScopes(const QCString &s);
@@ -162,7 +168,8 @@ bool hasVisibleRoot(BaseClassList *bcl);
 int minClassDistance(ClassDef *cd,ClassDef *bcd,int level=0);
 QCString convertNameToFile(const char *name,bool allowDots=FALSE);
 void extractNamespaceName(const QCString &scopeName,
-                          QCString &className,QCString &namespaceName);
+                          QCString &className,QCString &namespaceName,
+                          bool allowEmptyClass=FALSE);
 QCString insertTemplateSpecifierInScope(const QCString &scope,const QCString &templ);
 QCString stripScope(const char *name);
 int iSystem(const char *command,const char *args,bool isBatchFile=FALSE);
@@ -171,7 +178,7 @@ QCString convertToXML(const char *s);
 const char * getOverloadDocs();
 void addMembersToMemberGroup(MemberList *ml,
                              MemberGroupSDict *memberGroupSDict,
-                             Definition *context=0);
+                             Definition *context);
 bool extractClassNameFromType(const QCString &type,int &pos,
                               QCString &name,QCString &templSpec);
 QCString substituteTemplateArgumentsInString(
@@ -190,8 +197,8 @@ int getScopeFragment(const QCString &s,int p,int *l);
 int filterCRLF(char *buf,int len);
 void addRefItem(const QList<ListItemInfo> *sli,const char *prefix,
                         const char *name,const char *title,const char *args=0);
-PageInfo *addRelatedPage(const char *name,const QCString &ptitle,
-                           const QCString &doc,QList<QCString> *anchors,
+PageDef *addRelatedPage(const char *name,const QCString &ptitle,
+                           const QCString &doc,QList<SectionInfo> *anchors,
                            const char *fileName,int startLine,
                            const QList<ListItemInfo> *sli,
                            GroupDef *gd=0,
@@ -200,8 +207,21 @@ PageInfo *addRelatedPage(const char *name,const QCString &ptitle,
 QCString escapeCharsInString(const char *name,bool allowDots);
 void addGroupListToTitle(OutputList &ol,Definition *d);
 void filterLatexString(QTextStream &t,const char *str,
-                       bool insideTabbing=FALSE,bool insidePre=FALSE);
+                       bool insideTabbing=FALSE,bool insidePre=FALSE,
+                       bool insideItem=FALSE);
 QCString rtfFormatBmkStr(const char *name);
+QCString linkToText(const char *link,bool isFileName);
+QCString stripExtension(const char *fName);
+void replaceNamespaceAliases(QCString &scope,int i);
+int isAccessibleFrom(Definition *scope,FileDef *fileScope,Definition *item);
+int isAccessibleFromWithExpScope(Definition *scope,FileDef *fileScope,Definition *item,
+                     const QCString &explicitScopePart);
+int computeQualifiedIndex(const QString &name);
+void addDirPrefix(QCString &fileName);
+QCString relativePathToRoot(const char *name);
+#define REL_PATH_TO_ROOT "../../"
+void createSubDirs(QDir &d);
+QCString stripPath(const char *s);
 
 #endif
 
