@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -25,9 +25,11 @@
 #include "code.h"
 #include "dot.h"
 #include "message.h"
+#include "util.h"
+#include <qfileinfo.h> 
 
 XmlDocVisitor::XmlDocVisitor(QTextStream &t,BaseCodeDocInterface &ci) 
-  : m_t(t), m_ci(ci), m_insidePre(FALSE), m_hide(FALSE) 
+  : DocVisitor(DocVisitor_XML), m_t(t), m_ci(ci), m_insidePre(FALSE), m_hide(FALSE) 
 {
 }
 
@@ -76,6 +78,8 @@ void XmlDocVisitor::visit(DocSymbol *s)
     case DocSymbol::Hash:    m_t << "#"; break;
     case DocSymbol::Percent: m_t << "%"; break;
     case DocSymbol::Copy:    m_t << "<copy/>"; break;
+    case DocSymbol::Tm:      m_t << "<trademark/>"; break;
+    case DocSymbol::Reg:     m_t << "<registered/>"; break;
     case DocSymbol::Apos:    m_t << "'"; break;
     case DocSymbol::Quot:    m_t << "\""; break;
     case DocSymbol::Uml:     m_t << "<umlaut char=\"" << s->letter() << "\"/>"; break;
@@ -86,6 +90,7 @@ void XmlDocVisitor::visit(DocSymbol *s)
     case DocSymbol::Szlig:   m_t << "<szlig/>"; break;
     case DocSymbol::Cedil:   m_t << "<cedil char=\"" << s->letter() << "\"/>"; break;
     case DocSymbol::Ring:    m_t << "<ring char=\"" << s->letter() << "\"/>"; break;
+    case DocSymbol::Slash:   m_t << "<slash char=\"" << s->letter() << "\"/>"; break;
     case DocSymbol::Nbsp:    m_t << "<nonbreakablespace/>"; break;
     default:
                              err("Error: unknown symbol found\n");
@@ -95,7 +100,9 @@ void XmlDocVisitor::visit(DocSymbol *s)
 void XmlDocVisitor::visit(DocURL *u)
 {
   if (m_hide) return;
-  m_t << "<ulink url=\"" << u->url() << "\">";
+  m_t << "<ulink url=\""; 
+  if (u->isEmail()) m_t << "mailto:";
+  m_t << u->url() << "\">";
   filter(u->url());
   m_t << "</ulink>";
 }
@@ -138,6 +145,20 @@ void XmlDocVisitor::visit(DocStyleChange *s)
     case DocStyleChange::Small:
       if (s->enable()) m_t << "<small>";  else m_t << "</small> ";
       break;
+    case DocStyleChange::Preformatted:
+      if (s->enable()) 
+      {
+        m_t << "<preformatted>";  
+        m_insidePre=TRUE;
+      }
+      else 
+      {
+        m_t << "</preformatted> ";
+        m_insidePre=FALSE;
+      }
+      break;
+    case DocStyleChange::Div:  /* HTML only */ break;
+    case DocStyleChange::Span: /* HTML only */ break;
   }
 }
 
@@ -148,24 +169,37 @@ void XmlDocVisitor::visit(DocVerbatim *s)
   {
     case DocVerbatim::Code: // fall though
       m_t << "<programlisting>"; 
-      parseCode(m_ci,s->context(),s->text(),FALSE,0);
+      parseCode(m_ci,s->context(),s->text().latin1(),s->isExample(),s->exampleFile());
       m_t << "</programlisting>"; 
       break;
     case DocVerbatim::Verbatim: 
-      m_t << "<preformatted>";
+      m_t << "<verbatim>";
       filter(s->text());
-      m_t << "</preformatted>"; 
+      m_t << "</verbatim>"; 
       break;
     case DocVerbatim::HtmlOnly: 
       m_t << "<htmlonly>";
       filter(s->text());
       m_t << "</htmlonly>";
       break;
+    case DocVerbatim::ManOnly: 
+      m_t << "<manonly>";
+      filter(s->text());
+      m_t << "</manonly>";
+      break;
     case DocVerbatim::LatexOnly: 
       m_t << "<latexonly>";
       filter(s->text());
       m_t << "</latexonly>";
       break;
+    case DocVerbatim::XmlOnly: 
+      m_t << s->text();
+      break;
+    case DocVerbatim::Dot: 
+      m_t << "<dot>";
+      filter(s->text());
+      m_t << "</dot>";
+      break;
   }
 }
 
@@ -180,9 +214,18 @@ void XmlDocVisitor::visit(DocInclude *inc)
   if (m_hide) return;
   switch(inc->type())
   {
+    case DocInclude::IncWithLines:
+      { 
+         m_t << "<programlisting>";
+         QFileInfo cfi( inc->file() );
+         FileDef fd( cfi.dirPath(), cfi.fileName() );
+         parseCode(m_ci,inc->context(),inc->text().latin1(),inc->isExample(),inc->exampleFile(), &fd);
+         m_t << "</programlisting>"; 
+      }
+      break;    
     case DocInclude::Include: 
       m_t << "<programlisting>";
-      parseCode(m_ci,inc->context(),inc->text(),FALSE,0);
+      parseCode(m_ci,inc->context(),inc->text().latin1(),inc->isExample(),inc->exampleFile());
       m_t << "</programlisting>"; 
       break;
     case DocInclude::DontInclude: 
@@ -193,9 +236,9 @@ void XmlDocVisitor::visit(DocInclude *inc)
       m_t << "</htmlonly>";
       break;
     case DocInclude::VerbInclude: 
-      m_t << "<preformatted>";
+      m_t << "<verbatim>";
       filter(inc->text());
-      m_t << "</preformatted>"; 
+      m_t << "</verbatim>"; 
       break;
   }
 }
@@ -206,21 +249,28 @@ void XmlDocVisitor::visit(DocIncOperator *op)
   //    op->type(),op->isFirst(),op->isLast(),op->text().data());
   if (op->isFirst()) 
   {
-    m_t << "<programlisting>";
+    if (!m_hide)
+    {
+      m_t << "<programlisting>";
+    }
+    pushEnabled();
     m_hide = TRUE;
   }
   if (op->type()!=DocIncOperator::Skip) 
   {
-    parseCode(m_ci,op->context(),op->text(),FALSE,0);
+    popEnabled();
+    if (!m_hide) parseCode(m_ci,op->context(),op->text().latin1(),op->isExample(),op->exampleFile());
+    pushEnabled();
+    m_hide=TRUE;
   }
   if (op->isLast())  
   {
-    m_hide = FALSE;
-    m_t << "</programlisting>"; 
+    popEnabled();
+    if (!m_hide) m_t << "</programlisting>"; 
   }
   else
   {
-    m_t << endl;
+    if (!m_hide) m_t << endl;
   }
 }
 
@@ -234,6 +284,7 @@ void XmlDocVisitor::visit(DocFormula *f)
 
 void XmlDocVisitor::visit(DocIndexEntry *ie)
 {
+  if (m_hide) return;
   m_t << "<indexentry>"
            "<primaryie>";
   filter(ie->entry());
@@ -248,6 +299,7 @@ void XmlDocVisitor::visit(DocIndexEntry *ie)
 
 void XmlDocVisitor::visitPre(DocAutoList *l)
 {
+  if (m_hide) return;
   if (l->isEnumList())
   {
     m_t << "<orderedlist>\n";
@@ -260,6 +312,7 @@ void XmlDocVisitor::visitPre(DocAutoList *l)
 
 void XmlDocVisitor::visitPost(DocAutoList *l)
 {
+  if (m_hide) return;
   if (l->isEnumList())
   {
     m_t << "</orderedlist>\n";
@@ -272,21 +325,25 @@ void XmlDocVisitor::visitPost(DocAutoList *l)
 
 void XmlDocVisitor::visitPre(DocAutoListItem *)
 {
+  if (m_hide) return;
   m_t << "<listitem>";
 }
 
 void XmlDocVisitor::visitPost(DocAutoListItem *) 
 {
+  if (m_hide) return;
   m_t << "</listitem>";
 }
 
 void XmlDocVisitor::visitPre(DocPara *) 
 {
+  if (m_hide) return;
   m_t << "<para>";
 }
 
 void XmlDocVisitor::visitPost(DocPara *)
 {
+  if (m_hide) return;
   m_t << "</para>";
 }
 
@@ -302,7 +359,8 @@ void XmlDocVisitor::visitPost(DocRoot *)
 
 void XmlDocVisitor::visitPre(DocSimpleSect *s)
 {
-  m_t << "<simplesect kind=\">";
+  if (m_hide) return;
+  m_t << "<simplesect kind=\"";
   switch(s->type())
   {
     case DocSimpleSect::See: 
@@ -320,7 +378,7 @@ void XmlDocVisitor::visitPre(DocSimpleSect *s)
     case DocSimpleSect::Date: 
       m_t << "date"; break;
     case DocSimpleSect::Note: 
-      m_t << "bug"; break;
+      m_t << "note"; break;
     case DocSimpleSect::Warning:
       m_t << "warning"; break;
     case DocSimpleSect::Pre:
@@ -335,6 +393,8 @@ void XmlDocVisitor::visitPre(DocSimpleSect *s)
       m_t << "attention"; break;
     case DocSimpleSect::User: 
       m_t << "par"; break;
+    case DocSimpleSect::Rcs: 
+      m_t << "rcs"; break;
     case DocSimpleSect::Unknown:  break;
   }
   m_t << "\">";
@@ -342,52 +402,65 @@ void XmlDocVisitor::visitPre(DocSimpleSect *s)
 
 void XmlDocVisitor::visitPost(DocSimpleSect *)
 {
+  if (m_hide) return;
   m_t << "</simplesect>\n";
 }
 
 void XmlDocVisitor::visitPre(DocTitle *)
 {
+  if (m_hide) return;
   m_t << "<title>";
 }
 
 void XmlDocVisitor::visitPost(DocTitle *)
 {
+  if (m_hide) return;
   m_t << "</title>";
 }
 
 void XmlDocVisitor::visitPre(DocSimpleList *)
 {
+  if (m_hide) return;
   m_t << "<itemizedlist>\n";
 }
 
 void XmlDocVisitor::visitPost(DocSimpleList *)
 {
+  if (m_hide) return;
   m_t << "</itemizedlist>\n";
 }
 
 void XmlDocVisitor::visitPre(DocSimpleListItem *)
 {
+  if (m_hide) return;
   m_t << "<listitem>";
 }
 
 void XmlDocVisitor::visitPost(DocSimpleListItem *) 
 {
+  if (m_hide) return;
   m_t << "</listitem>\n";
 }
 
 void XmlDocVisitor::visitPre(DocSection *s)
 {
-  m_t << "<sect" << s->level()+1 << ">";
+  if (m_hide) return;
+  m_t << "<sect" << s->level() << " id=\"" << s->file();
+  if (!s->anchor().isEmpty()) m_t << "_1" << s->anchor();
+  m_t << "\">" << endl;
+  m_t << "<title>";
   filter(s->title());
-  m_t << "</sect" << s->level()+1 << ">\n";
+  m_t << "</title>" << endl;
 }
 
-void XmlDocVisitor::visitPost(DocSection *) 
+void XmlDocVisitor::visitPost(DocSection *s) 
 {
+  m_t << "</sect" << s->level() << ">\n";
 }
 
 void XmlDocVisitor::visitPre(DocHtmlList *s)
 {
+  if (m_hide) return;
   if (s->type()==DocHtmlList::Ordered) 
     m_t << "<orderedlist>\n"; 
   else 
@@ -396,6 +469,7 @@ void XmlDocVisitor::visitPre(DocHtmlList *s)
 
 void XmlDocVisitor::visitPost(DocHtmlList *s) 
 {
+  if (m_hide) return;
   if (s->type()==DocHtmlList::Ordered) 
     m_t << "</orderedlist>\n"; 
   else 
@@ -404,129 +478,140 @@ void XmlDocVisitor::visitPost(DocHtmlList *s)
 
 void XmlDocVisitor::visitPre(DocHtmlListItem *)
 {
+  if (m_hide) return;
   m_t << "<listitem>\n";
 }
 
 void XmlDocVisitor::visitPost(DocHtmlListItem *) 
 {
+  if (m_hide) return;
   m_t << "</listitem>\n";
 }
 
-void XmlDocVisitor::visitPre(DocHtmlPre *)
-{
-  m_t << "<preformatted>\n";
-  m_insidePre=TRUE;
-}
-
-void XmlDocVisitor::visitPost(DocHtmlPre *) 
-{
-  m_insidePre=FALSE;
-  m_t << "</preformatted>\n";
-}
-
 void XmlDocVisitor::visitPre(DocHtmlDescList *)
 {
+  if (m_hide) return;
   m_t << "<variablelist>\n";
 }
 
 void XmlDocVisitor::visitPost(DocHtmlDescList *) 
 {
+  if (m_hide) return;
   m_t << "</variablelist>\n";
 }
 
 void XmlDocVisitor::visitPre(DocHtmlDescTitle *)
 {
+  if (m_hide) return;
   m_t << "<varlistentry><term>";
 }
 
 void XmlDocVisitor::visitPost(DocHtmlDescTitle *) 
 {
+  if (m_hide) return;
   m_t << "</term></varlistentry>\n";
 }
 
 void XmlDocVisitor::visitPre(DocHtmlDescData *)
 {
+  if (m_hide) return;
   m_t << "<listitem>";
 }
 
 void XmlDocVisitor::visitPost(DocHtmlDescData *) 
 {
+  if (m_hide) return;
   m_t << "</listitem>\n";
 }
 
 void XmlDocVisitor::visitPre(DocHtmlTable *t)
 {
+  if (m_hide) return;
   m_t << "<table rows=\"" << t->numRows() 
       << "\" cols=\"" << t->numCols() << "\">" ;
 }
 
 void XmlDocVisitor::visitPost(DocHtmlTable *) 
 {
+  if (m_hide) return;
   m_t << "</table>\n";
 }
 
 void XmlDocVisitor::visitPre(DocHtmlRow *)
 {
+  if (m_hide) return;
   m_t << "<row>\n";
 }
 
 void XmlDocVisitor::visitPost(DocHtmlRow *) 
 {
+  if (m_hide) return;
   m_t << "</row>\n";
 }
 
 void XmlDocVisitor::visitPre(DocHtmlCell *c)
 {
+  if (m_hide) return;
   if (c->isHeading()) m_t << "<entry thead=\"yes\">"; else m_t << "<entry thead=\"no\">";
 }
 
 void XmlDocVisitor::visitPost(DocHtmlCell *) 
 {
+  if (m_hide) return;
   m_t << "</entry>"; 
 }
 
 void XmlDocVisitor::visitPre(DocHtmlCaption *)
 {
+  if (m_hide) return;
   m_t << "<caption>";
 }
 
 void XmlDocVisitor::visitPost(DocHtmlCaption *) 
 {
+  if (m_hide) return;
   m_t << "</caption>\n";
 }
 
 void XmlDocVisitor::visitPre(DocInternal *)
 {
+  if (m_hide) return;
   m_t << "<internal>";
 }
 
 void XmlDocVisitor::visitPost(DocInternal *) 
 {
+  if (m_hide) return;
   m_t << "</internal>" << endl;
 }
 
 void XmlDocVisitor::visitPre(DocHRef *href)
 {
+  if (m_hide) return;
   m_t << "<ulink url=\"" << href->url() << "\">";
 }
 
 void XmlDocVisitor::visitPost(DocHRef *) 
 {
+  if (m_hide) return;
   m_t << "</ulink>";
 }
 
 void XmlDocVisitor::visitPre(DocHtmlHeader *header)
 {
-  m_t << "<sect" << header->level() << ">";
+  if (m_hide) return;
+  m_t << "<heading level=\"" << header->level() << "\">";
 }
 
-void XmlDocVisitor::visitPost(DocHtmlHeader *header) 
+void XmlDocVisitor::visitPost(DocHtmlHeader *) 
 {
-  m_t << "</sect" << header->level() << ">\n";
+  if (m_hide) return;
+  m_t << "</heading>\n";
 }
 
 void XmlDocVisitor::visitPre(DocImage *img)
 {
+  if (m_hide) return;
   m_t << "<image type=\"";
   switch(img->type())
   {
@@ -536,7 +621,7 @@ void XmlDocVisitor::visitPre(DocImage *img)
   }
   m_t << "\"";
 
-  QCString baseName=img->name();
+  QString baseName=img->name();
   int i;
   if ((i=baseName.findRev('/'))!=-1 || (i=baseName.findRev('\\'))!=-1)
   {
@@ -560,73 +645,87 @@ void XmlDocVisitor::visitPre(DocImage *img)
 
 void XmlDocVisitor::visitPost(DocImage *) 
 {
+  if (m_hide) return;
   m_t << "</image>" << endl;
 }
 
 void XmlDocVisitor::visitPre(DocDotFile *df)
 {
+  if (m_hide) return;
   m_t << "<dotfile name=\"" << df->file() << "\">";
 }
 
 void XmlDocVisitor::visitPost(DocDotFile *) 
 {
+  if (m_hide) return;
   m_t << "</dotfile>" << endl;
 }
 
 void XmlDocVisitor::visitPre(DocLink *lnk)
 {
+  if (m_hide) return;
   startLink(lnk->ref(),lnk->file(),lnk->anchor());
 }
 
 void XmlDocVisitor::visitPost(DocLink *) 
 {
+  if (m_hide) return;
   endLink();
 }
 
 void XmlDocVisitor::visitPre(DocRef *ref)
 {
-  startLink(ref->ref(),ref->file(),ref->anchor());
+  if (m_hide) return;
+   if (!ref->file().isEmpty()) startLink(ref->ref(),ref->file(),ref->anchor());
   if (!ref->hasLinkText()) filter(ref->targetTitle());
 }
 
-void XmlDocVisitor::visitPost(DocRef *) 
+void XmlDocVisitor::visitPost(DocRef *ref) 
 {
-  endLink();
+  if (m_hide) return;
+  if (!ref->file().isEmpty()) endLink();
   m_t << " ";
 }
 
 void XmlDocVisitor::visitPre(DocSecRefItem *ref)
 {
+  if (m_hide) return;
   m_t << "<tocitem id=\"" << ref->file() << "_1" << ref->anchor() << "\">";
 }
 
 void XmlDocVisitor::visitPost(DocSecRefItem *) 
 {
+  if (m_hide) return;
   m_t << "</tocitem>" << endl;
 }
 
 void XmlDocVisitor::visitPre(DocSecRefList *)
 {
+  if (m_hide) return;
   m_t << "<toclist>" << endl;
 }
 
 void XmlDocVisitor::visitPost(DocSecRefList *) 
 {
+  if (m_hide) return;
   m_t << "</toclist>" << endl;
 }
 
 void XmlDocVisitor::visitPre(DocLanguage *l)
 {
-  m_t << "<language id=\"" << l->id() << "\">";
+  if (m_hide) return;
+  m_t << "<language langid=\"" << l->id() << "\">";
 }
 
 void XmlDocVisitor::visitPost(DocLanguage *) 
 {
+  if (m_hide) return;
   m_t << "</language>" << endl;
 }
 
 void XmlDocVisitor::visitPre(DocParamSect *s)
 {
+  if (m_hide) return;
   m_t << "<parameterlist kind=\"";
   switch(s->type())
   {
@@ -644,16 +743,36 @@ void XmlDocVisitor::visitPre(DocParamSect *s)
 
 void XmlDocVisitor::visitPost(DocParamSect *)
 {
+  if (m_hide) return;
   m_t << "</parameterlist>" << endl;
 }
 
 void XmlDocVisitor::visitPre(DocParamList *pl)
 {
+  if (m_hide) return;
   QStrListIterator li(pl->parameters());
   const char *s;
   for (li.toFirst();(s=li.current());++li)
   {
-    m_t << "<parametername>";
+    m_t << "<parametername";
+    if (pl->direction()!=DocParamSect::Unspecified)
+    {
+      m_t << " direction=\"";
+      if (pl->direction()==DocParamSect::In)
+      {
+        m_t << "in";
+      }
+      else if (pl->direction()==DocParamSect::Out)
+      {
+        m_t << "out";
+      }
+      else if (pl->direction()==DocParamSect::InOut)
+      {
+        m_t << "inout";
+      }
+      m_t << "\"";
+    }
+    m_t << ">";
     filter(s);
     m_t << "</parametername>" << endl;
   }
@@ -662,11 +781,13 @@ void XmlDocVisitor::visitPre(DocParamList *pl)
 
 void XmlDocVisitor::visitPost(DocParamList *)
 {
+  if (m_hide) return;
   m_t << "</parameterdescription>";
 }
 
 void XmlDocVisitor::visitPre(DocXRefItem *x)
 {
+  if (m_hide) return;
   m_t << "<xrefsect id=\"";
   m_t << x->file() << "_1" << x->anchor();
   m_t << "\">";
@@ -678,48 +799,50 @@ void XmlDocVisitor::visitPre(DocXRefItem *x)
 
 void XmlDocVisitor::visitPost(DocXRefItem *)
 {
+  if (m_hide) return;
   m_t << "</xrefdescription>";
   m_t << "</xrefsect>";
 }
 
 void XmlDocVisitor::visitPre(DocInternalRef *ref)
 {
+  if (m_hide) return;
   startLink(0,ref->file(),ref->anchor());
 }
 
 void XmlDocVisitor::visitPost(DocInternalRef *) 
 {
+  if (m_hide) return;
   endLink();
   m_t << " ";
 }
 
-void XmlDocVisitor::visitPre(DocCopy *)
+void XmlDocVisitor::visitPre(DocCopy *c)
 {
+  if (m_hide) return;
+  m_t << "<copydoc link=\"" << c->link() << "\">";
 }
 
 void XmlDocVisitor::visitPost(DocCopy *)
+{
+  if (m_hide) return;
+  m_t << "</copydoc>" << endl;
+}
+
+void XmlDocVisitor::visitPre(DocText *)
+{
+}
+
+void XmlDocVisitor::visitPost(DocText *)
 {
 }
 
 void XmlDocVisitor::filter(const char *str)
 { 
-  if (str==0) return;
-  const char *p=str;
-  char c;
-  while (*p)
-  {
-    c=*p++;
-    switch(c)
-    {
-      case '<':  m_t << "&lt;"; break;
-      case '>':  m_t << "&gt;"; break;
-      case '&':  m_t << "&amp;"; break;
-      default:   m_t << c;
-    }
-  }
+  m_t << convertToXML(str);
 }
 
-void XmlDocVisitor::startLink(const QCString &ref,const QCString &file,const QCString &anchor)
+void XmlDocVisitor::startLink(const QString &ref,const QString &file,const QString &anchor)
 {
   m_t << "<ref refid=\"" << file;
   if (!anchor.isEmpty()) m_t << "_1" << anchor;
@@ -735,3 +858,16 @@ void XmlDocVisitor::endLink()
   m_t << "</ref>";
 }
 
+void XmlDocVisitor::pushEnabled()
+{
+  m_enabled.push(new bool(m_hide));
+}
+
+void XmlDocVisitor::popEnabled()
+{
+  bool *v=m_enabled.pop();
+  ASSERT(v!=0);
+  m_hide = *v;
+  delete v;
+}
+
